﻿using System;
using System.Drawing;
using System.IO;
using Clash_Server;
using System.Reflection;

namespace Clash_Library
{
    public enum Class : byte { Warrior, Wizard, Taoist, Assassin }
    public enum Direction : byte { Up, Up_Right, Right, Down_Right, Down, Down_Left, Left, Up_Left }
    public enum Gender : byte { Male, Female }
    public enum ItemLocation : byte
    {
        Weapon,
        Armour,
        Helmet,
        Necklace,
        Candle,
        BraceletR,
        BraceletL,
        RingR,
        RingL,
        Poison,
        Amulet,
        Belt,
        Boots,
        Stone,
        Medal
    }
    public enum MirAction : byte
    {
        Standing,
        Walking,
        Running,
        BattleStance,
        Attack1,
        Attack2,
        Attack3,
        Spell,
        Struck,
        Harvest,
        Dieing,
        Dead,
        Show,
        Hide//, HorseStanding, HorseWalk, HorseRun, HorseStruck;
    }
    [Flags]
    public enum Element : byte
    {
        None = 0,
        Fire = 1,
        Lightning = 2,
        Ice = 4,
        Wind = 8,
        Dark = 16,
        Holy = 32,
        Illusion = 64
    }

    public interface Packet
    {
        void Read(BinaryReader BReader);
        void WriteTo(BinaryWriter BWriter);
    }

    public struct CharSelInfo : Packet
    {
        public int PlayerID;
        public string Name;
        public int Level;
        public Class Job;
        public Gender Sex;
        public CharSelInfo(int ID, string N, int L, Class C, Gender G)
        {
            PlayerID = ID;
            Name = N;
            Level = L;
            Job = C;
            Sex = G;
        }
        public void Read(BinaryReader BReader)
        {
            PlayerID = BReader.ReadInt32();
            Name = BReader.ReadString();
            Level = BReader.ReadInt32();
            Job = (Class)BReader.ReadByte();
            Sex = (Gender)BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(PlayerID);
            BWriter.Write(Name);
            BWriter.Write(Level);
            BWriter.Write((byte)Job);
            BWriter.Write((byte)Sex);
        }
    }
    public class ActorInfo : Packet
    {
        public CMapInfo MapInfo;
        public Point Location;
        public Item[] BagItems;
        public Item[] Equipment;
        public long Exp, MaxExp;
        public Direction Dir;
        public bool Dead, Dev;
        public byte Hair;
        public int CurrentHP, CurrentMP;
        public AllStats BaseStats;
        #region Don't Send fields
        public AllStats TotalStats;
        public int PlayerID;
        public string Name;
        public int Level;
        public Class Job;
        public Gender Sex;
        #endregion
        public ActorInfo()
        { }
        public ActorInfo(Player P)
        {
            Location = P.Location;
            BagItems = P.BagItems;
            Equipment = P.Equipment;
            Dir = P.Dir;
            Dead = P.Dead;
            Dev = P.Dev;
            Hair = P.Hair;
            BaseStats = P.BaseStats;
            Exp = P.Exp;
            MaxExp = P.MaxEXP;
            CurrentHP = P.CurrentHP;
            CurrentMP = P.CurrentMP;
            MapInfo = new CMapInfo(P.Map.MInfo);
        }

        public void MakeTotalStats()
        {
            TotalStats = new AllStats();
            TotalStats += BaseStats;
            foreach (Item E in Equipment)
                if (E != null)
                    TotalStats += E.TotalStats;
        }
        private void UpdateItems()
        {
            foreach (Item I in BagItems)
            {
                if (I == null) continue;
                if (I.Index >= 0 && I.Index < Item.IList.Count && I.Index == Item.IList[I.Index].Index)
                {
                    I.IInfo = Item.IList[I.Index];
                    I.UpdateStats();
                }
                else
                    for (int X = 0; X < Item.IList.Count; X++)
                        if (I.Index == Item.IList[X].Index)
                        {
                            I.IInfo = Item.IList[X];
                            I.UpdateStats();
                            break;
                        }
            }
            foreach (Item I in Equipment)
            {
                if (I == null) continue;
                if (I.Index >= 0 && I.Index < Item.IList.Count && I.Index == Item.IList[I.Index].Index)
                {
                    I.IInfo = Item.IList[I.Index];
                    I.UpdateStats();
                }
                else
                    for (int X = 0; X < Item.IList.Count; X++)
                        if (I.Index == Item.IList[X].Index)
                        {
                            I.IInfo = Item.IList[X];
                            I.UpdateStats();
                            break;
                        }
            }
        }

        public void Read(BinaryReader BReader)
        {
            MapInfo = new CMapInfo();
            MapInfo.Read(BReader);
            Location = new Point(BReader.ReadInt16(), BReader.ReadInt16());

            BagItems = new Item[BReader.ReadInt32()];
            for (int I = 0; I < BagItems.Length; I++)
                if (BReader.ReadBoolean())
                {
                    BagItems[I] = new Item();
                    BagItems[I].Read(BReader);
                }

            Equipment = new Item[BReader.ReadInt32()];
            for (int I = 0; I < Equipment.Length; I++)
                if (BReader.ReadBoolean())
                {
                    Equipment[I] = new Item();
                    Equipment[I].Read(BReader);
                }

            Exp = BReader.ReadInt64();
            MaxExp = BReader.ReadInt64();
            Dir = (Direction)BReader.ReadByte();
            Dead = BReader.ReadBoolean();
            Dev = BReader.ReadBoolean();
            Hair = BReader.ReadByte();
            CurrentHP = BReader.ReadInt32();
            CurrentMP = BReader.ReadInt32();

            BaseStats = new AllStats();
            BaseStats.Read(BReader);

            UpdateItems();
            MakeTotalStats();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            MapInfo.WriteTo(BWriter);
            BWriter.Write((short)Location.X);
            BWriter.Write((short)Location.Y);

            BWriter.Write(BagItems.Length);
            foreach (Item I in BagItems)
            {
                if (I == null) BWriter.Write(false);
                else
                {
                    BWriter.Write(true);
                    I.WriteTo(BWriter);
                }
            }

            BWriter.Write(Equipment.Length);
            foreach (Item I in Equipment)
            {
                if (I == null) BWriter.Write(false);
                else
                {
                    BWriter.Write(true);
                    I.WriteTo(BWriter);
                }
            }

            BWriter.Write(Exp);
            BWriter.Write(MaxExp);
            BWriter.Write((byte)Dir);
            BWriter.Write(Dead);
            BWriter.Write(Dev);
            BWriter.Write(Hair);
            BWriter.Write(CurrentHP);
            BWriter.Write(CurrentMP);
            BaseStats.WriteTo(BWriter);

        }
    }

    public class CMapInfo : Packet
    {
        public string FileName;
        public string MapDescription;
        public MapType Type;
        public MapFlags Flags;
        public CMapInfo()
        { }
        public CMapInfo(MapInfo M)
        {
            FileName = M.FileName;
            MapDescription = M.MapDescription;
            Type = M.MapType;
            Flags = M.Flags;
        }
        public void Read(BinaryReader BReader)
        {
            FileName = BReader.ReadString();
            MapDescription = BReader.ReadString();
            Type = (MapType)BReader.ReadByte();
            Flags = (MapFlags)BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(FileName);
            BWriter.Write(MapDescription);
            BWriter.Write((byte)Type);
            BWriter.Write((byte)Flags);
        }
    }
    [Serializable]
    public class ImageInfo
    {
        public long Start;
        public int Length;
        public Point OffSet;
        public Size ImageSize;
    }
    public class PlayerStats : Packet
    {
        public int UniqueID;
        public string Name;
        public byte Hair, Armor, Weapon;
        public Gender Sex;
        public Class Job;
        public Direction Dir;
        public Point Location;
        public Color NameColour;
        public int CurrentHP, MaxHP;
        public int CurrentMP, MaxMP;
        public sbyte Light;
        public bool Dev, Dead;
        public PlayerStats()
        { }
        public PlayerStats(Player P)
        {
            UniqueID = P.Index;
            Name = P.CI.CharacterName;
            Hair = P.Hair;
            if (P.Equipment[(byte)ItemLocation.Armour] != null)
                Armor = P.Equipment[(byte)ItemLocation.Armour].IInfo.Shape;
            else Armor = 0;
            if (P.Equipment[(byte)ItemLocation.Weapon] != null)
                Weapon = P.Equipment[(byte)ItemLocation.Weapon].IInfo.Shape;
            else Weapon = 0;
            Sex = P.Gender;
            Job = P.Class;
            Dir = P.Dir;
            Location = P.Location;
            NameColour = Color.White;
            CurrentHP = P.CurrentHP;
            CurrentMP = P.CurrentMP;
            MaxHP = P.TotalStats.Basic.HP;
            MaxMP = P.TotalStats.Basic.MP;
            Dev = P.Dev;
            Dead = P.Dead;
            Light = P.TotalStats.Advanced.Light;
        }

        public void Read(BinaryReader BReader)
        {
            UniqueID = BReader.ReadInt32();
            Name = BReader.ReadString();
            Hair = BReader.ReadByte();
            Armor = BReader.ReadByte();
            Weapon = BReader.ReadByte();
            Sex = (Gender)BReader.ReadByte();
            Job = (Class)BReader.ReadByte();
            Dir = (Direction)BReader.ReadByte();
            Location = new Point(BReader.ReadInt16(), BReader.ReadInt16());
            NameColour = Color.FromArgb(BReader.ReadInt32());
            CurrentHP = BReader.ReadInt32();
            MaxHP = BReader.ReadInt32();
            CurrentMP = BReader.ReadInt32();
            MaxMP = BReader.ReadInt32();
            Light = BReader.ReadSByte();
            Dev = BReader.ReadBoolean();
            Dead = BReader.ReadBoolean();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(UniqueID);
            BWriter.Write(Name);
            BWriter.Write(Hair);
            BWriter.Write(Armor);
            BWriter.Write(Weapon);
            BWriter.Write((byte)Sex);
            BWriter.Write((byte)Job);
            BWriter.Write((byte)Dir);
            BWriter.Write((short)Location.X);
            BWriter.Write((short)Location.Y);
            BWriter.Write(NameColour.ToArgb());
            BWriter.Write(CurrentHP);
            BWriter.Write(MaxHP);
            BWriter.Write(CurrentMP);
            BWriter.Write(MaxMP);
            BWriter.Write(Light);
            BWriter.Write(Dev);
            BWriter.Write(Dead);

        }
    }

    public class MonsterStats : Packet
    {
        public int MobID;
        public string Name;
        public Color NameColour;
        public Direction Dir;
        public Point Location;
        public int CurrentHP, CurrentMP, MaxHP, MaxMP;
        public bool Dead;
        public int Image;
        public sbyte Light;

        public MonsterStats()
        { }
        public MonsterStats(Monster M)
        {
            MobID = M.MobID;
            Name = M.Info.Behaviour.HasFlag(MonsterBehaviour.HideName) ? "" : M.Info.Name;
            NameColour = M.Info.NameColour;
            Dir = M.Dir;
            Location = M.Location;
            CurrentHP = M.CurrentHP;
            CurrentMP = M.CurrentMP;
            Dead = M.Dead;
            Image = M.Info.Image;
            Light = M.TotalStats.Advanced.Light;
            MaxHP = M.TotalStats.Basic.HP;
            MaxMP = M.TotalStats.Basic.MP;
        }
        public void Read(System.IO.BinaryReader BReader)
        {
            MobID = BReader.ReadInt32();
            Name = BReader.ReadString();
            NameColour = Color.FromArgb(BReader.ReadInt32());
            Dir = (Direction)BReader.ReadByte();
            Location = new Point(BReader.ReadInt16(), BReader.ReadInt16());
            CurrentHP = BReader.ReadInt32();
            CurrentMP = BReader.ReadInt32();
            MaxHP = BReader.ReadInt32();
            MaxMP = BReader.ReadInt32();
            Dead = BReader.ReadBoolean();
            Image = BReader.ReadInt32();
            Light = BReader.ReadSByte();
        }
        public void WriteTo(System.IO.BinaryWriter BWriter)
        {
            BWriter.Write(MobID);
            BWriter.Write(Name);
            BWriter.Write(NameColour.ToArgb());
            BWriter.Write((byte)Dir);
            BWriter.Write((short)Location.X);
            BWriter.Write((short)Location.Y);
            BWriter.Write(CurrentHP);
            BWriter.Write(CurrentMP);
            BWriter.Write(MaxHP);
            BWriter.Write(MaxMP);
            BWriter.Write(Dead);
            BWriter.Write(Image);
            BWriter.Write(Light);
        }
    }

}
