﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;

namespace Clash_Library
{
    public class IniReader
    {
        private string FileName;
        private List<string> Lines;
        public bool DontSave { get; set; }

        public IniReader(string path)
        {
            try
            {
                FileName = path;
                Lines = new List<string>();
                if (File.Exists(FileName))
                    Lines.AddRange(File.ReadAllLines(FileName));
            }
            catch
            {
            }
        }
        public void Save()
        {
            if (DontSave) return;
            try
            {
                File.WriteAllLines(FileName, Lines);
            }
            catch
            {
            }
        }
        #region Read
        public bool ReadBoolean(string Section, string Key, bool Default)
        {
            bool Result;
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            if (bool.TryParse(Lines[L].Split('=')[1], out Result))
                                return Result;
                            else
                                return Default;
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public byte ReadByte(string Section, string Key, byte Default)
        {
            byte Result;
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            if (byte.TryParse(Lines[L].Split('=')[1], out Result))
                                return Result;
                            else
                                return Default;
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public sbyte ReadSByte(string Section, string Key, sbyte Default)
        {
            sbyte Result;
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            if (sbyte.TryParse(Lines[L].Split('=')[1], out Result))
                                return Result;
                            else
                                return Default;
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public ushort ReadUInt16(string Section, string Key, ushort Default)
        {
            ushort Result;
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            if (ushort.TryParse(Lines[L].Split('=')[1], out Result))
                                return Result;
                            else
                                return Default;
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public short ReadInt16(string Section, string Key, short Default)
        {
            short Result;
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            if (short.TryParse(Lines[L].Split('=')[1], out Result))
                                return Result;
                            else
                                return Default;
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public uint ReadUInt32(string Section, string Key, uint Default)
        {
            uint Result;
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            if (uint.TryParse(Lines[L].Split('=')[1], out Result))
                                return Result;
                            else
                                return Default;
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public int ReadInt32(string Section, string Key, int Default)
        {
            int Result;
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            if (int.TryParse(Lines[L].Split('=')[1], out Result))
                                return Result;
                            else
                                return Default;
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public ulong ReadUInt64(string Section, string Key, ulong Default)
        {
            ulong Result;
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            if (ulong.TryParse(Lines[L].Split('=')[1], out Result))
                                return Result;
                            else
                                return Default;
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public long ReadInt64(string Section, string Key, long Default)
        {
            long Result;
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            if (long.TryParse(Lines[L].Split('=')[1], out Result))
                                return Result;
                            else
                                return Default;
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public float ReadSingle(string Section, string Key, float Default)
        {
            float Result;
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            if (float.TryParse(Lines[L].Split('=')[1], out Result))
                                return Result;
                            else
                                return Default;
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public double ReadDouble(string Section, string Key, double Default)
        {
            double Result;
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            if (double.TryParse(Lines[L].Split('=')[1], out Result))
                                return Result;
                            else
                                return Default;
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public decimal ReadDecimal(string Section, string Key, decimal Default)
        {
            decimal Result;
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            if (decimal.TryParse(Lines[L].Split('=')[1], out Result))
                                return Result;
                            else
                                return Default;
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public string ReadString(string Section, string Key, string Default)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            return Lines[L].Split('=')[1];
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public char ReadChar(string Section, string Key, char Default)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            return Lines[L].Split('=')[1][0];
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public TimeSpan ReadTimeSpan(string Section, string Key, TimeSpan Default)
        {
            TimeSpan Result;
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            if (TimeSpan.TryParse(Lines[L].Split('=')[1], out Result))
                                return Result;
                            else
                                return Default;
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public DateTime ReadDateTime(string Section, string Key, DateTime Default)
        {
            DateTime Result;
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            if (DateTime.TryParse(Lines[L].Split('=')[1], out Result))
                                return Result;
                            else
                                return Default;
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public Point ReadPoint(string Section, string Key, Point Default)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)                            
                            return (Point)new PointConverter().ConvertFromString(Lines[L].Split('=')[1]);
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        public Size ReadSize(string Section, string Key, Size Default)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                            return (Size)new SizeConverter().ConvertFromString(Lines[L].Split('=')[1]);
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                            return Default;
            return Default;
        }
        #endregion
        #region Write

        public void Write(string Section, string Key, bool Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }
                        else if (L == Lines.Count - 1)
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }


            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, byte Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, sbyte Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, ushort Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, short Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, uint Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, int Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, ulong Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, long Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, float Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, double Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, decimal Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, string Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value;
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value);
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value);
            Save();
        }
        public void Write(string Section, string Key, char Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, TimeSpan Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, DateTime Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + Value.ToString();
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + Value.ToString());
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + Value.ToString());
            Save();
        }
        public void Write(string Section, string Key, Point Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + new PointConverter().ConvertToString(Value);
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + new PointConverter().ConvertToString(Value));
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + new PointConverter().ConvertToString(Value));
            Save();
        }
        public void Write(string Section, string Key, Size Value)
        {
            for (int I = 0; I < Lines.Count; I++)
                if (string.Compare(Lines[I], "[" + Section + "]") == 0)
                    for (int L = I + 1; L < Lines.Count; L++)
                        if (string.Compare(Lines[L].Split('=')[0], Key) == 0)
                        {
                            Lines[L] = Key + "=" + new SizeConverter().ConvertToString(Value);
                            Save();
                            return;
                        }
                        else if (Lines[L].StartsWith("[") && Lines[L].EndsWith("]"))
                        {
                            Lines.Insert(L, Key + "=" + new SizeConverter().ConvertToString(Value));
                            Save();
                            return;
                        }

            Lines.Add("[" + Section + "]");
            Lines.Add(Key + "=" + new SizeConverter().ConvertToString(Value));
            Save();
        }
        #endregion

    }
}
