﻿using System;
using System.Collections.Generic;
using System.IO;

namespace Clash_Library
{
    public enum Rarity { Junk, Common, Uncommon, Rare, Legendary, Mythic, Unique }
    public enum Requirement { Level, DC, MC, SC }
    [Flags]
    public enum SpecialEffect
    {
        Invisible = 1,
        Teleport = 2,
        Revival = 4,
        Paralysis = 8,
        Muscle = 16,
        Protection = 32,
        Probe = 64,
        Indestructable = 128,
        Leach = 256,
        Mystery = 512
    }
    [Flags]
    public enum ReqClass
    {
        Warrior = 1,
        Wizard = 2,
        Taoist = 4,
        Assassin = 8
    }
    [Flags]
    public enum ReqGender
    {
        Male = 1,
        Female = 2
    }
    public enum ItemType : byte
    {
        Nothing,
        Potion,
        Scroll,
        Book,
        Candle,
        Weapon,
        Armor,
        Helmet,
        Necklace,
        Bracelet,
        Ring,
        Amulet,
        Belt,
        Boots,
        Stone,
        Junk,
        Gems,
        Meat,
        Ore,
        Poison,
        Medal
    }


    [Serializable]
    public class Item : Packet
    {
        public static List<ItemInfo> IList = new List<ItemInfo>();
        public int UniqueID;
        public int Index;
        public string CustomName;

        //Stats
        public long CurrentDura, MaxDura;
        public uint Amount;
        public AllStats AddedStats;

        //Level / Refine Information
        public int Level;
        public long Experience;
        public int RefinesLeft;

        //Generated Information
        [NonSerialized]
        public AllStats TotalStats;
        [NonSerialized]
        public ItemInfo IInfo;
        public Item()
        {
        }
        public Item(ItemInfo II, int UID)
        {
            UniqueID = UID;
            IInfo = II;
            Index = IInfo.Index;
            MaxDura = IInfo.Durability;
            CurrentDura = MaxDura;
            AddedStats = new AllStats();
            Level = 0;
            Experience = 0;
            RefinesLeft = IInfo.DefaultRefineCount;
            IInfo.ExistCount++;
            UpdateStats();
        }
        public void UpdateStats()
        {
            TotalStats = new AllStats();
            TotalStats += IInfo.Stats;
            TotalStats += AddedStats;
        }
        public bool CorrectSlot(ItemLocation Loc)
        {
            switch (IInfo.Type)
            {
                case ItemType.Armor:
                    return Loc == ItemLocation.Armour;
                case ItemType.Weapon:
                    return Loc == ItemLocation.Weapon;
                case ItemType.Helmet:
                    return Loc == ItemLocation.Helmet;
                case ItemType.Necklace:
                    return Loc == ItemLocation.Necklace;
                case ItemType.Bracelet:
                    return Loc == ItemLocation.BraceletL || Loc == ItemLocation.BraceletR;
                case ItemType.Ring:
                    return Loc == ItemLocation.RingL || Loc == ItemLocation.RingR;
                case ItemType.Amulet:
                    return Loc == ItemLocation.Amulet;
                case ItemType.Poison:
                    return Loc == ItemLocation.Poison;
                case ItemType.Belt:
                    return Loc == ItemLocation.Belt;
                case ItemType.Boots:
                    return Loc == ItemLocation.Boots;
                case ItemType.Candle:
                    return Loc == ItemLocation.Candle;
                case ItemType.Stone:
                    return Loc == ItemLocation.Stone;
                case ItemType.Medal:
                    return Loc == ItemLocation.Medal;
            }

            return false;
        }

        public void Read(BinaryReader BReader)
        {
            UniqueID = BReader.ReadInt32();
            Index = BReader.ReadInt32();
            CustomName = BReader.ReadString();

            CurrentDura = BReader.ReadInt64();
            MaxDura = BReader.ReadInt64();
            Amount = BReader.ReadUInt32();

            AddedStats = new AllStats();
            AddedStats.Read(BReader);

            Level = BReader.ReadInt32();
            Experience = BReader.ReadInt64();
            RefinesLeft = BReader.ReadInt32();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(UniqueID);
            BWriter.Write(Index);
            if (CustomName != null)
                BWriter.Write(CustomName);
            else
                BWriter.Write("");
            BWriter.Write(CurrentDura);
            BWriter.Write(MaxDura);
            BWriter.Write(Amount);
            AddedStats.WriteTo(BWriter);
            BWriter.Write(Level);
            BWriter.Write(Experience);
            BWriter.Write(RefinesLeft);

        }
    }
        

    [Serializable]
    public class ItemInfo : Packet
    {
        public int Index;
        public string Name = "";
        public int Image, BodyImage, FloorImage;
        public byte Shape;
        public ItemType Type = ItemType.Nothing;

        //Rarity
        public Rarity Rarity = Rarity.Unique; //
        public int ExistCount = 0;//

        //Stats
        public long Durability, Price;
        public byte Weight;
        public SpecialEffect Effect = 0;
        public AllStats Stats = new AllStats(); //
        public ItemBehaviour Behaviour;
        public sbyte HandWeight, BagWeight, BodyWeight; //

        //Requirement 
        public Requirement ReqType = Requirement.Level;
        public int ReqAmount;
        public ReqClass RequiredClass = 0;
        public ReqGender RequiredGender = 0;

        //Level / Refine Information
        public uint MaxLevel;
        public List<long> ExpList = new List<long>();
        public int DefaultRefineCount;

        public override string ToString()
        {
            return Index.ToString() + " - " + Name;
        }
        public void Read(BinaryReader BReader)
        {
            Index = BReader.ReadInt32();
            Name = BReader.ReadString();
            Image = BReader.ReadInt32();
            BodyImage = BReader.ReadInt32();
            FloorImage = BReader.ReadInt32();
            Shape = BReader.ReadByte();
            Type = (ItemType)BReader.ReadByte();
            Rarity = (Rarity)BReader.ReadByte();
            ExistCount = BReader.ReadInt32();
            Durability = BReader.ReadInt64();
            Price = BReader.ReadInt64();
            Weight = BReader.ReadByte();
            Effect = (SpecialEffect)BReader.ReadInt32();
            Stats = new AllStats();
            Stats.Read(BReader);
            Behaviour = (ItemBehaviour)BReader.ReadInt32();
            HandWeight = BReader.ReadSByte();
            BagWeight = BReader.ReadSByte();
            BodyWeight = BReader.ReadSByte();
            ReqType = (Requirement)BReader.ReadByte();
            ReqAmount = BReader.ReadInt32();
            RequiredClass = (ReqClass)BReader.ReadByte();
            RequiredGender = (ReqGender)BReader.ReadByte();
            MaxLevel = BReader.ReadUInt32();
            ExpList = new List<long>();
            for (int i = 0; i < MaxLevel; i++)
                ExpList.Add(BReader.ReadInt64());
            DefaultRefineCount = BReader.ReadInt32();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(Index);
            BWriter.Write(Name);
            BWriter.Write(Image);
            BWriter.Write(BodyImage);
            BWriter.Write(FloorImage);
            BWriter.Write(Shape);
            BWriter.Write((byte)Type);
            BWriter.Write((byte)Rarity);
            BWriter.Write(ExistCount);
            BWriter.Write(Durability);
            BWriter.Write(Price);
            BWriter.Write(Weight);
            BWriter.Write((int)Effect);

            Stats.WriteTo(BWriter);
            BWriter.Write((int)Behaviour);

            BWriter.Write(HandWeight);
            BWriter.Write(BagWeight);
            BWriter.Write(BodyWeight);
            BWriter.Write((byte)ReqType);
            BWriter.Write(ReqAmount);
            BWriter.Write((byte)RequiredClass);
            BWriter.Write((byte)RequiredGender);
            BWriter.Write(MaxLevel);
            foreach (long L in ExpList)
                BWriter.Write(L);
            BWriter.Write(DefaultRefineCount);
        }
    }
    [Serializable]
    public class ItemRestrictions : Packet
    {
        #region Fields
        public bool CanTrade = true, CanDrop = true, CanStore = true, CanDDrop = true, CanBreak = false, CanSell = true;
        public bool CanRepair = true, CanSRepair = true, Stuck = false, CanGem = true, CanOrb = true, CanStack = false;
        #endregion

        public void Read(BinaryReader BReader)
        {
            CanTrade = BReader.ReadBoolean();
            CanDrop = BReader.ReadBoolean();
            CanStore = BReader.ReadBoolean();
            CanDDrop = BReader.ReadBoolean();
            CanBreak = BReader.ReadBoolean();
            CanSell = BReader.ReadBoolean();
            CanRepair = BReader.ReadBoolean();
            CanSRepair = BReader.ReadBoolean();
            Stuck = BReader.ReadBoolean();
            CanGem = BReader.ReadBoolean();
            CanOrb = BReader.ReadBoolean();
            CanStack = BReader.ReadBoolean();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(CanTrade);
            BWriter.Write(CanDrop);
            BWriter.Write(CanStore);
            BWriter.Write(CanDDrop);
            BWriter.Write(CanBreak);
            BWriter.Write(CanSell);
            BWriter.Write(CanRepair);
            BWriter.Write(CanSRepair);
            BWriter.Write(Stuck);
            BWriter.Write(CanGem);
            BWriter.Write(CanOrb);
            BWriter.Write(CanStack);
        }
    }
}
