﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;

namespace Clash_Library
{
    public enum CellAttribute { Walk, HighWall, LowWall }
    public enum MapType { Mir2, Mir3 }
    [Flags]
    public enum MapFlags
    {
        Normal = 1,
        Light = 2,
        Dark = 4
    }
    public enum MonsterAI
    {
        Oma,
        OmaFighter,
        OmaWarrior,
        OmaWarlord
    }
    public enum MonsterBehaviour : short
    {
        Undead = 1,
        CanMove = 2,
        Tameable = 4,
        Moveable = 8,
        Turnable = 16,
        Passive = 32,
        HideName = 64,
        CanAttack = 128
    }
    public enum ItemBehaviour : short
    {
        Drop = 1,
        DeathDrop = 2,
        Repair = 4,
        SpecialRepair = 8,
        Sell = 16,
        Store = 32,
        Stack = 64,
        Stuck = 128,
        BreakOnDeath = 256
    }

    [Serializable]
    public class ServerData
    {
        public List<MonsterInfo> MonsterList = new List<MonsterInfo>();
        public List<ItemInfo> ItemList = new List<ItemInfo>();
        public List<MapInfo> MapList = new List<MapInfo>();
    }

    [Serializable]
    public class MapInfo
    {
        public int Index = 0;
        public string FileName = "", MapDescription = "";
        public MapType MapType = MapType.Mir2;
        public MapFlags Flags =  0;
        public List<MovementInfo> MoveList = new List<MovementInfo>();
        public List<RespawnInfo> RespawnList = new List<RespawnInfo>();
        public override string ToString()
        {
            return "[" + Index.ToString() + "] - " + MapDescription;
        }
    }
    

    [Serializable]
    public class MovementInfo
    {
        public MapInfo DestMapInfo = null;
        public Point SourcePoint, DestinationPoint;
        public override string ToString()
        {
            if (DestMapInfo != null)
                return SourcePoint.ToString() + "\t-> " + DestinationPoint.ToString() + "\t " + DestMapInfo.ToString();
            else
                return SourcePoint.ToString() + "\t-> " + DestinationPoint.ToString() + "\t <No Map Selected>";
        }
    }

    [Serializable]
    public class RespawnInfo
    {
        public MonsterInfo Monster;
        public int Spread, Amount;
        public Point Location;
        public TimeSpan RespawnTime;
        public int DistortionTime;        
        public override string ToString()
        {
            if (Monster != null)
                return Location.ToString() + "\t" + RespawnTime.ToString() + "\t\t" + Amount.ToString() + "x\t " + Monster.ToString();
            else
                return Location.ToString() + "\t" + RespawnTime.ToString() + "\t\t" + Amount.ToString() + "x\t";

        }
    }

    [Serializable]
    public class MonsterInfo
    {
        public int Index;
        public string Name = "";

        public MonsterAI AI;
        public int Image, SpellImage, Level;
        public long Experience;
        public long Gold;
        public double GoldRate;
        public byte CoolEye;

        public Color NameColour = Color.White;
        public TimeSpan MoveSpeed, AttackSpeed;
        public AllStats Stats = new AllStats();
        public List<DropInfo> DropList = new List<DropInfo>();
        public MonsterBehaviour Behaviour = new MonsterBehaviour();

        public MonsterInfo DeepClone()
        {
            MonsterInfo MI = new MonsterInfo();
            MI.Index = Index;
            MI.Name = Name;
            MI.AI = AI;
            MI.Image = Image;
            MI.SpellImage = SpellImage;
            MI.Level = Level;
            MI.Experience = Experience;
            MI.Gold = Gold;
            MI.GoldRate = GoldRate;
            MI.CoolEye = CoolEye;
            MI.NameColour = NameColour;
            MI.MoveSpeed = MoveSpeed;
            MI.AttackSpeed = AttackSpeed;
            Stats.Clone();

            MI.DropList = new List<DropInfo>();

            for (int I = 0; I < DropList.Count; I++)
                MI.DropList.Add(DropList[I].Clone());

            MI.Behaviour = Behaviour;

            return MI;
        }


        public override string ToString()
        {
            if (!string.IsNullOrEmpty(Name))
                return "[" + Index.ToString() + "] - " + Name.ToString();
            return "[" + Index.ToString() + "] - ";
        }
    }

    [Serializable]
    public class DropInfo
    {
        public ItemInfo Item;
        public ushort Amount;
        public double Chance;
        public bool Enabled;

        public DropInfo Clone()
        {
            DropInfo DI = new DropInfo();
            DI.Item = Item;
            DI.Amount = Amount;
            DI.Chance = Chance;
            DI.Enabled = Enabled;
            return DI;
        }

        public override string ToString()
        {
            if (Item != null)
                return "Item: " + Item.ToString() + ", Chance: 1/" + (1 / Chance).ToString() + ", Enabled: " + Enabled.ToString();
            return "Item: None, Chance: 1/" + (1 / Chance).ToString() + ", Enabled: " + Enabled.ToString();
        }
    }
}
