﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using Clash_Library;
using Clash_Server;

namespace Server_Packets
{
    #region Common Packets

    public struct Disconnect : Packet
    {
        public byte Reason;
        public Disconnect(byte R)
        {
            Reason = R;
        }
        public void Read(BinaryReader BReader)
        {
            Reason = BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(Reason);
        }
    }
    public struct NewObjects : Packet
    {
        public List<PlayerStats> Characters;
        public List<MonsterStats> Monsters;
        //Items, Magic
        public NewObjects(List<object> Objects)
        {
            Characters = new List<PlayerStats>();
            Monsters = new List<MonsterStats>();
            foreach (object o in Objects)
                switch (o.GetType().Name)
                {
                    case "Player":
                        Characters.Add(new PlayerStats((Player)o));
                        break;
                    case "Merchant":
                        break;
                    case "Monster":
                        Monsters.Add(new MonsterStats((Monster)o));
                        break;
                    case "Item":
                        break;
                    case "Magic":
                        break;
                    default:
                        if (o as Monster != null)
                            Monsters.Add(new MonsterStats((Monster)o));
                        break;
                }
        }
        public void Read(BinaryReader BReader)
        {
            Characters = new List<PlayerStats>();
            PlayerStats CI;
            int Count = BReader.ReadInt32();
            for (int I = 0; I < Count; I++)
            {
                CI = new PlayerStats();
                CI.Read(BReader);
                Characters.Add(CI);
            }

            Monsters = new List<MonsterStats>();
            MonsterStats CM;
            Count = BReader.ReadInt32();
            for (int I = 0; I < Count; I++)
            {
                CM = new MonsterStats();
                CM.Read(BReader);
                Monsters.Add(CM);
            }
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            if (Characters == null)
                BWriter.Write((int)0);
            else
            {
                BWriter.Write(Characters.Count);
                foreach (PlayerStats CI in Characters)
                    CI.WriteTo(BWriter);
            }

            if (Monsters == null)
                BWriter.Write((int)0);
            else
            {
                BWriter.Write(Monsters.Count);
                foreach (MonsterStats CM in Monsters)
                    CM.WriteTo(BWriter);
            }
        }
    }

    #endregion
    #region Intro Packets

    public struct Login : Packet
    {
        public byte Result;
        public int BadPassCount;
        public string BanReason;
        public DateTime ExipryDate;
        public Login(byte R, int C)
        {
            Result = R;
            BadPassCount = C;
            BanReason = "";
            ExipryDate = DateTime.MinValue;
        }
        public Login(byte R)
        {
            Result = R;
            BadPassCount = 0;
            BanReason = "";
            ExipryDate = DateTime.MinValue;
        }
        public Login(byte R, string S, DateTime D)
        {
            Result = R;
            BadPassCount = 0;
            BanReason = S;
            ExipryDate = D;
        }
        public void Read(BinaryReader BReader)
        {
            Result = BReader.ReadByte();
            BadPassCount = BReader.ReadInt32();
            BanReason = BReader.ReadString();
            ExipryDate = DateTime.FromBinary(BReader.ReadInt64());
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(Result);
            BWriter.Write(BadPassCount);
            BWriter.Write(BanReason);
            BWriter.Write(ExipryDate.ToBinary());
        }
    }
    public struct NewAccount : Packet
    {
        public byte Result;
        public NewAccount(byte R)
        {
            Result = R;
        }
        public void Read(BinaryReader BReader)
        {
            Result = BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(Result);
        }
    }
    public struct ChangePass : Packet
    {
        public byte Result;
        public string BanReason;
        public DateTime ExipryDate;
        public ChangePass(byte R)
        {
            Result = R;
            BanReason = "";
            ExipryDate = DateTime.MinValue;
        }
        public ChangePass(byte R, string S, DateTime D)
        {
            Result = R;
            BanReason = S;
            ExipryDate = D;
        }
        public void Read(BinaryReader BReader)
        {
            Result = BReader.ReadByte();
            BanReason = BReader.ReadString();
            ExipryDate = DateTime.FromBinary(BReader.ReadInt64());
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(Result);
            BWriter.Write(BanReason);
            BWriter.Write(ExipryDate.ToBinary());
        }
    }
    public struct CharacterList : Packet
    {
        public List<CharSelInfo> CharList;
        public int LastCharID;
        public CharacterList(List<CharSelInfo> CL, int LastID)
        {
            CharList = CL;
            LastCharID = LastID;
        }
        public void Read(BinaryReader BReader)
        {
            CharList = new List<CharSelInfo>();
            int Count = BReader.ReadInt32();

            for (int I = 0; I < Count; I++)
            {
                CharSelInfo CI = new CharSelInfo();
                CI.Read(BReader);
                CharList.Add(CI);
            }
            LastCharID = BReader.ReadInt32();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(CharList.Count);
            foreach (CharSelInfo CI in CharList)
                CI.WriteTo(BWriter);
            BWriter.Write(LastCharID);
        }
    }
    public struct NewCharacter : Packet
    {
        public byte Result;
        public int CharID;
        public int Level;
        public NewCharacter(byte R, int CID, int level)
        {
            Result = R;
            CharID = CID;
            Level = level;
        }
        public NewCharacter(byte R)
        {
            Result = R;
            CharID = 0;
            Level = 0;
        }
        public void Read(BinaryReader BReader)
        {
            Result = BReader.ReadByte();
            CharID = BReader.ReadInt32();
            Level = BReader.ReadInt32();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(Result);
            BWriter.Write(CharID);
            BWriter.Write(Level);
        }
    }
    public struct DeleteCharacter : Packet
    {
        public byte Result;
        public DeleteCharacter(byte R)
        {
            Result = R;
        }
        public void Read(BinaryReader BReader)
        {
            Result = BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(Result);
        }
    }
    public struct Logout : Packet
    {
        public byte Result;
        public Logout(byte R)
        {
            Result = R;
        }
        public void Read(BinaryReader BReader)
        {
            Result = BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(Result);
        }
    }
    public struct StartGame : Packet
    {
        public byte Result;
        public string BanReason;
        public DateTime ExipryDate;
        public StartGame(byte R)
        {
            Result = R;
            BanReason = "";
            ExipryDate = DateTime.MinValue;
        }
        public StartGame(byte R, string B, DateTime D)
        {
            Result = R;
            BanReason = B;
            ExipryDate = D;
        }
        public void Read(BinaryReader BReader)
        {
            Result = BReader.ReadByte();
            BanReason = BReader.ReadString();
            ExipryDate = DateTime.FromBinary(BReader.ReadInt64());
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(Result);
            BWriter.Write(BanReason);
            BWriter.Write(ExipryDate.ToBinary());
        }
    }

    #endregion
    #region User Packets

    public struct UserInfo : Packet
    {
        public ActorInfo AI;
        public UserInfo(Player P)
        {
            AI = new ActorInfo(P);
        }
        public void Read(BinaryReader BReader)
        {
            AI = new ActorInfo();
            AI.Read(BReader);
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            AI.WriteTo(BWriter);
        }
    }
    public struct MoveFail : Packet
    {
        public Point Location;
        public Direction Dir;
        public MoveFail(Point L, Direction D)
        {
            Location = L;
            Dir = D;
        }
        public void Read(BinaryReader BReader)
        {
            Location = new Point(BReader.ReadInt16(), BReader.ReadInt16());
            Dir = (Direction)BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write((short)Location.X);
            BWriter.Write((short)Location.Y);
            BWriter.Write((byte)Dir);
        }
    }
    public struct Struck : Packet
    {
        public int CurrentHP;
        public Element Elems;
        public Struck(int HP, Element E)
        {
            CurrentHP = HP;
            Elems = E;
        }
        public void Read(BinaryReader BReader)
        {
            CurrentHP = BReader.ReadInt32();
            Elems = (Element)BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(CurrentHP);
            BWriter.Write((byte)Elems);
        }
    }
    public struct Regen : Packet
    {
        public int CurrentHP, CurrentMP;

        public Regen( int HP, int MP)
        {
            CurrentHP = HP;
            CurrentMP = MP;
        }
        public void Read(BinaryReader BReader)
        {
            CurrentHP = BReader.ReadInt32();
            CurrentMP = BReader.ReadInt32();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(CurrentHP);
            BWriter.Write(CurrentMP);
        }
    }
    public struct Revive : Packet
    {
        public Point Location;
        public Revive(Point Loc)
        {
            Location = Loc;
        }
        public void Read(BinaryReader BReader)
        {
            Location = new Point(BReader.ReadInt16(), BReader.ReadInt16());
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write((short)Location.X);
            BWriter.Write((short)Location.Y);
        }
    }
   public struct ChangeMap : Packet
   {
       public CMapInfo Info;
       public Point Location;
       public ChangeMap(MapInfo MI, Point P)
       {
           Info = new CMapInfo(MI);
           Location = P;
       }
       public void Read(BinaryReader BReader)
       {
           Info = new CMapInfo();
           Info.Read(BReader);
           Location = new Point(BReader.ReadInt16(), BReader.ReadInt16());
       }
       public void WriteTo(BinaryWriter BWriter)
       {
           Info.WriteTo(BWriter);
           BWriter.Write((short)Location.X);
           BWriter.Write((short)Location.Y);
       }
   }
   public struct WinEXP : Packet
   {
       public long Exp;
       public WinEXP(long E)
       {
           Exp = E;
       }
       public void Read(BinaryReader BReader)
       {
           Exp = BReader.ReadInt64();
       }
       public void WriteTo(BinaryWriter BWriter)
       {
           BWriter.Write(Exp);
       }
   }
   public struct LevelUp : Packet
   {
       public long MaxExp;
       public LevelUp(long ME)
       {
           MaxExp = ME;
       }
       public void Read(BinaryReader BReader)
       {
           MaxExp = BReader.ReadInt64();
       }
       public void WriteTo(BinaryWriter BWriter)
       {
           BWriter.Write(MaxExp);
       }
   }
    #endregion
    #region Other Player Packets
    public struct PlayerNew : Packet
    {
        public PlayerStats Info;
        public PlayerNew(Player P)
        {
            Info = new PlayerStats(P);
        }
        public void Read(BinaryReader BReader)
        {
            Info = new PlayerStats();
            Info.Read(BReader);
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            Info.WriteTo(BWriter);
        }
    }
    public struct PlayerRemove : Packet
    {
        public int PlayerID;
        public PlayerRemove(int ID)
        {
            PlayerID = ID;
        }
        public void Read(BinaryReader BReader)
        {
            PlayerID = BReader.ReadInt32();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(PlayerID);
        }
    }
    public struct PlayerTurn : Packet
    {
        public int PlayerID;
        public Direction Dir;
        public PlayerTurn(int ID, Direction D)
        {
            PlayerID = ID;
            Dir = D;
        }
        public void Read(BinaryReader BReader)
        {
            PlayerID = BReader.ReadInt32();
            Dir = (Direction)BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(PlayerID);
            BWriter.Write((byte)Dir);
        }
    }
    public struct PlayerWalk : Packet
    {
        public int PlayerID;
        public Point Location;
        public PlayerWalk(int ID, Point loc)
        {
            PlayerID = ID;
            Location = loc;
        }
        public void Read(BinaryReader BReader)
        {
            PlayerID = BReader.ReadInt32();
            Location = new Point(BReader.ReadInt16(), BReader.ReadInt16());
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(PlayerID);
            BWriter.Write((short)Location.X);
            BWriter.Write((short)Location.Y);
        }
    }
    public struct PlayerRun : Packet
    {
        public int PlayerID;
        public Point Location;
        public PlayerRun(int ID, Point loc)
        {
            PlayerID = ID;
            Location = loc;
        }
        public void Read(BinaryReader BReader)
        {
            PlayerID = BReader.ReadInt32();
            Location = new Point(BReader.ReadInt16(), BReader.ReadInt16());
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(PlayerID);
            BWriter.Write((short)Location.X);
            BWriter.Write((short)Location.Y);
        }
    }
    public struct PlayerHarvest : Packet
    {
        public int PlayerID;
        public Direction Dir;
        public PlayerHarvest(int ID, Direction D)
        {
            PlayerID = ID;
            Dir = D;
        }
        public void Read(BinaryReader BReader)
        {
            PlayerID = BReader.ReadInt32();
            Dir = (Direction)BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(PlayerID);
            BWriter.Write((byte)Dir);
        }
    }
    public struct PlayerAttack : Packet
    {
        public int PlayerID;
        public Direction Dir;
        public PlayerAttack(int ID, Direction D)
        {
            PlayerID = ID;
            Dir = D;
        }
        public void Read(BinaryReader BReader)
        {
            PlayerID = BReader.ReadInt32();
            Dir = (Direction)BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(PlayerID);
            BWriter.Write((byte)Dir);
        }
    }
    public struct PlayerChanged : Packet
    {
        public int PlayerID;
        public byte Armor, Weapon, Hair;
        public Gender Sex;
        public int MaxHP, MaxMP;
        public PlayerChanged(int ID, byte A, byte W, byte H, Gender S, int HP, int MP)
        {
            PlayerID = ID;
            Armor = A;
            Weapon = W;
            Hair = H;
            Sex = S;
            MaxHP = HP;
            MaxMP = MP;
        }
        public void Read(BinaryReader BReader)
        {
            PlayerID = BReader.ReadInt32();
            Armor = BReader.ReadByte();
            Weapon = BReader.ReadByte();
            Hair = BReader.ReadByte();
            Sex = (Gender)BReader.ReadByte();
            MaxHP = BReader.ReadInt32();
            MaxMP = BReader.ReadInt32();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(PlayerID);
            BWriter.Write(Armor);
            BWriter.Write(Weapon);
            BWriter.Write(Hair);
            BWriter.Write((byte)Sex);
            BWriter.Write(MaxHP);
            BWriter.Write(MaxMP);
        }
    }
    public struct PlayerStruck : Packet
    {
        public int PlayerID;
        public int CurrentHP;
        public Element Elems;
        public Direction Dir;
        public PlayerStruck(int ID, int HP, Element E, Direction D)
        {
            PlayerID = ID;
            CurrentHP = HP;
            Elems = E;
            Dir = D;
        }
        public void Read(BinaryReader BReader)
        {
            PlayerID = BReader.ReadInt32();
            CurrentHP = BReader.ReadInt32();
            Elems = (Element)BReader.ReadByte();
            Dir = (Direction)BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(PlayerID);
            BWriter.Write(CurrentHP);
            BWriter.Write((byte)Elems);
            BWriter.Write((byte)Dir);
        }
    }
    public struct PlayerRegen : Packet
    {
        public int PlayerID;
        public int CurrentHP, CurrentMP;

        public PlayerRegen(int ID, int HP, int MP)
        {
            PlayerID = ID;
            CurrentHP = HP;
            CurrentMP = MP;
        }
        public void Read(BinaryReader BReader)
        {
            PlayerID = BReader.ReadInt32();
            CurrentHP = BReader.ReadInt32();
            CurrentMP = BReader.ReadInt32();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(PlayerID);
            BWriter.Write(CurrentHP);
            BWriter.Write(CurrentMP);
        }
    }
    public struct PlayerRevive : Packet
    {
        public int PlayerID;
        public Point Location;
        public int CurrentHP;
        public PlayerRevive(int ID, Point Loc, int HP)
        {
            PlayerID = ID;
            Location = Loc;
            CurrentHP = HP;
        }
        public void Read(BinaryReader BReader)
        {
            PlayerID = BReader.ReadInt32();
            Location = new Point(BReader.ReadInt16(), BReader.ReadInt16());
            CurrentHP = BReader.ReadInt32();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(PlayerID);
            BWriter.Write((short)Location.X);
            BWriter.Write((short)Location.Y);
            BWriter.Write(CurrentHP);
        }

    }

    //Spell


    #endregion
    #region Monster Packets
    public struct MonsterNew : Packet
    {
        public MonsterStats Info;
        public MonsterNew(Monster M)
        {
            Info = new MonsterStats(M);
        }
        public void Read(BinaryReader BReader)
        {
            Info = new MonsterStats();
            Info.Read(BReader);
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            Info.WriteTo(BWriter);
        }
    }
    public struct MonsterRemove : Packet
    {
        public int MonsterID;
        public MonsterRemove(int ID)
        {
            MonsterID = ID;
        }
        public void Read(BinaryReader BReader)
        {
            MonsterID = BReader.ReadInt32();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(MonsterID);
        }
    }
    public struct MonsterTurn : Packet
    {
        public int MonsterID;
        public Direction Dir;
        public MonsterTurn(Direction D, int ID)
        {
            MonsterID = ID;
            Dir = D;
        }
        public void Read(BinaryReader BReader)
        {
            MonsterID = BReader.ReadInt32();
            Dir = (Direction)BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(MonsterID);
            BWriter.Write((byte)Dir);
        }
    }
    public struct MonsterWalk : Packet
    {
        public int MonsterID;
        public Point Location;
        public MonsterWalk(int ID, Point loc)
        {
            MonsterID = ID;
            Location = loc;
        }
        public void Read(BinaryReader BReader)
        {
            MonsterID = BReader.ReadInt32();
            Location = new Point(BReader.ReadInt16(), BReader.ReadInt16());
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(MonsterID);
            BWriter.Write((short)Location.X);
            BWriter.Write((short)Location.Y);
        }
    }
    public struct MonsterAttack : Packet
    {
        public int MonsterID;
        public Direction Dir;
        public MonsterAttack(Direction D, int ID)
        {
            MonsterID = ID;
            Dir = D;
        }
        public void Read(BinaryReader BReader)
        {
            MonsterID = BReader.ReadInt32();
            Dir = (Direction)BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(MonsterID);
            BWriter.Write((byte)Dir);
        }
    }
    public struct MonsterStruck : Packet
    {
        public int MonsterID;
        public int CurrentHP;
        public Element Elems;
        public Direction Dir;
        public MonsterStruck(int ID, int HP, Element E, Direction D)
        {
            MonsterID = ID;
            CurrentHP = HP;
            Elems = E;
            Dir = D;
        }
        public void Read(BinaryReader BReader)
        {
            MonsterID = BReader.ReadInt32();
            CurrentHP = BReader.ReadInt32();
            Elems = (Element)BReader.ReadByte();
            Dir = (Direction)BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(MonsterID);
            BWriter.Write(CurrentHP);
            BWriter.Write((byte)Elems);
            BWriter.Write((byte)Dir);
        }
    }
    public struct MonsterRegen : Packet
    {
        public int MonsterID;
        public int CurrentHP, CurrentMP;
        public MonsterRegen(int ID, int HP, int MP)
        {
            MonsterID = ID;
            CurrentHP = HP;
            CurrentMP = MP;
        }
        public void Read(BinaryReader BReader)
        {
            MonsterID = BReader.ReadInt32();
            CurrentHP = BReader.ReadInt32();
            CurrentMP = BReader.ReadInt32();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(MonsterID);
            BWriter.Write(CurrentHP);
            BWriter.Write(CurrentMP);
        }
    }
    public struct MonsterSpell : Packet
    {
        public int MonsterID;
        public short SpellID;
        public Direction Dir;
        public MonsterSpell(int MID, short SID, Direction D)
        {
            MonsterID = MID;
            SpellID = SID;
            Dir = D;
        }
        public void Read(BinaryReader BReader)
        {
            MonsterID = BReader.ReadInt32();
            SpellID = BReader.ReadInt16();
            Dir = (Direction)BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(MonsterID);
            BWriter.Write(SpellID);
            BWriter.Write((byte)Dir);
        }
    }


    //Show (Zuma, CanniPlant, EC, ZT ...)
    //Hide



    #endregion
    #region Item Packets

    public struct ItemList : Packet
    {
        public List<ItemInfo> InfoList;
        public ItemList(List<ItemInfo> IL)
        {
            InfoList = IL;
        }
        public void Read(BinaryReader BReader)
        {
            InfoList = new List<ItemInfo>();
            int Count = BReader.ReadInt32();
            ItemInfo IF;
            for (int I = 0; I < Count; I++)
            {
                IF = new ItemInfo();
                IF.Read(BReader);
                InfoList.Add(IF);
            }
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(InfoList.Count);
            foreach (ItemInfo I in InfoList)
                I.WriteTo(BWriter);
        }
    }
    public struct EquipItem : Packet
    {
        public int ItemID;
        public ItemLocation Loc;
        public EquipItem(ItemLocation loc, int itemID)
        {
            Loc = loc;
            ItemID = itemID;
        }
        public void Read(BinaryReader BReader)
        {
            ItemID = BReader.ReadInt32();
            Loc = (ItemLocation)BReader.ReadByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(ItemID);
            BWriter.Write((byte)Loc);
        }
    }
    public struct UnEquipItem : Packet
    {
        public int ItemID;
        public UnEquipItem(int itemID)
        {
            ItemID = itemID;
        }
        public void Read(BinaryReader BReader)
        {
            ItemID = BReader.ReadInt32();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(ItemID);
        }
    }
    #endregion

    
}
