﻿using System;
using System.IO;

namespace Clash_Library
{
    [Serializable]
    public class AllStats : Packet
    {
        #region Fields
        public BasicStats Basic;
        public AdvancedStats Advanced;
        public ElementalStats Elemental;
        #endregion
        #region Constructor
        public AllStats()
        {
            Basic = new BasicStats();
            Advanced = new AdvancedStats();
            Elemental = new ElementalStats();
        }
        public AllStats(BasicStats B, AdvancedStats A, ElementalStats E)
        {
            Basic = B;
            Advanced = A;
            Elemental = E;
        }
        #endregion
        #region Operators
        public static AllStats operator +(AllStats A1, AllStats A2)
        {
            A1.Basic += A2.Basic;
            A1.Advanced += A2.Advanced;
            A1.Elemental += A2.Elemental;
            return A1;
        }
        public static AllStats operator -(AllStats A1, AllStats A2)
        {
            A1.Basic -= A2.Basic;
            A1.Advanced -= A2.Advanced;
            A1.Elemental -= A2.Elemental;
            return A1;
        }
        #endregion
        #region Packet
        public void Read(BinaryReader BReader)
        {
            Basic = new BasicStats();
            Basic.Read(BReader);
            Advanced = new AdvancedStats();
            Advanced.Read(BReader);
            Elemental = new ElementalStats();
            Elemental.Read(BReader);
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            Basic.WriteTo(BWriter);
            Advanced.WriteTo(BWriter);
            Elemental.WriteTo(BWriter);
        }
        #endregion
        #region Methods
        public short GetAC(Random R)
        {
            if (Basic.AC == 0 && Basic.AC2 == 0) return 0;
            if (Basic.AC >= Basic.AC2) return Basic.AC;

            if (Advanced.Luck > 0)
            {
                if (R.NextDouble() * 10 <= Advanced.Luck)
                    return Basic.AC2;
            }
            else if (Advanced.Luck < 0)
            {
                if (R.NextDouble() * 10 <= -Advanced.Luck)
                    return Basic.AC;
            }

            return (short)R.Next(Basic.AC, Basic.AC2 + 1);
        }
        public short GetMAC(Random R)
        {
            if (Basic.MAC == 0 && Basic.MAC2 == 0) return 0;
            if (Basic.MAC >= Basic.MAC2) return Basic.MAC;

            if (Advanced.Luck > 0)
            {
                if (R.NextDouble() * 10 <= Advanced.Luck)
                    return Basic.MAC2;
            }
            else if (Advanced.Luck < 0)
            {
                if (R.NextDouble() * 10 <= -Advanced.Luck)
                    return Basic.MAC;
            }

            return (short)R.Next(Basic.MAC, Basic.MAC2 + 1);
        }
        public short GetDC(Random R)
        {
            if (Basic.DC == 0 && Basic.DC2 == 0) return 0;
            if (Basic.DC >= Basic.DC2) return Basic.DC;

            if (Advanced.Luck > 0)
            {
                if (R.NextDouble() * 10 <= Advanced.Luck)
                    return Basic.DC2;
            }
            else if (Advanced.Luck < 0)
            {
                if (R.NextDouble() * 10 <= -Advanced.Luck)
                    return Basic.DC;
            }

            return (short)R.Next(Basic.DC, Basic.DC2 + 1);
        }
        public short GetMC(Random R)
        {
            if (Basic.MC == 0 && Basic.MC2 == 0) return 0;
            if (Basic.MC >= Basic.MC2) return Basic.MC;

            if (Advanced.Luck > 0)
            {
                if (R.NextDouble() * 10 <= Advanced.Luck)
                    return Basic.MC2;
            }
            else if (Advanced.Luck < 0)
            {
                if (R.NextDouble() * 10 <= -Advanced.Luck)
                    return Basic.MC;
            }

            return (short)R.Next(Basic.MC, Basic.MC2 + 1);
        }
        public short GetSC(Random R)
        {
            if (Basic.SC == 0 && Basic.SC2 == 0) return 0;
            if (Basic.SC >= Basic.SC2) return Basic.SC;

            if (Advanced.Luck > 0)
            {
                if (R.NextDouble() * 10 <= Advanced.Luck)
                    return Basic.SC2;
            }
            else if (Advanced.Luck < 0)
            {
                if (R.NextDouble() * 10 <= -Advanced.Luck)
                    return Basic.SC;
            }

            return (short)R.Next(Basic.SC, Basic.SC2 + 1);
        }
        public int PhysicalStruck(AllStats Stats, Random R)
        {
            if (R.Next(Basic.Agility + 1) >= Stats.Basic.Accuracy) return 0;
            int DC = Stats.GetDC(R);
            if (DC == 0) return 0;
            int AC = GetAC(R);

            int
                FDmg = Stats.Elemental.FireA > 0 ? (int)(DC * (Stats.Elemental.FireA / 10F)) : 0,
                IDmg = Stats.Elemental.IceA > 0 ? (int)(DC * (Stats.Elemental.IceA / 10F)) : 0,
                LDmg = Stats.Elemental.LightningA > 0 ? (int)(DC * (Stats.Elemental.LightningA / 10F)) : 0,
                WDmg = Stats.Elemental.WindA > 0 ? (int)(DC * (Stats.Elemental.WindA / 10F)) : 0,
                DDmg = Stats.Elemental.DarkA > 0 ? (int)(DC * (Stats.Elemental.DarkA / 10F)) : 0,
                HDmg = Stats.Elemental.HolyA > 0 ? (int)(DC * (Stats.Elemental.HolyA / 10F)) : 0,
                IlDmg = Stats.Elemental.IllusionA > 0 ? (int)(DC * (Stats.Elemental.IllusionA / 10F)) : 0;

            int EleCount = 0;

            EleCount += Stats.Elemental.FireA > 0 ? Stats.Elemental.FireA : 0;
            EleCount += Stats.Elemental.IceA > 0 ? Stats.Elemental.IceA : 0;
            EleCount += Stats.Elemental.LightningA > 0 ? Stats.Elemental.LightningA : 0;
            EleCount += Stats.Elemental.WindA > 0 ? Stats.Elemental.WindA : 0;
            EleCount += Stats.Elemental.DarkA > 0 ? Stats.Elemental.DarkA : 0;
            EleCount += Stats.Elemental.HolyA > 0 ? Stats.Elemental.HolyA : 0;
            EleCount += Stats.Elemental.IllusionA > 0 ? Stats.Elemental.IllusionA : 0;

            int Dmg = EleCount >= 10 ? 0 : (int)(DC * ((10 - EleCount) / 10F));
            Dmg = Dmg - AC > 0 ? (Dmg - AC) : 0;


            if (FDmg > 0)
                FDmg -= (int)(FDmg * (Elemental.FireD / 10F));
            if (IDmg > 0)
                IDmg -= (int)(IDmg * (Elemental.IceD / 10F));
            if (LDmg > 0)
                LDmg -= (int)(LDmg * (Elemental.LightningD / 10F));
            if (WDmg > 0)
                WDmg -= (int)(WDmg * (Elemental.WindD / 10F));
            if (DDmg > 0)
                DDmg -= (int)(DDmg * (Elemental.DarkD / 10F));
            if (HDmg > 0)
                HDmg -= (int)(HDmg * (Elemental.HolyD / 10F));
            if (IlDmg > 0)
                IlDmg -= (int)(IlDmg * (Elemental.IllusionD / 10F));

            return Dmg + FDmg + IDmg + LDmg + WDmg + DDmg + HDmg + IlDmg;
        }

        public AllStats Clone()
        {
            AllStats AS = new AllStats();
            AS.Advanced = Advanced.Clone();
            AS.Basic = Basic.Clone();
            AS.Elemental = Elemental.Clone();
            return AS;
        }

        #endregion
    }
    [Serializable]
    public class BasicStats : Packet
    {
        #region Fields
        public int HP;
        public int MP;
        public short AC, AC2;
        public short MAC, MAC2;
        public short DC, DC2;
        public short MC, MC2;
        public short SC, SC2;
        public short Accuracy, Agility;
        #endregion
        public static BasicStats operator +(BasicStats A1, BasicStats A2)
        {
            A1.HP += A2.HP;
            A1.MP += A2.MP;
            A1.AC += A2.AC;
            A1.AC2 += A2.AC2;
            A1.MAC += A2.MAC;
            A1.MAC2 += A2.MAC2;
            A1.DC += A2.DC;
            A1.DC2 += A2.DC2;
            A1.MC += A2.MC;
            A1.MC2 += A2.MC2;
            A1.SC += A2.SC;
            A1.SC2 += A2.SC2;

            A1.Accuracy += A2.Accuracy;
            A1.Agility += A2.Agility;
            return A1;
        }
        public static BasicStats operator -(BasicStats A1, BasicStats A2)
        {
            A1.HP -= A2.HP;
            A1.MP -= A2.MP;
            A1.AC -= A2.AC;
            A1.AC2 -= A2.AC2;
            A1.MAC -= A2.MAC;
            A1.MAC2 -= A2.MAC2;
            A1.DC -= A2.DC;
            A1.DC2 -= A2.DC2;
            A1.MC -= A2.MC;
            A1.MC2 -= A2.MC2;
            A1.SC -= A2.SC;
            A1.SC2 -= A2.SC2;

            A1.Accuracy -= A2.Accuracy;
            A1.Agility -= A2.Agility;
            return A1;
        }

        public BasicStats Clone()
        {
            BasicStats BS = new BasicStats();

            BS.HP = HP;
            BS.MP = MP;
            BS.AC = AC;
            BS.AC2 = AC2;
            BS.MAC = MAC2;
            BS.DC = DC;
            BS.DC2 = DC2;
            BS.MC = MC;
            BS.MC2 = MC2;
            BS.SC = SC;
            BS.SC2 = SC2;
            BS.Accuracy = Accuracy;
            BS.Agility = Agility;
            return BS;
        }

        public void Read(BinaryReader BReader)
        {
            HP = BReader.ReadInt32();
            MP = BReader.ReadInt32();
            AC = BReader.ReadInt16();
            AC2 = BReader.ReadInt16();
            MAC = BReader.ReadInt16();
            MAC2 = BReader.ReadInt16();
            DC = BReader.ReadInt16();
            DC2 = BReader.ReadInt16();
            MC = BReader.ReadInt16();
            MC2 = BReader.ReadInt16();
            SC = BReader.ReadInt16();
            SC2 = BReader.ReadInt16();
            Accuracy = BReader.ReadInt16();
            Agility = BReader.ReadInt16();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(HP);
            BWriter.Write(MP);
            BWriter.Write(AC);
            BWriter.Write(AC2);
            BWriter.Write(MAC);
            BWriter.Write(MAC2);
            BWriter.Write(DC);
            BWriter.Write(DC2);
            BWriter.Write(MC);
            BWriter.Write(MC2);
            BWriter.Write(SC);
            BWriter.Write(SC2);
            BWriter.Write(Accuracy);
            BWriter.Write(Agility);
        }
    }
    [Serializable]
    public class AdvancedStats : Packet
    {
        #region Fields
        public short ASpeed;
        public sbyte Luck, Light;
        public sbyte MR, PR, CR;
        public sbyte CA, PA;
        #endregion
        public static AdvancedStats operator +(AdvancedStats A1, AdvancedStats A2)
        {
            A1.ASpeed += A2.ASpeed;
            A1.Luck += A2.Luck;
            A1.Light += A2.Light;
            A1.MR += A2.MR;
            A1.PR += A2.PR;
            A1.CR += A2.CR;
            A1.CA += A2.CA;
            A1.PA += A2.PA;
            return A1;
        }
        public static AdvancedStats operator -(AdvancedStats A1, AdvancedStats A2)
        {
            A1.ASpeed -= A2.ASpeed;
            A1.Luck -= A2.Luck;
            A1.Light -= A2.Light;
            A1.MR -= A2.MR;
            A1.PR -= A2.PR;
            A1.CR -= A2.CR;
            A1.CA -= A2.CA;
            A1.PA -= A2.PA;
            return A1;
        }

        public AdvancedStats Clone()
        {
            AdvancedStats AS = new AdvancedStats();

            AS.ASpeed = ASpeed;
            AS.Luck = Luck;
            AS.Light = Light;
            AS.MR = MR;
            AS.PR = PR;
            AS.CR = CR;
            AS.CA = CA;
            AS.PA = PA;
            return AS;
        }

        public void Read(BinaryReader BReader)
        {
            ASpeed = BReader.ReadInt16();
            Luck = BReader.ReadSByte();
            Light = BReader.ReadSByte();
            MR = BReader.ReadSByte();
            PR = BReader.ReadSByte();
            CR = BReader.ReadSByte();
            CA = BReader.ReadSByte();
            PA = BReader.ReadSByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(ASpeed);
            BWriter.Write(Luck);
            BWriter.Write(Light);
            BWriter.Write(MR);
            BWriter.Write(PR);
            BWriter.Write(CR);
            BWriter.Write(CA);
            BWriter.Write(PA);
        }
    }
    [Serializable]
    public class ElementalStats : Packet
    {
        #region Fields
        public sbyte FireA, LightningA, IceA, WindA, DarkA, HolyA, IllusionA;
        public sbyte FireD, LightningD, IceD, WindD, DarkD, HolyD, IllusionD;
        #endregion
        public static ElementalStats operator +(ElementalStats A1, ElementalStats A2)
        {
            A1.FireA += A2.FireA;
            A1.LightningA += A2.LightningA;
            A1.IceA += A2.IceA;
            A1.WindA += A2.WindA;
            A1.DarkA += A2.DarkA;
            A1.HolyA += A2.HolyA;
            A1.IllusionA += A2.IllusionA;

            A1.FireD += A2.FireD;
            A1.LightningD += A2.LightningD;
            A1.IceD += A2.IceD;
            A1.WindD += A2.WindD;
            A1.DarkD += A2.DarkD;
            A1.HolyD += A2.HolyD;
            A1.IllusionD += A2.IllusionD;

            return A1;
        }
        public static ElementalStats operator -(ElementalStats A1, ElementalStats A2)
        {
            A1.FireA -= A2.FireA;
            A1.LightningA -= A2.LightningA;
            A1.IceA -= A2.IceA;
            A1.WindA -= A2.WindA;
            A1.DarkA -= A2.DarkA;
            A1.HolyA -= A2.HolyA;
            A1.IllusionA -= A2.IllusionA;

            A1.FireD -= A2.FireD;
            A1.LightningD -= A2.LightningD;
            A1.IceD -= A2.IceD;
            A1.WindD -= A2.WindD;
            A1.DarkD -= A2.DarkD;
            A1.HolyD -= A2.HolyD;
            A1.IllusionD -= A2.IllusionD;

            return A1;
        }

        public ElementalStats Clone()
        {
            ElementalStats ES = new ElementalStats();

            ES.FireA = FireA;
            ES.LightningA = LightningA;
            ES.IceA = IceA;
            ES.WindA = WindA;
            ES.DarkA = DarkA;
            ES.HolyA = HolyA;
            ES.IllusionA = IllusionA;

            ES.FireD = FireD;
            ES.LightningD = LightningD;
            ES.IceD = IceD;
            ES.WindD = WindD;
            ES.DarkD = DarkD;
            ES.HolyD = HolyD;
            ES.IllusionD = IllusionD;
            return ES;
        }

        public void Read(BinaryReader BReader)
        {
            FireA = BReader.ReadSByte();
            LightningA = BReader.ReadSByte();
            IceA = BReader.ReadSByte();
            WindA = BReader.ReadSByte();
            DarkA = BReader.ReadSByte();
            HolyA = BReader.ReadSByte();
            IllusionA = BReader.ReadSByte();
            FireD = BReader.ReadSByte();
            LightningD = BReader.ReadSByte();
            IceD = BReader.ReadSByte();
            WindD = BReader.ReadSByte();
            DarkD = BReader.ReadSByte();
            HolyD = BReader.ReadSByte();
            IllusionD = BReader.ReadSByte();
        }
        public void WriteTo(BinaryWriter BWriter)
        {
            BWriter.Write(FireA);
            BWriter.Write(LightningA);
            BWriter.Write(IceA);
            BWriter.Write(WindA);
            BWriter.Write(DarkA);
            BWriter.Write(HolyA);
            BWriter.Write(IllusionA);
            BWriter.Write(FireD);
            BWriter.Write(LightningD);
            BWriter.Write(IceD);
            BWriter.Write(WindD);
            BWriter.Write(DarkD);
            BWriter.Write(HolyD);
            BWriter.Write(IllusionD);
        }
    }
}
