﻿using System.Drawing;

namespace Clash_Client
{
    public enum ButtonState { Normal, Pressed }
    class Button
    {
        public static Button SelectedButton;
        public int BOffSet = 0, SIndex = -1;
        public Point Location, DrawPoint;
        public Size S;
        public DefaultDelegate Function;
        public ButtonState State = ButtonState.Normal;
        public bool Visible, NormalImage, Sticky, Locked;
        public string Text;

        private bool Initialized = false;
        private int image;
        private Window parent;

        #region Properties
        public int Image
        {
            get { return image; }
            set
            {
                image = value;
                Initialized = false;
            }
        }
        public Window Parent
        {
            get { return parent; }
            set
            {
                parent = value;
                if (parent != null)
                    parent.ButtonList.Add(this);
            }
        }
        #endregion
        #region Constructor
        public Button(int I, Point L, bool V, int S)
        {
            Image = I;
            Location = L;
            Visible = V;
            SIndex = S;
        }
        public Button(int I, Point L, bool V, int S, bool N, int O)
        {
            Image = I;
            Location = L;
            Visible = V;
            SIndex = S;
            NormalImage = N;
            BOffSet = O;
        }
        #endregion

        public void CleanUp()
        {
            State = ButtonState.Normal;
            Text = "";
        }
        private void Initialize()
        {
            Initialized = true;
            UpdateLocation();
            S = CLib.Interface.GetImageSize(Image);
        }
        public void Draw()
        {
            if (!Initialized) Initialize();
            if (Visible)
            {
                switch (State)
                {
                    case ButtonState.Normal:
                        if (NormalImage) CLib.Interface.Draw(Image, DrawPoint, false);
                        break;
                    case ButtonState.Pressed:
                        CLib.Interface.Draw(Image + BOffSet, DrawPoint, false);
                        break;
                }
                if (!string.IsNullOrEmpty(Text))
                    Graphics.DXNormalFont.DrawText(Graphics.S, Text, DrawPoint.X + 10, DrawPoint.Y + 10, Color.White);                
                
            }

        }
        public void UpdateLocation()
        {
            DrawPoint = Location;
            if (Parent != null) DrawPoint.Offset(Parent.Location);
        }
        public bool Click(Point P)
        {
            if (new Rectangle(Location, S).Contains(P))
            {
                P.Offset(-Location.X, -Location.Y);
                if (CLib.Interface.GetTransparent(image, P))
                {
                    if (!Locked && (SelectedButton == this || Sticky) && Function != null) Function.Invoke();
                    return true;
                }
            }
            return false;
        }
        public bool MouseDown(Point P)
        {
            if (new Rectangle(Location, S).Contains(P))
            {
                P.Offset(-Location.X, -Location.Y);
                if (CLib.Interface.GetTransparent(image, P))
                {
                    if (!Sticky && !Locked)
                    {
                        State = ButtonState.Pressed;
                        SelectedButton = this;
                    }
                    return true;
                }
            }
            return false;
        }
        public void MouseUp()
        {
            if (!Sticky)
                State = ButtonState.Normal;
            if (SelectedButton == this) SelectedButton = null;
        }
        public bool MouseMove(Point P)
        {
            if (new Rectangle(Location, S).Contains(P))
            {
                if (SelectedButton == this && !Sticky)
                {
                    State = ButtonState.Pressed;
                    return true;
                }
            }
            if (!Sticky)
                State = ButtonState.Normal;
            return false;
        }
    }
}
