﻿using System.Drawing;
using Microsoft.DirectX;

namespace Clash_Client
{
    class TextBox
    {
        public static TextBox SelectedTextBox, Focused;
        public bool Password, Visible, AnyLetter, Locked;
        public Point Location;
        public Size S;
        public int MaxLength = 15;
        public string PasswordText;



        private int SS = 0, SW = 0, TP = 0;        
        private int SSX = 0, SWX = 0, TPX = 0;
        private Point DrawPoint;
        private string text;
        private Window parent;
        private bool focus;

        public bool Focus
        {
            get { return focus; }
            set
            {
                if (focus = value)
                {
                    if (Focused != null &&  Focused != this) Focused.Focus = false;
                    Focused = this;
                }
                else
                {
                    if (Focused != null && Focused == this)
                    {
                        Focused.focus = false;
                        Focused = null;
                    }
                    TextPointer = 0;
                    SelStart = 0;
                    SelWidth = 0;
                }
            }
        }

        public Window Parent
        {
            get { return parent; }
            set
            {
                parent = value;
                if (parent != null)
                    parent.TextBoxList.Add(this);
            }
        }
        public string Text
        {
            get { return text; }
            set
            {
                if (Password)
                {
                    PasswordText = value;
                    text = "";
                    if (!string.IsNullOrEmpty(value))
                        for (int I = 0; I < value.Length; I++)
                            text += "*";
                }
                else
                    text = value;
            }
        }
        public int TextPointer
        {
            get { return TP; }
            set
            {
                TP = value;
                if (string.IsNullOrEmpty(Text))
                    TPX = 0;
                else
                    TPX = Functions.GetPointerLocation(Text.Substring(0, value));
            }
        }
        public int SelWidth
        {
            get { return SW; }
            set {

                SW = value;
                if (string.IsNullOrEmpty(Text))
                    SWX = 0;
                else
                    if (value < 0)
                        SWX = -Functions.GetPointerLocation(Text.Substring(SS + value, -value));
                    else
                        SWX = Functions.GetPointerLocation(Text.Substring(SS, value));
            }
        }
        public int SelStart
        {
            get { return SS; }
            set
            {
                SS = value;
                if (string.IsNullOrEmpty(Text))
                    SSX = 0;
                else
                    SSX = Functions.GetPointerLocation(Text.Substring(0, value));
            }
        }


        #region Constructor
        public TextBox(Point L, Size size, bool V)
        {
            Location = L;
            S = size;
            Visible = V;
        }
        public TextBox(Point L, Size size, bool V, bool P)
        {
            Location = L;
            S = size;
            Visible = V;
            Password = P;
        }
        #endregion

        public void UpdateLocation()
        {
            DrawPoint = Location;
            if (Parent != null) DrawPoint.Offset(Parent.Location);
        }
        public void Draw()
        {
            if (Visible)
            {
                Graphics.S.Flush();
                if (Config.DrawMethod2)
                    Graphics.S.Draw2D(Graphics.GTexture, new Rectangle(DrawPoint, S), Rectangle.Empty, DrawPoint, Color.Black.ToArgb());
                else
                    Graphics.S.Draw(Graphics.GTexture, new Rectangle(DrawPoint, S), Vector3.Empty, new Vector3(DrawPoint.X, DrawPoint.Y, 0), Color.Black.ToArgb());


                if (SWX != 0)
                    if (Config.DrawMethod2)
                        Graphics.S.Draw2D(Graphics.GTexture, new Rectangle(DrawPoint, new Size(SWX, S.Height - 2)), Rectangle.Empty, new Point(DrawPoint.X + SSX, DrawPoint.Y + 1), Color.Gray.ToArgb());
                    else
                        Graphics.S.Draw(Graphics.GTexture, new Rectangle(DrawPoint, new Size(SWX, S.Height - 2)), Vector3.Empty, new Vector3(DrawPoint.X + SSX, DrawPoint.Y + 1, 0), Color.Gray.ToArgb());

                if (!string.IsNullOrEmpty(Text))
                    Graphics.DXNormalFont.DrawText(Graphics.S, Text, DrawPoint, Color.White);

                if (focus)
                    if (Config.DrawMethod2)
                        Graphics.S.Draw2D(Graphics.GTexture, new Rectangle(DrawPoint, new Size(1, S.Height - 2)), Rectangle.Empty, new Point(DrawPoint.X + TPX, DrawPoint.Y + 1), Color.DarkCyan.ToArgb());
                    else
                        Graphics.S.Draw(Graphics.GTexture, new Rectangle(DrawPoint, new Size(1, S.Height - 2)), Vector3.Empty, new Vector3(DrawPoint.X + TPX, DrawPoint.Y + 1, 0), Color.DarkCyan.ToArgb());

            }
        }
        public void Select()
        {
            if (!string.IsNullOrEmpty(text))
            {
                SelStart = 0;
                SelWidth = text.Length;
                TextPointer = Text.Length;
            }
            Focus = true;

        }
        public bool Click(Point P)
        {
            if (new Rectangle(Location, S).Contains(P) || SelectedTextBox == this)
            {
                P.Offset(-Location.X, Location.Y);
                if (string.IsNullOrEmpty(Text))
                    TextPointer = 0;
                else
                {
                    TextPointer = Functions.GetTextPointer(text, P.X);
                    SelWidth = TextPointer - SelStart;
                }
                SelectedTextBox = null;
                return true;
            }

            return false;
        }
        public bool DoubleClick(Point P)
        {
            if (new Rectangle(Location, S).Contains(P))
            {
                if (!string.IsNullOrEmpty(text))
                {
                    SelStart = 0;
                    SelWidth = text.Length;
                    TextPointer = Text.Length;
                }
                SelectedTextBox = null;
                return true;
            }
            return false;
        }
        public bool MouseDown(Point P)
        {
            if (new Rectangle(Location, S).Contains(P))
            {
                SelectedTextBox = this;
                Focus = true;
                P.Offset(-Location.X, Location.Y);
                TextPointer = 0;
                SelStart = 0;
                SelWidth = 0;
                if (!string.IsNullOrEmpty(Text))
                {
                    TextPointer = Functions.GetTextPointer(text, P.X);
                    SelStart = TextPointer;
                }
                return true;
            }
                        return false;
        }
        public void CleanUp()
        {
            Focus = false;
            Text = "";
        }
        public bool MouseMove(Point P)
        {
            if (new Rectangle(Location, S).Contains(P))
            {
                if (SelectedTextBox == this)
                {
                    P.Offset(-Location.X, Location.Y);
                    if (string.IsNullOrEmpty(Text))
                        TextPointer = 0;
                    else
                    {
                        TextPointer = Functions.GetTextPointer(text, P.X);
                        SelWidth = TextPointer - SelStart;
                    }
                }
                return true;
            }

            return false;
        }
        public void Type(string Str)
        {
            if (Locked || string.IsNullOrEmpty(Str)) return;

            if (SW < 0)
            {
                if (Password)
                    Text = PasswordText.Remove(SS + SW, -SW);
                else
                    Text = Text.Remove(SS + SW, -SW);
            }
            else if (SW > 0)
            {
                if (Password)
                    Text = PasswordText.Remove(SS, SW);
                else
                    Text = Text.Remove(SS, SW);
                TextPointer = SS;
            }
            SelStart = 0;
            SelWidth = 0;
            
            if (string.IsNullOrEmpty(text))
            {
                Text = "";
            }
            if (Text.Length + Str.Length > MaxLength)
            {
                if (Password)
                    Text = PasswordText.Insert(TP, Str.Substring(0, MaxLength - Text.Length));
                else
                    Text = Text.Insert(TP, Str.Substring(0, MaxLength - Text.Length));
                TextPointer = Text.Length;
            }
            else
            {
                if (Password)
                    Text = PasswordText.Insert(TP, Str);
                else
                    Text = Text.Insert(TP, Str);
                TextPointer = TP + Str.Length;
            }
        }
        public void Delete()
        {
            if (Locked || string.IsNullOrEmpty(text)) return;

            if (SW < 0)
            {
                if (Password)
                    Text = PasswordText.Remove(SS + SW, -SW);
                else
                    Text = Text.Remove(SS + SW, -SW);
            }
            else if (SW > 0)
            {
                if (Password)
                    Text = PasswordText.Remove(SS, SW);
                else
                    Text = Text.Remove(SS, SW);
                TextPointer = SS;
            }
            else if (TP < Text.Length)
            {
                if (Password)
                    Text = PasswordText.Remove(TP, 1);                
                else
                    Text = Text.Remove(TP, 1);
            }
            SelStart = 0;
            SelWidth = 0;
        }
        public void BackSpace()
        {
            if (Locked || string.IsNullOrEmpty(text)) return;

            if (SW < 0)
            {
                if (Password)
                    Text = PasswordText.Remove(SS + SW, -SW);
                else
                    Text = Text.Remove(SS + SW, -SW);
            }
            else if (SW > 0)
            {
                if (Password)
                    Text = PasswordText.Remove(SS, SW);
                else
                    Text = Text.Remove(SS, SW);
                TextPointer = SS;
            }
            else if (TP >0)
            {
                TextPointer = TP - 1;
                if (Password)
                    Text = PasswordText.Remove(TP, 1);
                else
                    Text = Text.Remove(TP, 1);
            }
            SelStart = 0;
            SelWidth = 0;
        }
    }
}
