﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using Clash_Library;
using Server_Packets;

namespace Clash_Client
{
    static class In
    {
        public static void Read(Packet P)
        {
            if (P != null)
            try
            {
                typeof(In).GetMethod(P.GetType().Name).Invoke(null, new object[] { P });
            }
            catch (Exception ex)
            {
                File.AppendAllText(@".\Error.txt", string.Format("[{0}] {1}" + Environment.NewLine, Engine.Time, ex));
            }
        }

        public static void Login(Login Packet)
        {
            IntroScenes.LoginDialog.OK.Locked = false;
            switch (Packet.Result)
            {
                case 0:
                    Functions.ShowMessage(Constants.LoginDisabled);
                    break;
                case 1:
                    Functions.ShowMessage(Constants.MissingAccountName);
                    break;
                case 2:
                    Functions.ShowMessage(Constants.MissingPassword);
                    break;
                case 3:
                    Functions.ShowMessage(Constants.WrongIDorPassword);
                    break;
                case 4:
                    Functions.ShowMessage(
                        Constants.AccountBanned + "\n" +
                        "Expiry Date: " + Packet.ExipryDate + "\n" +
                        "Ban Reason: " + Packet.BanReason);
                    break;
                case 5:
                    Functions.ShowMessage(Constants.WrongIDorPassword);
                    break;
                case 6:
                    if (Packet.BadPassCount > 0)
                        Functions.ShowMessage(string.Format(Constants.FailedLoginCount, Packet.BadPassCount));
                    IntroScenes.LoginAccepted();
                    break;
                case 7:
                    Functions.ShowMessage(Constants.Unknown);
                    break;
            }
        }
        public static void NewAccount(NewAccount Packet)
        {
            IntroScenes.AccountDialog.Confirm.Locked = false;
            switch (Packet.Result)
            {
                case 0:
                    Functions.ShowMessage(Constants.CreateAccountDisabled);
                    break;
                case 1:
                    Functions.ShowMessage(Constants.MissingAccountName);
                    break;
                case 2:
                    Functions.ShowMessage(Constants.MissingPassword);
                    break;
                case 3:
                    Functions.ShowMessage(Constants.AccountExists);
                    break;
                case 4:
                    IntroScenes.AccountDialog.Window.CleanUp();
                    IntroScenes.LoginDialog.Window.Visible = true;
                    Functions.ShowMessage(Constants.AccountCreated);
                    break;
                case 5:
                    Functions.ShowMessage(Constants.Unknown);
                    break;
            }
        }
        public static void ChangePass(ChangePass Packet)
        {
            IntroScenes.PasswordDialog.Confirm.Locked = false;
            switch (Packet.Result)
            {
                case 0:
                    Functions.ShowMessage(Constants.ChangePasswordDisabled);
                    break;
                case 1:
                    Functions.ShowMessage(Constants.MissingAccountName);
                    break;
                case 2:
                    Functions.ShowMessage(Constants.MissingPassword);
                    break;
                case 3:
                    Functions.ShowMessage(Constants.MissingNewPassword);
                    break;
                case 4:
                    Functions.ShowMessage(Constants.WrongIDorPassword);
                    break;
                case 5:
                    Functions.ShowMessage(
                        Constants.AccountBanned + "\n" +
                        "Expiry Date: " + Packet.ExipryDate + "\n" +
                        "Ban Reason: " + Packet.BanReason);
                    break;
                case 6:
                    IntroScenes.PasswordDialog.Window.CleanUp();
                    IntroScenes.LoginDialog.Window.Visible = true;
                    Functions.ShowMessage(Constants.PasswordChanged);
                    break;
                case 7:
                    Functions.ShowMessage(Constants.Unknown);
                    break;
            }
        }
        public static void CharacterList(CharacterList Packet)
        {
            Engine.CharacterList = Packet.CharList;
            for (int I = 0; I < Packet.CharList.Count; I++)
            {
                if (Packet.CharList[I].PlayerID == Packet.LastCharID)
                {
                    Engine.CharSelected = I;
                    return;
                }

            }
        }
        public static void NewCharacter(NewCharacter Packet)
        {
            IntroScenes.NewCharacterVars.Confirm.Locked = false;
            switch (Packet.Result)
            {
                case 0:
                    Functions.ShowMessage(Constants.NewCharacterDisabled);
                    break;
                case 1:
                    Functions.ShowMessage(Constants.MissingCharacterName);
                    break;
                case 2:
                    Functions.ShowMessage(Constants.BadCharacterName);
                    break;
                case 3:
                    Functions.ShowMessage(Constants.WrongClass);
                    break;
                case 4:
                    Functions.ShowMessage(Constants.WrongGender);
                    break;
                case 5:
                    Functions.ShowMessage(Constants.AccountBanned);
                    break;
                case 6:
                    Functions.ShowMessage(Constants.MaxCharacters);
                    break;
                case 7:
                    Functions.ShowMessage(Constants.CharacterExists);
                    break;
                case 8:
                    Functions.ShowMessage(Constants.CharacterCreated);
                    if (Engine.CharacterList == null) Engine.CharacterList = new List<CharSelInfo>();
                    Engine.CharacterList.Add(new CharSelInfo(Packet.CharID, IntroScenes.NewCharacterVars.CharacterName.Text, Packet.Level, IntroScenes.NewCharacterVars.SelectedClass, IntroScenes.NewCharacterVars.SelectedSex));
                    IntroScenes.NewCharacterVars.Window.CleanUp();
                    Engine.CharSelected = Engine.CharacterList.Count - 1;
                    break;
                case 9:
                    Functions.ShowMessage(Constants.Unknown);
                    break;
            }
        }
        public static void DeleteCharacter(DeleteCharacter Packet)
        {
            switch (Packet.Result)
            {
                case 0:
                    Functions.ShowMessage(Constants.DelCharacterDisabled);
                    break;
                case 1:
                    Functions.ShowMessage(Constants.AccountBanned);
                    break;
                case 2:
                    Functions.ShowMessage(Constants.CharacterDeleted);
                    IntroScenes.QuestionDialogVars.Window.CleanUp();
                    Engine.CharacterList.RemoveAt(Engine.CharSelected);
                    if (Engine.CharacterList.Count <= Engine.CharSelected) Engine.CharSelected = Engine.CharacterList.Count - 1;
                    break;
                case 3:
                    Functions.ShowMessage(Constants.CharacterNotFound);
                    break;
                case 4:
                    Functions.ShowMessage(Constants.Unknown);
                    break;
            }
        }
        public static void Logout(Logout Packet)
        {
            SoundList.PlaySound(SoundList.LoggingIn, false);
            Engine.CurrentStage = GameStage.LoggingIn;
            IntroScenes.BackGrounds.LoggingOut = true;
            IntroScenes.NewCharacterVars.Window.CleanUp();
            IntroScenes.BackGrounds.LoggingIn.Visible = true;
        }
        public static void StartGame(StartGame Packet)
        {
            switch (Packet.Result)
            {
                case 0:
                    Functions.ShowMessage(Constants.StartGameDisabled);
                    break;
                case 1:
                    Functions.ShowMessage(Constants.AccountBanned);
                    break;
                case 2:
                    Functions.ShowMessage(Constants.CharacterDeleted);
                    break;
                case 3:
                    Functions.ShowMessage(
                        Constants.CharacterBanned + "\n" +
                        "Expiry Date: " + Packet.ExipryDate + "\n" +
                        "Ban Reason: " + Packet.BanReason);
                    break;
                case 4:
                    Engine.CurrentStage = GameStage.Loading;
                    IntroScenes.BackGrounds.Loading.Image = 193 + new Random().Next(8);
                    IntroScenes.BackGrounds.Loading.Visible = true;
                    IntroScenes.NewCharacterVars.Window.CleanUp();
                    ThreadPool.QueueUserWorkItem(Engine.LoadGame);
                    GameScene.Create();
                    break;
                case 5:
                    Functions.ShowMessage(Constants.CharacterNotFound);
                    break;
                case 6:
                    Functions.ShowMessage(Constants.Unknown);
                    break;
            }
        }

        #region User Packets
        public static void UserInfo(UserInfo Packet)
        {
            User.Info = Packet.AI;
            User.Info.Sex = Engine.CharacterList[Engine.CharSelected].Sex;
            User.Info.Job = Engine.CharacterList[Engine.CharSelected].Job;
            User.Info.Level = Engine.CharacterList[Engine.CharSelected].Level;
            User.Info.Name = Engine.CharacterList[Engine.CharSelected].Name;
            User.CalcGender();
            User.DoAction(MirAction.Standing);


            GameScene.Bag.ItemGrid.ItemArray = User.Info.BagItems;
            GameScene.Character.Necklace.ItemArray = User.Info.Equipment;
            GameScene.Character.BraceL.ItemArray = User.Info.Equipment;
            GameScene.Character.BraceR.ItemArray = User.Info.Equipment;
            GameScene.Character.RingL.ItemArray = User.Info.Equipment;
            GameScene.Character.RingR.ItemArray = User.Info.Equipment;
            GameScene.Character.Stone.ItemArray = User.Info.Equipment;
            GameScene.Character.Boots.ItemArray = User.Info.Equipment;
            GameScene.Character.Candle.ItemArray = User.Info.Equipment;
            GameScene.Character.Amulet.ItemArray = User.Info.Equipment;

            GameScene.Character.Armor.ItemArray = User.Info.Equipment;
            GameScene.Character.Weapon.ItemArray = User.Info.Equipment;
            GameScene.Character.Belt.ItemArray = User.Info.Equipment;
            GameScene.Character.Helmet.ItemArray = User.Info.Equipment;
            GameScene.Character.Medal.ItemArray = User.Info.Equipment;
            GameScene.Character.Poison.ItemArray = User.Info.Equipment;

            MapClass.LoadMap();
            Engine.Loaded = true;

            if (Engine.Cleaned)
                Engine.CurrentStage = GameStage.Game;
        }
        public static void Struck(Struck Packet)
        {
            User.Info.CurrentHP = Packet.CurrentHP;
            User.Struck = true;
            User.StruckEle = Packet.Elems;

        }
        public static void MoveFail(MoveFail Packet)
        {
            User.DoAction(MirAction.Standing);
            User.Info.Location = Packet.Location;
            User.Info.Dir = Packet.Dir;
            Engine.OldOffSet = new System.Drawing.Point(-1, -1);
            MapClass.OffSet = System.Drawing.Point.Empty;
        }
        public static void Regen(Regen Packet)
        {
            User.Info.CurrentHP = Packet.CurrentHP;
            User.Info.CurrentMP = Packet.CurrentMP;
        }
        public static void Revive(Revive Packet)
        {
            Engine.OldOffSet = new System.Drawing.Point(-1, -1);
            User.Info.CurrentHP = User.Info.TotalStats.Basic.HP;
            User.CanDoAction = true;
            User.Info.Dead = false;
            User.CanRun = false;
            User.Struck = false;
            User.DoAction(MirAction.Standing);
            User.Info.Location = Packet.Location;
            Monster.TargetMonster = null;
            Player.TargetPlayer = null;
        }
        public static void ChangeMap(ChangeMap Packet)
        {
            User.Info.MapInfo = Packet.Info;
            MapClass.LoadMap();
            User.Info.Location = Packet.Location;
            Engine.OldOffSet = new System.Drawing.Point(-1, -1);
            User.DoAction(MirAction.Standing);
            MapClass.OffSet = System.Drawing.Point.Empty;
            Monster.MonsterList.Clear();
            Monster.HoverMonster = null;
            Monster.TargetMonster = null;
            Player.PlayerList.Clear();
            Player.HoverPlayer = null;
            Player.TargetPlayer = null;
        }
        public static void WinEXP(WinEXP Packet)
        {
            User.Info.Exp += Packet.Exp;
            if (User.Info.Exp >= User.Info.MaxExp)
            {
                User.Info.Exp -= User.Info.MaxExp;
            }
        }
        public static void LevelUp(LevelUp Packet)
        {
            User.Info.Level++;
            User.Info.MaxExp = Packet.MaxExp;
        }
        #endregion
        public static void NewObjects(NewObjects Packet)
        {
            foreach (PlayerStats P in Packet.Characters)
            {
                for (int I = Player.PlayerList.Count - 1; I >= 0; I--)
                    if (Player.PlayerList[I].Info.UniqueID == P.UniqueID)
                        Player.PlayerList[I].Remove();
                Player.PlayerList.Add(new Player(P));
            }

            foreach (MonsterStats M in Packet.Monsters)
            {
                for (int I = Monster.MonsterList.Count - 1; I >= 0; I--)
                    if (Monster.MonsterList[I].Info.MobID == M.MobID)
                        Monster.MonsterList[I].Remove();
                Monster.MonsterList.Add(new Monster(M));
            }
        }
        #region Player Packers
        public static void PlayerNew(PlayerNew Packet)
        {
            for (int I = Player.PlayerList.Count - 1; I >= 0; I--)
            {
                Player P = Player.PlayerList[I];
                if (P.Info.UniqueID == Packet.Info.UniqueID)
                    P.Remove();
            }
            Player.PlayerList.Add(new Player(Packet.Info));
        }
        public static void PlayerRemove(PlayerRemove Packet)
        {
            for (int I = Player.PlayerList.Count - 1; I >= 0; I--)
            {
                Player P = Player.PlayerList[I];
                if (P.Info.UniqueID == Packet.PlayerID)
                    P.Remove();
            }

        }
        public static void PlayerTurn(PlayerTurn Packet)
        {
            for (int I = Player.PlayerList.Count - 1; I >= 0; I--)
            {
                Player P = Player.PlayerList[I];
                if (P.Info.UniqueID == Packet.PlayerID)
                {
                    P.QueueAction(MirAction.Standing, Packet.Dir);
                    return;
                }
            }
        }
        public static void PlayerWalk(PlayerWalk Packet)
        {
            for (int I = Player.PlayerList.Count - 1; I >= 0; I--)
            {
                Player P = Player.PlayerList[I];
                if (P.Info.UniqueID == Packet.PlayerID)
                {
                    P.Walk(Packet.Location);
                    return;
                }
            }
        }
        public static void PlayerRun(PlayerRun Packet)
        {
            for (int I = Player.PlayerList.Count - 1; I >= 0; I--)
            {
                Player P = Player.PlayerList[I];
                if (P.Info.UniqueID == Packet.PlayerID)
                {
                    P.Run(Packet.Location);
                    return;
                }
            }
        }
        public static void PlayerAttack(PlayerAttack Packet)
        {
            for (int I = Player.PlayerList.Count - 1; I >= 0; I--)
            {
                Player P = Player.PlayerList[I];
                if (P.Info.UniqueID == Packet.PlayerID)
                {
                    P.QueueAction(MirAction.Attack1, Packet.Dir);
                    return;
                }
            }
        }
        public static void PlayerStruck(PlayerStruck Packet)
        {
            for (int I = Player.PlayerList.Count - 1; I >= 0; I--)
            {
                Player P = Player.PlayerList[I];
                if (P.Info.UniqueID == Packet.PlayerID)
                {
                    P.Info.CurrentHP = Packet.CurrentHP;
                    P.QueueAction(MirAction.Struck, Packet.Dir, Packet.Elems);
                    return;
                }
            }
        }
        public static void PlayerChanged(PlayerChanged Packet)
        {
            for (int I = Player.PlayerList.Count - 1; I >= 0; I--)
            {
                Player P = Player.PlayerList[I];
                if (P.Info.UniqueID == Packet.PlayerID)
                {
                    P.Info.Weapon = Packet.Weapon;
                    P.Info.Armor = Packet.Armor;
                    P.Info.MaxHP = Packet.MaxHP;
                    P.Info.MaxMP = Packet.MaxMP;
                    P.Info.Hair = Packet.Hair;
                    if (P.Info.Sex != Packet.Sex)
                    {
                        P.Info.Sex = Packet.Sex;
                        P.CalcGender();
                    }
                }
            }

        }
        public static void PlayerRegen(PlayerRegen Packet)
        {
            for (int I = Player.PlayerList.Count - 1; I >= 0; I--)
            {
                Player P = Player.PlayerList[I];
                if (P.Info.UniqueID == Packet.PlayerID)
                {
                    P.Info.CurrentHP = Packet.CurrentHP;
                    P.Info.CurrentMP = Packet.CurrentMP;
                    return;
                }
            }
        }
        #endregion
        #region Monster Packets
        public static void MonsterNew(MonsterNew Packet)
        {
            for (int I = Monster.MonsterList.Count - 1; I >= 0; I--)
            {
                Monster M = Monster.MonsterList[I];
                if (M.Info.MobID == Packet.Info.MobID)
                    M.Remove();
            }
            Monster.MonsterList.Add(new Monster(Packet.Info));
        }
        public static void MonsterRemove(MonsterRemove Packet)
        {
            for (int I = Monster.MonsterList.Count - 1; I >= 0; I--)
            {
                Monster M = Monster.MonsterList[I];
                if (M.Info.MobID == Packet.MonsterID)
                    M.Remove();
            }

        }
        public static void MonsterTurn(MonsterTurn Packet)
        {
            for (int I = Monster.MonsterList.Count - 1; I >= 0; I--)
            {
                Monster M = Monster.MonsterList[I];
                if (M.Info.MobID == Packet.MonsterID)
                {
                    M.QueueAction(MirAction.Standing, Packet.Dir);
                    return;
                }
            }
        }
        public static void MonsterWalk(MonsterWalk Packet)
        {
            for (int I = Monster.MonsterList.Count - 1; I >= 0; I--)
            {
                Monster M = Monster.MonsterList[I];
                if (M.Info.MobID == Packet.MonsterID)
                {
                    M.Walk(Packet.Location);
                    return;
                }
            }
        }
        public static void MonsterAttack(MonsterAttack Packet)
        {
            for (int I = Monster.MonsterList.Count - 1; I >= 0; I--)
            {
                Monster M = Monster.MonsterList[I];
                if (M.Info.MobID == Packet.MonsterID)
                {
                    M.QueueAction(MirAction.Attack1, Packet.Dir);
                    return;
                }
            }
        }
        public static void MonsterStruck(MonsterStruck Packet)
        {
            for (int I = Monster.MonsterList.Count - 1; I >= 0; I--)
            {
                Monster P = Monster.MonsterList[I];
                if (P.Info.MobID == Packet.MonsterID)
                {
                    P.Info.CurrentHP = Packet.CurrentHP;
                    P.QueueAction(MirAction.Struck, Packet.Dir, Packet.Elems);
                    return;
                }
            }
        }
        public static void MonsterRegen(MonsterRegen Packet)
        {
            for (int I = Monster.MonsterList.Count - 1; I >= 0; I--)
            {
                Monster M = Monster.MonsterList[I];
                if (M.Info.MobID == Packet.MonsterID)
                {
                    M.Info.CurrentHP = Packet.CurrentHP;
                    M.Info.CurrentMP = Packet.CurrentMP;
                    return;
                }
            }
        }
        public static void MonsterSpell(MonsterSpell Packet)
        {
            for (int I = Monster.MonsterList.Count - 1; I >= 0; I--)
            {
                Monster M = Monster.MonsterList[I];
                if (M.Info.MobID == Packet.MonsterID)
                {
                    M.QueueAction(MirAction.Spell, Packet.Dir, Packet.SpellID);
                    return;
                }

            }
        }
        #endregion
        #region Items
        public static void ItemList(ItemList Packet)
        {
            foreach (ItemInfo II in Packet.InfoList)
                if (II.Type == ItemType.Armor && Engine.CharacterList[Engine.CharSelected].Sex == Gender.Female)
                    II.Image++;

            Item.IList.AddRange(Packet.InfoList);
        }
        public static void EquipItem(EquipItem Packet)
        {
            Item A;
            if (ItemGrid.PItem != null && ItemGrid.PItem.UniqueID == Packet.ItemID)
            {
                A = ItemGrid.PItem;

                if (User.Info.Equipment[(byte)Packet.Loc] != null)
                    ItemGrid.PItemArray[ItemGrid.PIndex] = User.Info.Equipment[(byte)Packet.Loc];

                User.Info.Equipment[(byte)Packet.Loc] = A;
                ItemGrid.PItem = null;
                ItemGrid.PItemArray = null;
                ItemGrid.PIndex = -1;
                User.Info.MakeTotalStats();
                return;
            }
            else for (int I = User.Info.BagItems.Length - 1; I >= 0; I--)
                {
                    A = User.Info.BagItems[I];
                    if (A != null && A.UniqueID == Packet.ItemID)
                    {
                        User.Info.BagItems[I] = User.Info.Equipment[(byte)Packet.Loc];
                        User.Info.Equipment[(byte)Packet.Loc] = A;
                        User.Info.MakeTotalStats();
                        return;
                    }
                }
        }
        public static void UnEquipItem(UnEquipItem Packet)
        {
            if (ItemGrid.PItem != null && ItemGrid.PItem.UniqueID == Packet.ItemID)
            {
                Functions.AddToBag(ItemGrid.PItem);
                ItemGrid.PItem = null;
                ItemGrid.PItemArray = null;
                ItemGrid.PIndex = -1;
                User.Info.MakeTotalStats();
                return;
            }

            for (int I = 0; I < User.Info.Equipment.Length; I++)
            {
                if (User.Info.Equipment[I] != null && User.Info.Equipment[I].UniqueID == Packet.ItemID)
                {
                    Functions.AddToBag(User.Info.Equipment[I]);
                    User.Info.Equipment[I] = null;
                    User.Info.MakeTotalStats();
                    return;
                }

            }
        }
        #endregion
    }
}
