﻿using System;
using System.IO;
using System.Text.RegularExpressions;
using Clash_Library;
using Client_Packets;

namespace Clash_Client
{
    public static class Out
    {
        #region Intro Section
        public static void Login()
        {
            if (string.IsNullOrEmpty(IntroScenes.LoginDialog.LoginID.Text))
            {
                Functions.ShowMessage(Constants.MissingAccountName);
                IntroScenes.LoginDialog.OK.Locked = false;
                return;
            }
            if (string.IsNullOrEmpty(IntroScenes.LoginDialog.Password.Text))
            {
                Functions.ShowMessage(Constants.MissingPassword);
                IntroScenes.LoginDialog.OK.Locked = false;
                return;
            }

            Login Packet = new Login();
            Packet.AccountName = IntroScenes.LoginDialog.LoginID.Text;
            Packet.Password = IntroScenes.LoginDialog.Password.PasswordText;
            SendPacket(Packet);
        }
        public static void NewAccount()
        {
            if (string.IsNullOrEmpty(IntroScenes.AccountDialog.LoginID.Text))
            {
                Functions.ShowMessage(Constants.MissingAccountName);
                IntroScenes.AccountDialog.Confirm.Locked = false;
                return;
            }
            if (string.IsNullOrEmpty(IntroScenes.AccountDialog.Password.Text))
            {
                Functions.ShowMessage(Constants.MissingPassword);
                IntroScenes.AccountDialog.Confirm.Locked = false;
                return;
            }
            if (string.IsNullOrEmpty(IntroScenes.AccountDialog.Password2.Text))
            {
                Functions.ShowMessage(Constants.MissingPassword2);
                IntroScenes.AccountDialog.Confirm.Locked = false;
                return;
            }
            if (IntroScenes.AccountDialog.Password.Text != IntroScenes.AccountDialog.Password2.Text)
            {
                Functions.ShowMessage(Constants.PasswordMisMatch);
                IntroScenes.AccountDialog.Confirm.Locked = false;
                return;
            }


            NewAccount Packet = new NewAccount();
            Packet.AccountID = IntroScenes.AccountDialog.LoginID.Text;
            Packet.Password = IntroScenes.AccountDialog.Password.PasswordText;
            Packet.Name = IntroScenes.AccountDialog.Name.Text;
            Packet.DateOfBirth = IntroScenes.AccountDialog.DateOfBirth.Text;
            Packet.Question1 = IntroScenes.AccountDialog.Question1.Text;
            Packet.Answer1 = IntroScenes.AccountDialog.Answer1.Text;
            Packet.Question2 = IntroScenes.AccountDialog.Question2.Text;
            Packet.Answer2 = IntroScenes.AccountDialog.Answer2.Text;
            Packet.Phone = IntroScenes.AccountDialog.Phone.Text;
            Packet.Mobile = IntroScenes.AccountDialog.Mobile.Text;
            Packet.Email = IntroScenes.AccountDialog.Email.Text;
            SendPacket(Packet);
        }
        public static void ChangePassword()
        {
            if (string.IsNullOrEmpty(IntroScenes.PasswordDialog.LoginID.Text))
            {
                Functions.ShowMessage(Constants.MissingAccountName);
                IntroScenes.PasswordDialog.Confirm.Locked = false;
                return;
            }
            if (string.IsNullOrEmpty(IntroScenes.PasswordDialog.Password.Text))
            {
                Functions.ShowMessage(Constants.MissingPassword);
                IntroScenes.PasswordDialog.Confirm.Locked = false;
                return;
            }
            if (string.IsNullOrEmpty(IntroScenes.PasswordDialog.NewPassword.Text))
            {
                Functions.ShowMessage(Constants.MissingNewPassword);
                IntroScenes.PasswordDialog.Confirm.Locked = false;
                return;
            }
            if (string.IsNullOrEmpty(IntroScenes.PasswordDialog.NewPassword2.Text))
            {
                Functions.ShowMessage(Constants.MissingNewPassword2);
                IntroScenes.PasswordDialog.Confirm.Locked = false;
                return;
            }
            if (IntroScenes.PasswordDialog.NewPassword.Text != IntroScenes.PasswordDialog.NewPassword2.Text)
            {
                Functions.ShowMessage(Constants.PasswordMisMatch);
                IntroScenes.PasswordDialog.Confirm.Locked = false;
                return;
            }

            ChangePass Packet = new ChangePass();
            Packet.AccountID = IntroScenes.PasswordDialog.LoginID.Text;
            Packet.CPassword = IntroScenes.PasswordDialog.Password.PasswordText;
            Packet.NPassword = IntroScenes.PasswordDialog.NewPassword.PasswordText;
            SendPacket(Packet);
        }
        public static void NewCharacter()
        {
            if (string.IsNullOrEmpty(IntroScenes.NewCharacterVars.CharacterName.Text))
            {
                Functions.ShowMessage(Constants.MissingCharacterName);
                return;
            }
            Regex r = new Regex(@"^[A-Za-z0-9]{3,15}$");
            if (!r.IsMatch(IntroScenes.NewCharacterVars.CharacterName.Text))
            {
                Functions.ShowMessage(Constants.BadCharacterName);
                return;
            }

            NewCharacter Packet = new NewCharacter();
            Packet.Name = IntroScenes.NewCharacterVars.CharacterName.Text;
            Packet.Job = IntroScenes.NewCharacterVars.SelectedClass;
            Packet.Sex = IntroScenes.NewCharacterVars.SelectedSex;
            SendPacket(Packet);
        }
        public static void DeleteCharacter()
        {
            if (Engine.CharacterList == null || Engine.CharacterList.Count == 0) return;
            if (Engine.CharSelected < 0 || Engine.CharSelected >= Engine.CharacterList.Count) return;

            DeleteCharacter Packet = new DeleteCharacter();
            Packet.CharID = Engine.CharacterList[Engine.CharSelected].PlayerID;
            SendPacket(Packet);
        }
        public static void Logout()
        {
            SendPacket(new Logout());
        }
        public static void StartGame()
        {
            if (Engine.CharacterList == null || Engine.CharacterList.Count == 0) return;
            if (Engine.CharSelected < 0 || Engine.CharSelected >= Engine.CharacterList.Count) return;

            StartGame Packet = new StartGame();
            Packet.CharID = Engine.CharacterList[Engine.CharSelected].PlayerID;
            SendPacket(Packet);

        }
        #endregion

        public static void Turn()
        {
            SendPacket(new Turn(User.Info.Dir));
        }
        public static void Walk()
        {
            SendPacket(new Walk(User.Info.Dir));
        }
        public static void Run()
        {
            SendPacket(new Run(User.Info.Dir));
        }
        public static void Harvest()
        {
            SendPacket(new Harvest(User.Info.Dir));
        }
        public static void Attack()
        {
            SendPacket(new Attack(User.Info.Dir));
        }
        public static void Revive()
        {
            SendPacket(new Revive());
        }

        public static void ChangeBagSlot(byte S, byte D)
        {
            SendPacket(new ChangeBagSlot(S, D));
        }
        public static void EquipItem(ItemLocation Loc, int ID)
        {
            SendPacket(new EquipItem(Loc, ID));
        }
        public static void UnEquipItem(int ID)
        {
            SendPacket(new UnEquipItem(ID));
        }

        public static void KeepAlive()
        {
            SendPacket(new KAlive());
        }
        public static void Disconnect()
        {
            try
            {
                if (Network.client == null || !Network.client.Connected) return;
                lock (Network.MStream)
                {
                    Network.MStream.SetLength(0);
                    Network.BWriter.Write("Disconnect");
                    new Disconnect().WriteTo(Network.BWriter);
                    Network.MStream.WriteTo(Network.NStream);
                }
            }
            catch (Exception ex)
            {
                File.AppendAllText(@".\Error.txt", string.Format("[{0}] {1}" + Environment.NewLine, Engine.Time, ex));
            }

        }
        public static void SendPacket(Packet O)
        {
            try
            {
                Network.LastPacket = Engine.Time;
                if (Network.MStream != null)
                {
                    lock (Network.MStream)
                    {
                        Network.BWriter.Write(O.GetType().Name);
                        O.WriteTo(Network.BWriter);
                    }
                }
            }
            catch (Exception ex)
            {
                File.AppendAllText(@".\Error.txt", string.Format("[{0}] {1}" + Environment.NewLine, Engine.Time, ex));
            }

        }        
    }
}
