﻿using System;
using System.Collections.Generic;
using System.Drawing;
using Clash_Library;
using Microsoft.DirectX;

namespace Clash_Client
{
    class Monster
    {
        public static Monster HoverMonster, TargetMonster;
        public static List<Monster> MonsterList = new List<Monster>();

        public MonsterStats Info;
        public MirAction CurrentAction;
        public bool CanDoAction, FreezeEnding;
        public DateTime LastFrame;
        public TimeSpan FrameTime;
        public int CurrentFrame, DrawFrame, FrameCount, SkipFrame, StartFrame = 0;
        public MonsterFrame Frame;
        public Point ActualPoint, FinalDrawPoint;
        public bool Visible;
        
        public Queue<MirAction> ActionList = new Queue<MirAction>();
        public Queue<Direction> DirectionList = new Queue<Direction>();
        public Queue<Element> ElementList = new Queue<Element>();
        public Queue<Magic> MagicList = new Queue<Magic>();

        public Monster(MonsterStats MInfo)
        {
            Info = MInfo;
            if (Info.Image == 0) Info.Image++;

            Visible = true;
            ActualPoint = Info.Location;
            MapClass.AddObject(this, Info.Location);

            switch (MInfo.Image)
            {
                case 108:
                    Frame = MonsterFrame.Frames[1];
                    break;
                default:
                    Frame = MonsterFrame.Frames[0];
                    break;
            }

            if (Info.Dead || Info.CurrentHP == 0)
            {
                DoAction(MirAction.Dead);
                CurrentFrame = Frame.Death.Count - 1;
            }
            else
                DoAction(MirAction.Standing);
        }

        public bool CanNextAction()
        {
            return CanDoAction && !Info.Dead;
        }

        public void TryStartAction()
        {
            if (CanDoAction)
            {
                if (ActionList.Count > 0)
                {
                    if (ActionList.Peek() == MirAction.Walking || ActionList.Peek() == MirAction.Running)
                        if (!Engine.CanMove) return;
                    Info.Dir = DirectionList.Dequeue();
                    DoAction(ActionList.Dequeue());
                    switch (CurrentAction)
                    {
                        case MirAction.Walking:
                            MapClass.RemoveObject(this, Info.Location);
                            Info.Location = Functions.Move(Info.Location, Info.Dir, 1);
                            MapClass.AddObject(this, Info.Location);
                            break;
                        case MirAction.Running:
                            MapClass.RemoveObject(this, Info.Location);
                            Info.Location = Functions.Move(Info.Location, Info.Dir, 2);
                            MapClass.AddObject(this, Info.Location);
                            break;
                    }
                }
                else if (CurrentAction != MirAction.Standing)
                    DoAction(MirAction.Standing);
            }
        }

        public void QueueAction(MirAction A, Direction D, Element E)
        {
            ActionList.Enqueue(A);
            DirectionList.Enqueue(D);
            ElementList.Enqueue(E);
        }
        public void QueueAction(MirAction A, Direction D, int SpellID)
        {
            ActionList.Enqueue(A);
            DirectionList.Enqueue(D);
            MagicList.Enqueue(Magic.MakeMagic(ActualPoint, SpellID));
        }
        public void QueueAction(MirAction A, Direction D)
        {
            ActionList.Enqueue(A);
            DirectionList.Enqueue(D);
        }

        public void Walk(Point P)
        {
            if (ActualPoint == P) return;
            QueueAction(MirAction.Walking, Functions.GetDirection(ActualPoint, P));
            ActualPoint = P;
        }
        public void DoAction(MirAction A)
        {
            if (A != MirAction.Standing)
                CanDoAction = false;
            CurrentAction = A;
            FreezeEnding = true;
            CurrentFrame = 0;
            switch (A)
            {
                case MirAction.Standing:
                    StartFrame = Frame.Stand.Start;
                    FrameCount = Frame.Stand.Count;
                    SkipFrame = Frame.Stand.Skip;
                    FrameTime = Frame.Stand.Time;
                    break;
                case MirAction.Walking:
                    StartFrame = Frame.Walk.Start;
                    FrameCount = Frame.Walk.Count;
                    SkipFrame = Frame.Walk.Skip;
                    FrameTime = Frame.Walk.Time;
                    break;
                case MirAction.Attack1:
                    StartFrame = Frame.Attack.Start;
                    FrameCount = Frame.Attack.Count;
                    SkipFrame = Frame.Attack.Skip;
                    FrameTime = Frame.Attack.Time;
                    break;
                case MirAction.Spell:
                    if (MagicList.Count > 0)
                        Magic.MagicList.Add(MagicList.Dequeue());
                    StartFrame = Frame.Spell.Start;
                    FrameCount = Frame.Spell.Count;
                    SkipFrame = Frame.Spell.Skip;
                    FrameTime = Frame.Spell.Time;
                    break;
                case MirAction.Struck:
                    if (ElementList.Count > 0)
                        Magic.MakeMagic(Info.Location, ElementList.Dequeue());
                    StartFrame = Frame.Struck.Start;
                    FrameCount = Frame.Struck.Count;
                    SkipFrame = Frame.Struck.Skip;
                    FrameTime = Frame.Struck.Time;
                    break;
                case MirAction.Dieing:
                    StartFrame = Frame.Die.Start;
                    FrameCount = Frame.Die.Count;
                    SkipFrame = Frame.Die.Skip;
                    FrameTime = Frame.Die.Time;
                    break;
                case MirAction.Dead:
                    MonsterList.Remove(this);
                    MonsterList.Add(this);
                    StartFrame = Frame.Death.Start;
                    FrameCount = Frame.Death.Count;
                    SkipFrame = Frame.Death.Skip;
                    FrameTime = Frame.Death.Time;
                    break;
            }
        }
        public void Operate()
        {
            switch (CurrentAction)
            {
                case MirAction.Standing:
                    if (Engine.Time > LastFrame + FrameTime)
                    {
                        LastFrame = Engine.Time;
                        if (CurrentFrame > 0) CanDoAction = true;

                        if (++CurrentFrame >= FrameCount)
                            CurrentFrame = 0;

                    }
                    break;
                case MirAction.Walking:
                case MirAction.Running:
                    if (Engine.CanMove)
                    {
                        if (Engine.Time - LastFrame > FrameTime)
                        {
                            LastFrame = Engine.Time;
                            if (++CurrentFrame >= FrameCount)
                            {
                                CanDoAction = true;
                                if (!FreezeEnding)
                                {
                                    CurrentFrame = 0;
                                    CanDoAction = true;
                                    DoAction(MirAction.Standing);
                                }
                                else
                                {
                                    CurrentFrame = FrameCount - 1;
                                    FreezeEnding = false;
                                }
                            }

                        }
                    }
                    break;
                case MirAction.BattleStance:
                    CanDoAction = true;
                    if (Engine.Time - LastFrame > FrameTime)
                    {
                        LastFrame = Engine.Time;
                        DoAction(MirAction.Standing);
                    }
                    break;
                case MirAction.Attack1:
                case MirAction.Attack2:
                case MirAction.Attack3:
                case MirAction.Spell: //6 Frames
                    if (Engine.Time - LastFrame > FrameTime)
                    {
                        LastFrame = Engine.Time;
                        if (++CurrentFrame >= FrameCount)
                        {
                            CurrentFrame = 0;
                            CanDoAction = true;
                            DoAction(MirAction.BattleStance);
                        }
                    }
                    break;
                case MirAction.Harvest: //6 Frames
                    if (Engine.Time - LastFrame > FrameTime)
                    {
                        LastFrame = Engine.Time;
                        if (++CurrentFrame >= FrameCount)
                        {
                            CurrentFrame = 0;
                            CanDoAction = true;
                            DoAction(MirAction.Standing);
                        }
                    }
                    break;
                case MirAction.Struck: //6 Frames
                    if (Engine.Time - LastFrame > FrameTime)
                    {
                        LastFrame = Engine.Time;
                        if (++CurrentFrame >= FrameCount)
                        {
                            if (Info.CurrentHP <= 0)
                            {
                                CanDoAction = false;
                                Info.Dead = true;
                                DoAction(MirAction.Dieing);
                                return;
                            }
                            CurrentFrame = 0;
                            CanDoAction = true;
                            DoAction(MirAction.Standing);
                        }
                    }
                    break;
                case MirAction.Dieing:
                    if (Engine.Time - LastFrame > FrameTime)
                    {
                        LastFrame = Engine.Time;
                        if (++CurrentFrame >= FrameCount)
                            DoAction(MirAction.Dead);
                    }
                    break;
                case MirAction.Dead:
                    if (Engine.Time - LastFrame > FrameTime)
                    {
                        LastFrame = Engine.Time;
                        if (++CurrentFrame >= FrameCount)
                        {
                            CurrentFrame--;
                            FrameTime = TimeSpan.MaxValue;
                        }
                    }
                    break;
            }
        }
        public Point MovementOffSet()
        {
            Point mapOffSet = Point.Empty;
            if (CurrentAction == MirAction.Walking || CurrentAction == MirAction.Running)
            {
                int multi = Convert.ToByte((CurrentAction == MirAction.Running)) + 1;
                switch (Info.Dir)
                {
                    case Direction.Down:
                        mapOffSet = new Point(0, (-32 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1));
                        break;
                    case Direction.Down_Left:
                        mapOffSet = new Point((48 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1), (-32 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1));
                        break;
                    case Direction.Left:
                        mapOffSet = new Point((48 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1), 0);
                        break;
                    case Direction.Up_Left:
                        mapOffSet = new Point((48 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1), (32 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1));
                        break;
                    case Direction.Up:
                        mapOffSet = new Point(0, (32 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1));
                        break;
                    case Direction.Up_Right:
                        mapOffSet = new Point((-48 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1), (32 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1));
                        break;
                    case Direction.Right:
                        mapOffSet = new Point((-48 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1), 0);
                        break;
                    case Direction.Down_Right:
                        mapOffSet = new Point((-48 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1), (-32 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1));
                        break;
                }
            }
            return mapOffSet;
        }
        public void Draw()
        {
            if (!Visible) return;
            DrawFrame = StartFrame + (byte)Info.Dir * (FrameCount + SkipFrame) + CurrentFrame;

            Point TPoint = Info.Location;
            TPoint.Offset(-User.Info.Location.X, -User.Info.Location.Y);
            TPoint.Offset(Config.PlayerOffSet);
            TPoint.X *= 48; TPoint.Y *= 32;
            TPoint.Offset(MapClass.OffSet);
            TPoint.Offset(MovementOffSet());
            FinalDrawPoint = TPoint;

            if (CLib.Monsters != null && Info.Image >= 0 && Info.Image < CLib.Monsters.Length)
            {
                CLib.Monsters[Info.Image].Draw(DrawFrame, TPoint, true);
                FinalDrawPoint.Offset(CLib.Monsters[Info.Image].GetImageOffSet(DrawFrame));
            }
        }
        public void DrawHover()
        {
            if (!Visible) return;
            Graphics.SetBlend(true);
            Draw();
            Graphics.SetBlend(false);

            Point TPoint = Point.Empty;
            TPoint.Y = (Info.Location.Y - User.Info.Location.Y + Config.PlayerOffSet.Y) * 32 + MapClass.OffSet.Y + MovementOffSet().Y -20;
            TPoint.X = (Info.Location.X - User.Info.Location.X + Config.PlayerOffSet.X) * 48 + MapClass.OffSet.X + MovementOffSet().X;

            int Width;
            if (Info.Dead) TPoint.Y += 20;
            TPoint.X += 24 - Functions.GetStringSize(Info.Name).Width / 2;
            TPoint.Offset(-1, 0);
            Graphics.DXNormalFont.DrawText(Graphics.S, Info.Name, TPoint, Color.Black);
            TPoint.Offset(0, -1);
            Graphics.DXNormalFont.DrawText(Graphics.S, Info.Name, TPoint, Color.Black);
            TPoint.Offset(2, 0);
            Graphics.DXNormalFont.DrawText(Graphics.S, Info.Name, TPoint, Color.Black);
            TPoint.Offset(0, 2);
            Graphics.DXNormalFont.DrawText(Graphics.S, Info.Name, TPoint, Color.Black);
            TPoint.Offset(-1, -1);
            if (Info.Dead)
                Graphics.DXNormalFont.DrawText(Graphics.S, Info.Name, TPoint, Color.Gray);
            else
                Graphics.DXNormalFont.DrawText(Graphics.S, Info.Name, TPoint, Info.NameColour);

            TPoint.Y -= 30;
            TPoint.X -= 24 - Functions.GetStringSize(Info.Name).Width / 2;

            Graphics.S.Flush();
            if (Info.Dead) return;
            if (Info.CurrentMP > 0 && Info.MaxMP > 0)
            {
                TPoint.Offset(1, 0);
                if (Config.DrawMethod2)
                    Graphics.S.Draw2D(Graphics.GTexture, new Rectangle(0, 0, 46, 5), Rectangle.Empty, TPoint, Color.Black.ToArgb());
                else
                    Graphics.S.Draw(Graphics.GTexture, new Rectangle(0, 0, 46, 5), Vector3.Empty, new Vector3(TPoint.X, TPoint.Y, 0), Color.Black.ToArgb());
                TPoint.Offset(1, 1);
                Width = (int)(44 * Info.CurrentMP / (float)Info.MaxMP);
                if (Width > 44) Width = 4;
                if (Width < 0) Width = 0;
                if (Config.DrawMethod2)
                    Graphics.S.Draw2D(Graphics.GTexture, new Rectangle(0, 0, Width, 3), Rectangle.Empty, TPoint, Color.Blue.ToArgb());
                else
                    Graphics.S.Draw(Graphics.GTexture, new Rectangle(0, 0, Width, 3), Vector3.Empty, new Vector3(TPoint.X, TPoint.Y, 0), Color.Blue.ToArgb());
                TPoint.Offset(-2, -7);
            }
            if (Info.CurrentHP > 0 && Info.MaxHP > 0)
            {
                TPoint.Offset(1, 0);
                if (Config.DrawMethod2)
                    Graphics.S.Draw2D(Graphics.GTexture, new Rectangle(0, 0, 46, 5), Rectangle.Empty, TPoint, Color.Black.ToArgb());
                else
                    Graphics.S.Draw(Graphics.GTexture, new Rectangle(0, 0, 46, 5), Vector3.Empty, new Vector3(TPoint.X, TPoint.Y, 0), Color.Black.ToArgb());
                TPoint.Offset(1, 1);
                Width = (int)(44 * Info.CurrentHP / (float)Info.MaxHP);
                if (Width > 44) Width = 44;
                if (Width < 0) Width = 0;
                if (Config.DrawMethod2)
                    Graphics.S.Draw2D(Graphics.GTexture, new Rectangle(0, 0, Width, 3), Rectangle.Empty, TPoint, Color.Red.ToArgb());
                else
                    Graphics.S.Draw(Graphics.GTexture, new Rectangle(0, 0, Width, 3), Vector3.Empty, new Vector3(TPoint.X, TPoint.Y, 0), Color.Red.ToArgb());
            }


                
        }
        public bool MouseOver(Point MousePoint)
        {
            if (CLib.Monsters != null && Info.Image >= 0 && Info.Image < CLib.Monsters.Length)
            {
                MousePoint.Offset(-FinalDrawPoint.X, -FinalDrawPoint.Y);
                if (new Rectangle(Point.Empty, CLib.Monsters[Info.Image].GetImageSize(DrawFrame)).Contains(MousePoint))
                    return CLib.Monsters[Info.Image].GetTransparent(DrawFrame, MousePoint);

            }
            return false;
        }

        public void Remove()
        {
            DirectionList.Clear();
            ElementList.Clear();
            DirectionList.Clear();
            DoAction(MirAction.Standing);
            MapClass.RemoveObject(this, Info.Location);
            MonsterList.Remove(this);
            Visible = false;
        }
    }
}
