﻿using System;
using System.Collections.Generic;
using System.Drawing;
using Clash_Library;
using Microsoft.DirectX;

namespace Clash_Client
{
    class Player
    {
        public static Player HoverPlayer, TargetPlayer;
        public static List<Player> PlayerList = new List<Player>();
        public PlayerStats Info;
        public MirAction CurrentAction;
        public bool CanDoAction, FreezeEnding;
        public DateTime LastFrame;
        public TimeSpan FrameTime;
        public int CurrentFrame, DrawFrame, FrameCount, SkipFrame, StartFrame = 0;
        public Queue<MirAction> ActionList = new Queue<MirAction>();
        public Queue<Direction> DirectionList = new Queue<Direction>();
        public Queue<Element> ElementList = new Queue<Element>();
        public Point ActualPoint, FinalDrawPoint;
        public bool Visible;
        private int WOffSet, AOffSet;

        public Player(PlayerStats CInfo)
        {
            Info = CInfo;
            ActualPoint = Info.Location;
            Visible = true;
            MapClass.AddObject(this, Info.Location);
            CalcGender();
            if (Info.Dead || Info.CurrentHP == 0)
                DoAction(MirAction.Dead);
            else
                DoAction(MirAction.Standing);
        }

        public bool CanNextAction()
        {
            return CanDoAction && !Info.Dead;
        }
        public void CalcGender()
        {
            if (Info.Job == Class.Assassin)
            {
                AOffSet = ((byte)Info.Sex * 512);
                WOffSet = ((byte)Info.Sex * 512);
            }
            else
            {
                AOffSet = ((byte)Info.Sex * 808);
                WOffSet = ((byte)Info.Sex * 416);
            }
        }
        public void TryStartAction()
        {
            if (CanDoAction)
            {
                if (ActionList.Count > 0)
                {
                    if (ActionList.Peek() == MirAction.Walking || ActionList.Peek() == MirAction.Running)
                        if (!Engine.CanMove) return;
                    Info.Dir = DirectionList.Dequeue();
                    DoAction(ActionList.Dequeue());
                    switch (CurrentAction)
                    {
                        case MirAction.Walking:
                            MapClass.RemoveObject(this, Info.Location);
                            Info.Location = Functions.Move(Info.Location, Info.Dir, 1);
                            MapClass.AddObject(this, Info.Location);
                            break;
                        case MirAction.Running:
                            MapClass.RemoveObject(this, Info.Location);
                            Info.Location = Functions.Move(Info.Location, Info.Dir, 2);
                            MapClass.AddObject(this, Info.Location);
                            break;
                    }
                }
                else if (FreezeEnding)
                    DoAction(MirAction.Standing);


            }
        }

        public void QueueAction(MirAction A, Direction D, Element E)
        {
            ActionList.Enqueue(A);
            DirectionList.Enqueue(D);
            ElementList.Enqueue(E);
        }
        public void QueueAction(MirAction A, Direction D)
        {
            ActionList.Enqueue(A);
            DirectionList.Enqueue(D);
        }

        public void Walk(Point P)
        {
            if (ActualPoint == P) return;
            QueueAction(MirAction.Walking, Functions.GetDirection(ActualPoint, P));
            ActualPoint = P;
        }
        public void Run(Point P)
        {
            if (ActualPoint == P) return;
            QueueAction(MirAction.Running, Functions.GetDirection(ActualPoint, P));
            ActualPoint = P;
        }
        public void DoAction(MirAction A)
        {
            if (A != MirAction.Standing)
                CanDoAction = false;
            CurrentAction = A;
            FreezeEnding = true;
            CurrentFrame = 0;
            switch (A)
            {
                case MirAction.Standing:
                    StartFrame = Human.Stand.Start;
                    FrameCount = Human.Stand.Count;
                    SkipFrame = Human.Stand.Skip;
                    FrameTime = Human.Stand.Time;
                    break;
                case MirAction.Walking:
                    StartFrame = Human.Walk.Start;
                    FrameCount = Human.Walk.Count;
                    SkipFrame = Human.Walk.Skip;
                    FrameTime = Human.Walk.Time;
                    break;
                case MirAction.Running:
                    StartFrame = Human.Run.Start;
                    FrameCount = Human.Run.Count;
                    SkipFrame = Human.Run.Skip;
                    FrameTime = Human.Run.Time;
                    break;
                case MirAction.Attack1:
                    StartFrame = Human.Attack1.Start;
                    FrameCount = Human.Attack1.Count;
                    SkipFrame = Human.Attack1.Skip;
                    FrameTime = Human.Attack1.Time;
                    break;
                case MirAction.Attack2:
                    StartFrame = Human.Attack2.Start;
                    FrameCount = Human.Attack2.Count;
                    SkipFrame = Human.Attack2.Skip;
                    FrameTime = Human.Attack2.Time;
                    break;
                case MirAction.Attack3:
                    StartFrame = Human.Attack3.Start;
                    FrameCount = Human.Attack3.Count;
                    SkipFrame = Human.Attack3.Skip;
                    FrameTime = Human.Attack3.Time;
                    break;
                case MirAction.Harvest:
                    StartFrame = Human.Harvest.Start;
                    FrameCount = Human.Harvest.Count;
                    SkipFrame = Human.Harvest.Skip;
                    FrameTime = Human.Harvest.Time;
                    break;
                case MirAction.BattleStance:
                    StartFrame = Human.BattleStance.Start;
                    FrameCount = Human.BattleStance.Count;
                    SkipFrame = Human.BattleStance.Skip;
                    FrameTime = Human.BattleStance.Time;
                    break;
                case MirAction.Spell:
                    StartFrame = Human.Spell.Start;
                    FrameCount = Human.Spell.Count;
                    SkipFrame = Human.Spell.Skip;
                    FrameTime = Human.Spell.Time;
                    break;
                case MirAction.Struck:
                    if (ElementList.Count > 0)
                        Magic.MakeMagic(Info.Location, ElementList.Dequeue());
                    StartFrame = Human.Struck.Start;
                    FrameCount = Human.Struck.Count;
                    SkipFrame = Human.Struck.Skip;
                    FrameTime = Human.Struck.Time;
                    break;
                case MirAction.Dieing:
                    StartFrame = Human.Die.Start;
                    FrameCount = Human.Die.Count;
                    SkipFrame = Human.Die.Skip;
                    FrameTime = Human.Die.Time;
                    break;
                case MirAction.Dead:
                    StartFrame = Human.Die.Start + Human.Die.Count - 1;
                    FrameCount = 1;
                    SkipFrame = Human.Die.Count - 1;
                    FrameTime = Human.Die.Time;
                    break;

            }
        }
        public void Operate()
        {
            switch (CurrentAction)
            {
                case MirAction.Standing:
                    if (Engine.Time > LastFrame + FrameTime)
                    {
                        LastFrame = Engine.Time;
                        if (CurrentFrame > 0) CanDoAction = true;

                        if (++CurrentFrame >= FrameCount)
                            CurrentFrame = 0;

                    }
                    break;
                case MirAction.Walking:
                case MirAction.Running:
                    if (Engine.CanMove)
                    {
                        if (Engine.Time - LastFrame > FrameTime)
                        {
                            LastFrame = Engine.Time;
                            if (++CurrentFrame >= FrameCount)
                            {
                                if (!FreezeEnding)
                                {
                                    CurrentFrame = 0;
                                    CanDoAction = true;
                                    DoAction(MirAction.Standing);
                                }
                                else
                                {
                                    CurrentFrame = FrameCount - 1;
                                    CanDoAction = true;
                                    FreezeEnding = false;
                                }
                            }

                        }
                    }
                    break;
                case MirAction.BattleStance:
                    CanDoAction = true;
                    if (Engine.Time - LastFrame > FrameTime)
                    {
                        LastFrame = Engine.Time;
                        DoAction(MirAction.Standing);
                    }
                    break;
                case MirAction.Attack1:
                case MirAction.Attack2:
                case MirAction.Attack3:
                case MirAction.Spell: //6 Frames
                    if (Engine.Time - LastFrame > FrameTime)
                    {
                        LastFrame = Engine.Time;
                        if (++CurrentFrame >= FrameCount)
                        {
                            CurrentFrame = 0;
                            CanDoAction = true;
                            DoAction(MirAction.BattleStance);
                        }
                    }
                    break;
                case MirAction.Harvest: //6 Frames
                    if (Engine.Time - LastFrame > FrameTime)
                    {
                        LastFrame = Engine.Time;
                        if (++CurrentFrame >= FrameCount)
                        {                            
                            CurrentFrame = 0;
                            CanDoAction = true;
                            DoAction(MirAction.Standing);
                        }
                    }
                    break;
                case MirAction.Struck: //6 Frames
                    if (Engine.Time - LastFrame > FrameTime)
                    {
                        LastFrame = Engine.Time;
                        if (++CurrentFrame >= FrameCount)
                        {
                            if (Info.CurrentHP <= 0)
                            {
                                CanDoAction = false;
                                Info.Dead = true;
                                DoAction(MirAction.Dieing);
                                return;
                            }
                            CurrentFrame = 0;
                            CanDoAction = true;
                            DoAction(MirAction.Standing);
                        }
                    }
                    break;
                case MirAction.Dieing:
                    if (Engine.Time - LastFrame > FrameTime)
                    {
                        LastFrame = Engine.Time;
                        if (++CurrentFrame >= FrameCount)
                            DoAction(MirAction.Dead);
                    }
                    break;
            }
        }

        public Point MovementOffSet()
        {
            Point mapOffSet = Point.Empty;
            if (CurrentAction == MirAction.Walking || CurrentAction == MirAction.Running)
            {
                int multi = Convert.ToByte((CurrentAction == MirAction.Running)) + 1;
                switch (Info.Dir)
                {
                    case Direction.Down:
                        mapOffSet = new Point(0, (-32 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1));
                        break;
                    case Direction.Down_Left:
                        mapOffSet = new Point((48 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1), (-32 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1));
                        break;
                    case Direction.Left:
                        mapOffSet = new Point((48 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1), 0);
                        break;
                    case Direction.Up_Left:
                        mapOffSet = new Point((48 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1), (32 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1));
                        break;
                    case Direction.Up:
                        mapOffSet = new Point(0, (32 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1));
                        break;
                    case Direction.Up_Right:
                        mapOffSet = new Point((-48 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1), (32 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1));
                        break;
                    case Direction.Right:
                        mapOffSet = new Point((-48 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1), 0);
                        break;
                    case Direction.Down_Right:
                        mapOffSet = new Point((-48 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1), (-32 * multi / (FrameCount)) * (FrameCount - CurrentFrame - 1));
                        break;
                }
            }
            return mapOffSet;
        }
        public void Draw()
        {
            if (!Visible) return;
            DrawFrame = StartFrame + (byte)Info.Dir * (FrameCount + SkipFrame) + CurrentFrame;

            Point TPoint = Info.Location;
            TPoint.Offset(-User.Info.Location.X, -User.Info.Location.Y);
            TPoint.Offset(Config.PlayerOffSet);
            TPoint.X *= 48; TPoint.Y *= 32;
            TPoint.Offset(MapClass.OffSet);
            TPoint.Offset(MovementOffSet());
            if (Info.Job == Class.Assassin) DrawA(TPoint);
            else DrawWWT(TPoint);
        }
        public void DrawHover()
        {
            if (!Visible) return;
            Graphics.SetBlend(true);
            Draw();
            Graphics.SetBlend(false);

            Point TPoint = Point.Empty;
            TPoint.Y = (Info.Location.Y - User.Info.Location.Y + Config.PlayerOffSet.Y) * 32 + MapClass.OffSet.Y + MovementOffSet().Y - 20;
            TPoint.X = (Info.Location.X - User.Info.Location.X + Config.PlayerOffSet.X) * 48 + MapClass.OffSet.X + MovementOffSet().X;

            int Width;
            if (Info.Dead) TPoint.Y += 20;
            TPoint.X += 24 - Functions.GetStringSize(Info.Name).Width / 2;
            TPoint.Offset(-1, 0);
            Graphics.DXNormalFont.DrawText(Graphics.S, Info.Name, TPoint, Color.Black);
            TPoint.Offset(0, -1);
            Graphics.DXNormalFont.DrawText(Graphics.S, Info.Name, TPoint, Color.Black);
            TPoint.Offset(2, 0);
            Graphics.DXNormalFont.DrawText(Graphics.S, Info.Name, TPoint, Color.Black);
            TPoint.Offset(0, 2);
            Graphics.DXNormalFont.DrawText(Graphics.S, Info.Name, TPoint, Color.Black);
            TPoint.Offset(-1, -1);
            if (Info.Dead)
                Graphics.DXNormalFont.DrawText(Graphics.S, Info.Name, TPoint, Color.Gray);
            else
                Graphics.DXNormalFont.DrawText(Graphics.S, Info.Name, TPoint, Info.NameColour);

            TPoint.Y -= 30;
            TPoint.X -= 24 - Functions.GetStringSize(Info.Name).Width / 2;

            Graphics.S.Flush();
            if (Info.Dead) return;
            if (Info.CurrentMP > 0 && Info.MaxMP > 0)
            {
                TPoint.Offset(1, 0);
                if (Config.DrawMethod2)
                    Graphics.S.Draw2D(Graphics.GTexture, new Rectangle(0, 0, 46, 5), Rectangle.Empty, TPoint, Color.Black.ToArgb());
                else
                    Graphics.S.Draw(Graphics.GTexture, new Rectangle(0, 0, 46, 5), Vector3.Empty, new Vector3(TPoint.X, TPoint.Y, 0), Color.Black.ToArgb());
                TPoint.Offset(1, 1);
                Width = (int)(44 * Info.CurrentMP / (float)Info.MaxMP);
                if (Width > 44) Width = 4;
                if (Width < 0) Width = 0;
                if (Config.DrawMethod2)
                    Graphics.S.Draw2D(Graphics.GTexture, new Rectangle(0, 0, Width, 3), Rectangle.Empty, TPoint, Color.Blue.ToArgb());
                else
                    Graphics.S.Draw(Graphics.GTexture, new Rectangle(0, 0, Width, 3), Vector3.Empty, new Vector3(TPoint.X, TPoint.Y, 0), Color.Blue.ToArgb());
                TPoint.Offset(-2, -7);
            }
            if (Info.CurrentHP > 0 && Info.MaxHP > 0)
            {
                TPoint.Offset(1, 0);
                if (Config.DrawMethod2)
                    Graphics.S.Draw2D(Graphics.GTexture, new Rectangle(0, 0, 46, 5), Rectangle.Empty, TPoint, Color.Black.ToArgb());
                else
                    Graphics.S.Draw(Graphics.GTexture, new Rectangle(0, 0, 46, 5), Vector3.Empty, new Vector3(TPoint.X, TPoint.Y, 0), Color.Black.ToArgb());
                TPoint.Offset(1, 1);
                Width = (int)(44 * Info.CurrentHP / (float)Info.MaxHP);
                if (Width > 44) Width = 44;
                if (Width < 0) Width = 0;
                if (Config.DrawMethod2)
                    Graphics.S.Draw2D(Graphics.GTexture, new Rectangle(0, 0, Width, 3), Rectangle.Empty, TPoint, Color.Red.ToArgb());
                else
                    Graphics.S.Draw(Graphics.GTexture, new Rectangle(0, 0, Width, 3), Vector3.Empty, new Vector3(TPoint.X, TPoint.Y, 0), Color.Red.ToArgb());
            }



        }
        public bool MouseOver(Point MousePoint)
        {
            MousePoint.Offset(-FinalDrawPoint.X, -FinalDrawPoint.Y);
            if (Info.Job != Class.Assassin)
            {

                if (CLib.Armours != null)
                    if (Info.Armor >= 0 && Info.Armor < CLib.Armours.Length && CLib.Armours[Info.Armor] != null)                    
                        if (new Rectangle(Point.Empty, CLib.Armours[Info.Armor].GetImageSize(DrawFrame)).Contains(MousePoint))
                            return CLib.Armours[Info.Armor].GetTransparent(DrawFrame, MousePoint);                    
                    else if (CLib.Armours.Length > 0 && CLib.Armours[0] != null)                    
                        if (new Rectangle(Point.Empty, CLib.Armours[0].GetImageSize(DrawFrame)).Contains(MousePoint))
                            return CLib.Armours[0].GetTransparent(DrawFrame, MousePoint);                    
            }
            else
            {
                if (CLib.ArmoursA != null)
                    if (Info.Armor >= 0 && Info.Armor < CLib.ArmoursA.Length && CLib.ArmoursA[Info.Armor] != null)
                        if (new Rectangle(Point.Empty, CLib.ArmoursA[Info.Armor].GetImageSize(DrawFrame)).Contains(MousePoint))
                            return CLib.ArmoursA[Info.Armor].GetTransparent(DrawFrame, MousePoint);
                    else if (CLib.ArmoursA.Length > 0 && CLib.ArmoursA[0] != null)
                        if (new Rectangle(Point.Empty, CLib.ArmoursA[0].GetImageSize(DrawFrame)).Contains(MousePoint))
                            return CLib.ArmoursA[0].GetTransparent(DrawFrame, MousePoint);
            }

            return false;
        }

        private void DrawWWT(Point TPoint)
        {
            #region Weapon
            switch (Info.Dir)
            {
                case Direction.Down:
                case Direction.Up:
                case Direction.Up_Left:
                case Direction.Left:
                case Direction.Down_Left:
                    if (CLib.Weapons != null)
                        if (Info.Weapon >= 0 && Info.Weapon < CLib.Weapons.Length && CLib.Weapons[Info.Weapon] != null)
                            CLib.Weapons[Info.Weapon].Draw(DrawFrame + WOffSet, TPoint, true);
                        else if (CLib.Weapons.Length > 0 && CLib.Weapons[0] != null)
                            CLib.Weapons[0].Draw(DrawFrame + WOffSet, TPoint, true);
                    break;
            }
            #endregion

            if (CLib.Armours != null)
                if (Info.Armor >= 0 && Info.Armor < CLib.Armours.Length && CLib.Armours[Info.Armor] != null)
                {
                    CLib.Armours[Info.Armor].Draw(DrawFrame + AOffSet, TPoint, true);
                    FinalDrawPoint = TPoint;
                    FinalDrawPoint.Offset(CLib.Armours[Info.Armor].GetImageOffSet(DrawFrame));
                }
                else if (CLib.Armours.Length > 0 && CLib.Armours[0] != null)
                {
                    CLib.Armours[0].Draw(DrawFrame + AOffSet, TPoint, true);
                    FinalDrawPoint = TPoint;
                    FinalDrawPoint.Offset(CLib.Armours[0].GetImageOffSet(DrawFrame));
                }

            if (CLib.Hairs != null)
                if (Info.Hair >= 0 && Info.Hair < CLib.Hairs.Length && CLib.Hairs[Info.Hair] != null)
                    CLib.Hairs[Info.Hair].Draw(DrawFrame + AOffSet, TPoint, true);

            #region Weapon
            switch (Info.Dir)
            {
                case Direction.Down_Right:
                case Direction.Right:
                case Direction.Up_Right:
                    if (CLib.Weapons != null)
                        if (Info.Weapon >= 0 && Info.Weapon < CLib.Weapons.Length && CLib.Weapons[Info.Weapon] != null)
                            CLib.Weapons[Info.Weapon].Draw(DrawFrame + WOffSet, TPoint, true);
                        else if (CLib.Weapons.Length > 0 && CLib.Weapons[0] != null)
                            CLib.Weapons[0].Draw(DrawFrame + WOffSet, TPoint, true);
                    break;
            }
            #endregion
        }
        private void DrawA(Point TPoint)
        {
            #region Weapon
            switch (Info.Dir)
            {
                case Direction.Down:
                case Direction.Up:
                case Direction.Up_Left:
                case Direction.Left:
                case Direction.Down_Left:
                    if (CLib.WeaponsAL != null)
                        if (Info.Weapon >= 0 && Info.Weapon < CLib.WeaponsAL.Length && CLib.WeaponsAL[Info.Weapon] != null)
                            CLib.WeaponsAL[Info.Weapon].Draw(DrawFrame + WOffSet, TPoint, true);
                        else if (CLib.WeaponsAL.Length > 0 && CLib.WeaponsAL[0] != null)
                            CLib.WeaponsAL[0].Draw(DrawFrame + WOffSet, TPoint, true);

                    if (CLib.WeaponsAR != null)
                        if (Info.Weapon >= 0 && Info.Weapon < CLib.WeaponsAR.Length && CLib.WeaponsAR[Info.Weapon] != null)
                            CLib.WeaponsAR[Info.Weapon].Draw(DrawFrame + WOffSet, TPoint, true);
                        else if (CLib.WeaponsAR.Length > 0 && CLib.WeaponsAR[0] != null)
                            CLib.WeaponsAR[0].Draw(DrawFrame + WOffSet, TPoint, true);
                    break;
            }
            #endregion

            if (CLib.ArmoursA != null)
                if (Info.Armor >= 0 && Info.Armor < CLib.ArmoursA.Length && CLib.ArmoursA[Info.Armor] != null)
                {
                    CLib.ArmoursA[Info.Armor].Draw(DrawFrame + AOffSet, TPoint, true);
                    FinalDrawPoint = TPoint;
                    FinalDrawPoint.Offset(CLib.ArmoursA[Info.Armor].GetImageOffSet(DrawFrame));
                }

                else if (CLib.ArmoursA.Length > 0 && CLib.ArmoursA[0] != null)
                {
                    CLib.ArmoursA[0].Draw(DrawFrame + AOffSet, TPoint, true);
                    FinalDrawPoint = TPoint;
                    FinalDrawPoint.Offset(CLib.ArmoursA[0].GetImageOffSet(DrawFrame));
                }

            if (CLib.HairsA != null)
                if (Info.Hair >= 0 && Info.Hair < CLib.HairsA.Length && CLib.HairsA[Info.Hair] != null)
                    CLib.HairsA[Info.Hair].Draw(DrawFrame + AOffSet, TPoint, true);

            #region Weapon
            switch (Info.Dir)
            {
                case Direction.Down_Right:
                case Direction.Right:
                case Direction.Up_Right:
                    if (CLib.WeaponsAL != null)
                        if (Info.Weapon >= 0 && Info.Weapon < CLib.WeaponsAL.Length && CLib.WeaponsAL[Info.Weapon] != null)
                            CLib.WeaponsAL[Info.Weapon].Draw(DrawFrame + WOffSet, TPoint, true);
                        else if (CLib.WeaponsAL.Length > 0 && CLib.WeaponsAL[0] != null)
                            CLib.WeaponsAL[0].Draw(DrawFrame + WOffSet, TPoint, true);

                    if (CLib.WeaponsAR != null)
                        if (Info.Weapon >= 0 && Info.Weapon < CLib.WeaponsAR.Length && CLib.WeaponsAR[Info.Weapon] != null)
                            CLib.WeaponsAR[Info.Weapon].Draw(DrawFrame + WOffSet, TPoint, true);
                        else if (CLib.WeaponsAR.Length > 0 && CLib.WeaponsAR[0] != null)
                            CLib.WeaponsAR[0].Draw(DrawFrame + WOffSet, TPoint, true);
                    break;
            }
            #endregion
        }

        public void Remove()
        {
            DirectionList.Clear();
            ElementList.Clear();
            DirectionList.Clear();
            DoAction(MirAction.Standing);
            MapClass.RemoveObject(this, Info.Location);
            PlayerList.Remove(this);
            Visible = false;
        }
    }
}
