﻿using System.Drawing;
using System.IO;
using Clash_Library;

namespace Clash_Client
{
    static class Config
    {
        private const string ConfigPath = @".\Config.ini";
        public static Size CScreenSize = new Size(800, 600), GScreenSize = CScreenSize;
        public static Point PlayerOffSet = Point.Empty;
        public static bool WindowMode = true, PixelDetect = true, SoundOn = true, MManager = true, TopMost = true, UseHardware = true, DrawEffects = true, DrawMethod2 = true;
        public static string IPAddress = "127.0.0.1";
        public static int Port = 7000;

        public static void Load()
        {
            IniReader IReader = new IniReader(ConfigPath);

            if (File.Exists(ConfigPath))
            {
                GScreenSize = IReader.ReadSize("Configuration", "ScreenSize", GScreenSize);

                WindowMode = IReader.ReadBoolean("Configuration", "WindowMode", WindowMode);
                PixelDetect = IReader.ReadBoolean("Configuration", "PixelDetect", PixelDetect);
                SoundOn = IReader.ReadBoolean("Configuration", "SoundOn", SoundOn);
                MManager = IReader.ReadBoolean("Configuration", "MManager", MManager);
                TopMost = IReader.ReadBoolean("Configuration", "TopMost", TopMost);
                UseHardware = IReader.ReadBoolean("Configuration", "UseHardware", UseHardware);
                DrawEffects = IReader.ReadBoolean("Configuration", "DrawEffects", DrawEffects);
                DrawMethod2 = IReader.ReadBoolean("Configuration", "DrawMethod2", DrawMethod2);

                IPAddress = IReader.ReadString("Configuration", "IPAddress", IPAddress);
                Port = IReader.ReadInt32("Configuration", "Port", Port);
            }
            else
                Save();


            PlayerOffSet = new Point((GScreenSize.Width / 96), (GScreenSize.Height / 64));
            Constants.Load();
        }
        public static void Save()
        {
            IniReader IReader = new IniReader(ConfigPath);

            IReader.DontSave = true;
            IReader.Write("Configuration", "ScreenSize", CScreenSize);

            IReader.Write("Configuration", "WindowMode", WindowMode);
            IReader.Write("Configuration", "PixelDetect", PixelDetect);
            IReader.Write("Configuration", "SoundOn", SoundOn);
            IReader.Write("Configuration", "MManager", TopMost);
            IReader.Write("Configuration", "TopMost", MManager);
            IReader.Write("Configuration", "UseHardware", UseHardware);
            IReader.Write("Configuration", "DrawEffects", DrawEffects);
            IReader.Write("Configuration", "DrawMethod2", DrawMethod2);

            IReader.Write("Configuration", "IPAddress", IPAddress);
            IReader.Write("Configuration", "Port", Port);
            IReader.DontSave = false;
            IReader.Save();
        }
    }

    public static class Constants
    {
        public static string
            CreateAccountDisabled = "Creating new accounts is currently disabled.",
            MissingAccountName = "Please input your Account ID.",
            BadAccountName = "Your Account ID is invalid.",
            MissingPassword = "Please input your Password.",
            BadPassword = "Your Password is invalid.",
            MissingPassword2 = "Please input your other Password.",
            BadPassword2 = "Your other password is invalid.",
            MissingNewPassword = "Please input your new Password.",
            BadNewPassword = "Your new Password is invalid.",
            MissingNewPassword2 = "Please input your other new Password.",
            BadNewPassword2 = "Your other new password is invalid.",
            BadCreatorName = "Your name is invalid.",
            BadDoB = "Your Date of Birth is invalid.",
            PasswordMisMatch = "Your passwords do not match.",
            BadQ1 = "Question 1 is invalid.",
            BadA1 = "Answer 1 is invalid.",
            BadQ2 = "Question 2 is invalid.",
            BadA2 = "Answer 2 is invalid.",
            BadPhone = "Phone Number is invalid.",
            BadMobile = "Mobile Number is invalid.",
            BadEmail = "Email address is invalid.",
            AccountExists = "This Account ID is already in use.",
            AccountCreated = "Your account has been created.",
            FailedLoginCount = "There have been {0} Failed login attempts.",
            Unknown = "An unknown error has occured.",
            LoginDisabled = "Logging in is currently disabled.",
            WrongIDorPassword = "Wrong ID/Password Combination.",
            ChangePasswordDisabled = "Chaging Passwords is currently disabled.",
            AccountBanned = "This Account has been banned.",
            PasswordChanged = "Your Password has been changed.",
            MaxCharacters = "You Already have the Maximum characters allowed.",
            Disconnected0 = "You were disconnected by another user.",
            Disconnected1 = "You are disconnected from the server.",
            Disconnected2 = "Connection Timed out.",
            Disconnected3 = "Your IP Address is Banned.",
            ServerOffLine = "The Server is Offline.",
            DeleteCharacter = "Are you sure you want to delete {0}",
            DeleteCharacter2 = "Please wait 5 seconds to confirm.",
            MissingCharacterName = "Please input your character's name.",
            BadCharacterName = "Your character's name is invalid.",
            NewCharacterDisabled = "Character Creation is Disabled.",
            WrongClass = "You have chosen an invalid class.",
            WrongGender = "You have chosen an invalid gender.",
            CharacterExists = "This Character already exists.",
            CharacterCreated = "Your character has been created.",
            CharacterDeleted = "Your character has been deleted.",
            DelCharacterDisabled = "Character deletion is disabled.",
            CharacterNotFound = "Character was not found.",
            StartGameDisabled = "Starting the Game is Disabled.",
            CharacterBanned = "This Character has been banned.";

        public static void Load()
        {
            IniReader iReader = new IniReader(@".\Constants.ini");
            const string section = "Constants";

            if (File.Exists(@".\Constants.ini"))
            {
                CreateAccountDisabled = iReader.ReadString(section, "CreateAccountDisabled", CreateAccountDisabled);
                MissingAccountName = iReader.ReadString(section, "MissingAccountName", MissingAccountName);
                BadAccountName = iReader.ReadString(section, "BadAccountName", BadAccountName);
                MissingPassword = iReader.ReadString(section, "MissingPassword", MissingPassword);
                BadPassword = iReader.ReadString(section, "BadPassword", BadPassword);
                MissingPassword2 = iReader.ReadString(section, "MissingPassword2", MissingPassword2);
                BadPassword2 = iReader.ReadString(section, "BadPassword2", BadPassword2);
                MissingNewPassword = iReader.ReadString(section, "MissingNewPassword", MissingNewPassword);
                BadNewPassword = iReader.ReadString(section, "BadNewPassword", BadNewPassword);
                MissingNewPassword2 = iReader.ReadString(section, "MissingNewPassword2", MissingNewPassword2);
                BadNewPassword2 = iReader.ReadString(section, "BadNewPassword2", BadNewPassword2);
                BadCreatorName = iReader.ReadString(section, "BadCreatorName", BadCreatorName);
                BadDoB = iReader.ReadString(section, "BadDoB", BadDoB);
                PasswordMisMatch = iReader.ReadString(section, "PasswordMisMatch", PasswordMisMatch);
                BadQ1 = iReader.ReadString(section, "BadQ1", BadQ1);
                BadA1 = iReader.ReadString(section, "BadA1", BadA1);
                BadQ2 = iReader.ReadString(section, "BadQ2", BadQ2);
                BadA2 = iReader.ReadString(section, "BadA2", BadA2);
                BadPhone = iReader.ReadString(section, "BadPhone", BadPhone);
                BadMobile = iReader.ReadString(section, "BadMobile", BadMobile);
                BadEmail = iReader.ReadString(section, "BadEmail", BadEmail);
                AccountExists = iReader.ReadString(section, "AccountExists", AccountExists);
                AccountCreated = iReader.ReadString(section, "AccountCreated", AccountCreated);
                FailedLoginCount = iReader.ReadString(section, "FailedLoginCount", FailedLoginCount);
                Unknown = iReader.ReadString(section, "Unknown", Unknown);
                LoginDisabled = iReader.ReadString(section, "LoginDisabled", LoginDisabled);
                WrongIDorPassword = iReader.ReadString(section, "WrongIDorPassword", WrongIDorPassword);
                ChangePasswordDisabled = iReader.ReadString(section, "ChangePasswordDisabled", ChangePasswordDisabled);
                AccountBanned = iReader.ReadString(section, "AccountBanned", AccountBanned);
                PasswordChanged = iReader.ReadString(section, "PasswordChanged", PasswordChanged);
                DeleteCharacter = iReader.ReadString(section, "DeleteCharacter", DeleteCharacter);
                DeleteCharacter2 = iReader.ReadString(section, "DeleteCharacter2", DeleteCharacter2);
                MaxCharacters = iReader.ReadString(section, "MaxCharacters", MaxCharacters);
                Disconnected0 = iReader.ReadString(section, "Disconnected0", Disconnected0);
                Disconnected1 = iReader.ReadString(section, "Disconnected1", Disconnected1);
                Disconnected2 = iReader.ReadString(section, "Disconnected2", Disconnected2);
                Disconnected3 = iReader.ReadString(section, "Disconnected3", Disconnected3);
                ServerOffLine = iReader.ReadString(section, "ServerOffLine", ServerOffLine);
                MissingCharacterName = iReader.ReadString(section, "MissingCharacterName", MissingCharacterName);
                BadCharacterName = iReader.ReadString(section, "BadCharacterName", BadCharacterName);
                NewCharacterDisabled = iReader.ReadString(section, "NewCharacterDisabled", NewCharacterDisabled);
                WrongClass = iReader.ReadString(section, "WrongClass", WrongClass);
                WrongGender = iReader.ReadString(section, "WrongGender", WrongGender);
                CharacterExists = iReader.ReadString(section, "CharacterExists", CharacterExists);
                CharacterCreated = iReader.ReadString(section, "CharacterCreated", CharacterCreated);
                CharacterDeleted = iReader.ReadString(section, "CharacterDeleted", CharacterDeleted);
                DelCharacterDisabled = iReader.ReadString(section, "DelCharacterDisabled", DelCharacterDisabled);
                CharacterNotFound = iReader.ReadString(section, "CharacterNotFound", CharacterNotFound);
                StartGameDisabled = iReader.ReadString(section, "StartGameDisabled", StartGameDisabled);
                CharacterBanned = iReader.ReadString(section, "CharacterBanned", CharacterBanned);
            }
            else
            {
                iReader.Write(section, "CreateAccountDisabled", CreateAccountDisabled);
                iReader.Write(section, "MissingAccountName", MissingAccountName);
                iReader.Write(section, "BadAccountName", BadAccountName);
                iReader.Write(section, "MissingPassword", MissingPassword);
                iReader.Write(section, "BadPassword", BadPassword);
                iReader.Write(section, "MissingPassword2", MissingPassword2);
                iReader.Write(section, "BadPassword2", BadPassword2);
                iReader.Write(section, "MissingNewPassword", MissingNewPassword);
                iReader.Write(section, "BadNewPassword", BadNewPassword);
                iReader.Write(section, "MissingNewPassword2", MissingNewPassword2);
                iReader.Write(section, "BadNewPassword2", BadNewPassword2);
                iReader.Write(section, "BadCreatorName", BadCreatorName);
                iReader.Write(section, "BadDoB", BadDoB);
                iReader.Write(section, "PasswordMisMatch", PasswordMisMatch);
                iReader.Write(section, "BadQ1", BadQ1);
                iReader.Write(section, "BadA1", BadA1);
                iReader.Write(section, "BadQ2", BadQ2);
                iReader.Write(section, "BadA2", BadA2);
                iReader.Write(section, "BadPhone", BadPhone);
                iReader.Write(section, "BadMobile", BadMobile);
                iReader.Write(section, "BadEmail", BadEmail);
                iReader.Write(section, "AccountExists", AccountExists);
                iReader.Write(section, "AccountCreated", AccountCreated);
                iReader.Write(section, "FailedLoginCount", FailedLoginCount);
                iReader.Write(section, "Unknown", Unknown);
                iReader.Write(section, "LoginDisabled", LoginDisabled);
                iReader.Write(section, "WrongIDorPassword", WrongIDorPassword);
                iReader.Write(section, "ChangePasswordDisabled", ChangePasswordDisabled);
                iReader.Write(section, "AccountBanned", AccountBanned);
                iReader.Write(section, "PasswordChanged", PasswordChanged);
                iReader.Write(section, "DeleteCharacter", DeleteCharacter);
                iReader.Write(section, "DeleteCharacter2", DeleteCharacter2);
                iReader.Write(section, "MaxCharacters", MaxCharacters);
                iReader.Write(section, "Disconnected0", Disconnected0);
                iReader.Write(section, "Disconnected1", Disconnected1);
                iReader.Write(section, "Disconnected2", Disconnected2);
                iReader.Write(section, "Disconnected3", Disconnected3);
                iReader.Write(section, "ServerOffLine", ServerOffLine);
                iReader.Write(section, "MissingCharacterName", MissingCharacterName);
                iReader.Write(section, "BadCharacterName", BadCharacterName);
                iReader.Write(section, "NewCharacterDisabled", NewCharacterDisabled);
                iReader.Write(section, "WrongClass", WrongClass);
                iReader.Write(section, "WrongGender", WrongGender);
                iReader.Write(section, "CharacterExists", CharacterExists);
                iReader.Write(section, "CharacterCreated", CharacterCreated);
                iReader.Write(section, "CharacterDeleted", CharacterDeleted);
                iReader.Write(section, "DelCharacterDisabled", DelCharacterDisabled);
                iReader.Write(section, "CharacterNotFound", CharacterNotFound);
                iReader.Write(section, "StartGameDisabled", StartGameDisabled);
                iReader.Write(section, "CharacterBanned", CharacterBanned);
            }


        }
    }
}
