﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using Clash_Library;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX;

namespace Clash_Client
{
    public enum GameStage { Login, LoggingIn, CharSel, Loading, Game }
    static class Engine
    {
        public static GameStage CurrentStage = GameStage.Login;
        public static List<CharSelInfo> CharacterList = new List<CharSelInfo>();
        public static DateTime MoveTime, Time = DateTime.Now;
        public static bool GameRunning = true, Cleaned, Loaded, CanMove;
        public static bool LeftDown, RightDown;
        public static int CharSelected = -1;
        public static Point OldOffSet = new Point(-1, -1);
        public static bool DrawItem;
        public static System.Threading.Thread T;

        public static Color[] RColours = {
            Color.Gray, 
            Color.White, 
            Color.Lime, 
            Color.SkyBlue, 
            Color.Purple,
            Color.DarkSalmon,
            Color.DarkOrange};


        public static void RenderIScene()
        {
            if (Graphics.DeviceLost)
            {
                Graphics.AttemptReset();
                return;
            }
            try
            {
                Graphics.D.Clear(Microsoft.DirectX.Direct3D.ClearFlags.Target, Color.Black, 0, 0);
                Graphics.D.BeginScene();
                Graphics.S.Begin(Microsoft.DirectX.Direct3D.SpriteFlags.AlphaBlend);

                switch (CurrentStage)
                {
                    case GameStage.Login:
                        IntroScenes.BackGrounds.Login.Draw();
                        break;
                    case GameStage.LoggingIn:
                        IntroScenes.BackGrounds.LoggingIn.Draw();
                        break;
                    case GameStage.CharSel:
                        IntroScenes.BackGrounds.CharSel.Draw();
                        break;
                    case GameStage.Loading:
                        IntroScenes.BackGrounds.Loading.Draw();
                        break;
                }

                for (int I = Window.WindowList.Count - 1; I >= 0; I--)
                    Window.WindowList[I].Draw();



                Graphics.S.End();
                Graphics.D.EndScene();
                Graphics.D.Present();
            }
            catch (DeviceLostException)
            {
                Graphics.DeviceLost = true;
            }
            catch (ApplicationException)
            {
                Graphics.AttemptRecovery();
            }
            catch (Exception ex)
            {
                File.AppendAllText(@".\Error.txt", string.Format("[{0}] {1}" + Environment.NewLine, Engine.Time, ex));
            }
        }
        public static void RenderGScene()
        {
            if (Graphics.DeviceLost)
            {
                Graphics.AttemptReset();
                return;
            }
            Point TPoint = Graphics.T.PointToClient(Cursor.Position);
            try
            {
                #region Set Movement OffSet
                if (User.CurrentAction == MirAction.Walking || User.CurrentAction == MirAction.Running)
                {
                    int multi = Convert.ToByte((User.CurrentAction == MirAction.Running)) + 1;
                    switch (User.Info.Dir)
                    {
                        case Direction.Up:
                            MapClass.OffSet = new Point(0, (-32 * multi / (User.FrameCount)) * (User.FrameCount - User.CurrentFrame - 1));
                            break;
                        case Direction.Up_Right:
                            MapClass.OffSet = new Point((48 * multi / (User.FrameCount)) * (User.FrameCount - User.CurrentFrame - 1), (-32 * multi / (User.FrameCount)) * (User.FrameCount - User.CurrentFrame - 1));
                            break;
                        case Direction.Right:
                            MapClass.OffSet = new Point((48 * multi / (User.FrameCount)) * (User.FrameCount - User.CurrentFrame - 1), 0);
                            break;
                        case Direction.Down_Right:
                            MapClass.OffSet = new Point((48 * multi / (User.FrameCount)) * (User.FrameCount - User.CurrentFrame - 1), (32 * multi / (User.FrameCount)) * (User.FrameCount - User.CurrentFrame - 1));
                            break;
                        case Direction.Down:
                            MapClass.OffSet = new Point(0, (32 * multi / (User.FrameCount)) * (User.FrameCount - User.CurrentFrame - 1));
                            break;
                        case Direction.Down_Left:
                            MapClass.OffSet = new Point((-48 * multi / (User.FrameCount)) * (User.FrameCount - User.CurrentFrame - 1), (32 * multi / (User.FrameCount)) * (User.FrameCount - User.CurrentFrame - 1));
                            break;
                        case Direction.Left:
                            MapClass.OffSet = new Point((-48 * multi / (User.FrameCount)) * (User.FrameCount - User.CurrentFrame - 1), 0);
                            break;
                        case Direction.Up_Left:
                            MapClass.OffSet = new Point((-48 * multi / (User.FrameCount)) * (User.FrameCount - User.CurrentFrame - 1), (-32 * multi / (User.FrameCount)) * (User.FrameCount - User.CurrentFrame - 1));
                            break;
                    }
                }
                #endregion

                Graphics.D.Clear(ClearFlags.Target, Color.Black, 0, 0);
                Graphics.D.BeginScene();
                #region Map and Objects

                if (OldOffSet != MapClass.OffSet)
                {
                    #region Set Scene
                    Graphics.S.Begin(SpriteFlags.AlphaBlend);
                    Graphics.D.SetRenderTarget(0, Graphics.MTexture.GetSurfaceLevel(0));
                    #endregion
                    MapClass.DrawTiles();
                    #region End Scene
                    Graphics.S.End();
                    #endregion
                    OldOffSet = MapClass.OffSet;
                }

                #region Set Scene
                Graphics.S.Begin(SpriteFlags.AlphaBlend);
                Graphics.D.SetRenderTarget(0, Graphics.GSurface);
                #endregion
                if (Config.DrawMethod2)
                    Graphics.S.Draw2D(Graphics.MTexture, Point.Empty, 0, Point.Empty, Color.White.ToArgb());
                else
                    Graphics.S.Draw(Graphics.MTexture, Vector3.Empty, Vector3.Empty, Color.White.ToArgb());
                MapClass.DrawObjects();
                if (Monster.HoverMonster != null && Monster.HoverMonster != Monster.TargetMonster) Monster.HoverMonster.DrawHover();
                if (Monster.TargetMonster != null) Monster.TargetMonster.DrawHover();
                if (Player.HoverPlayer != null && Player.HoverPlayer != Player.TargetPlayer) Player.HoverPlayer.DrawHover();
                if (Player.TargetPlayer != null) Player.TargetPlayer.DrawHover();

                #endregion
                #region Set Scene
                Graphics.SetBlend(true);
                #endregion
                foreach (Magic M in Magic.MagicList) M.Draw();
                #region Reset Scene
                Graphics.SetBlend(false);
                #endregion

                if (!User.Info.MapInfo.Flags.HasFlag(MapFlags.Light) || User.Info.Dead)
                {
                    #region Set Scene
                    Graphics.S.End();
                    Graphics.D.SetRenderTarget(0, Graphics.LTexture.GetSurfaceLevel(0));
                    if (User.Info.Dead)
                        Graphics.D.Clear(ClearFlags.Target, Color.Red, 0, 0);
                    else
                        Graphics.D.Clear(ClearFlags.Target, Color.Black, 0, 0);
                    Graphics.S.Begin(SpriteFlags.AlphaBlend);
                    Graphics.D.RenderState.AlphaBlendEnable = true;
                    Graphics.D.RenderState.SourceBlend = Blend.One;
                    Graphics.D.RenderState.DestinationBlend = Blend.One;
                    #endregion
                    RenderLight();
                    #region Reset Scene
                    Graphics.S.End();
                    Graphics.D.SetRenderTarget(0, Graphics.GSurface);
                    Graphics.S.Begin(SpriteFlags.AlphaBlend);
                    Graphics.D.RenderState.SourceBlend = Blend.Zero;
                    Graphics.D.RenderState.DestinationBlend = Blend.SourceColor;
                    #endregion
                    if (Config.DrawMethod2)
                        Graphics.S.Draw2D(Graphics.LTexture, Point.Empty, 0, Point.Empty, Color.White.ToArgb());
                    else
                        Graphics.S.Draw(Graphics.LTexture, Vector3.Empty, Vector3.Empty, Color.White.ToArgb());
                    Graphics.S.End();
                    Graphics.S.Begin(SpriteFlags.AlphaBlend);
                }


                #region Interface
                for (int I = Window.WindowList.Count - 1; I >= 0; I--) Window.WindowList[I].Draw();

                if (ItemGrid.HItem != null)
                {
                    #region Set Scene
                    Graphics.SetFade(0.85f);
                    #endregion
                    if (Config.DrawMethod2)
                        Graphics.S.Draw2D(Graphics.HTexture, new Rectangle(Point.Empty, ItemGrid.HintSize), Rectangle.Empty, TPoint, Color.White);
                    else
                        Graphics.S.Draw(Graphics.HTexture, new Rectangle(Point.Empty, ItemGrid.HintSize), Vector3.Empty, new Vector3(TPoint.X, TPoint.Y, 0), Color.White.ToArgb());
                    #region Reset Scene
                    Graphics.SetFade(0.85f);
                    #endregion
                }
                if (ItemGrid.PItem != null)
                {
                    Size TSize = CLib.Item.GetImageSize(ItemGrid.PItem.IInfo.Image);
                    TPoint.Offset(-TSize.Width / 2, -TSize.Height / 2);
                    CLib.Item.Draw(ItemGrid.PItem.IInfo.Image, TPoint, false);
                }

                Graphics.DXNormalFont.DrawText(Graphics.S, User.Info.Location.ToString(), Point.Empty, Color.Black);
                Graphics.DXNormalFont.DrawText(Graphics.S, User.Info.Location.ToString(), new Point(2, 2), Color.Black);
                Graphics.DXNormalFont.DrawText(Graphics.S, User.Info.Location.ToString(), new Point(0, 2), Color.Black);
                Graphics.DXNormalFont.DrawText(Graphics.S, User.Info.Location.ToString(), new Point(2, 0), Color.Black);
                Graphics.DXNormalFont.DrawText(Graphics.S, User.Info.Location.ToString(), new Point(1, 1), Color.White);
                #endregion
                #region End Scene
                Graphics.S.End();
                Graphics.D.EndScene();
                #endregion
                Graphics.D.Present();
            }
            catch (DeviceLostException)
            {
                Graphics.DeviceLost = true;
            }
            catch (ApplicationException)
            {
                Graphics.AttemptRecovery();
            }
            catch (Exception ex)
            {
                File.AppendAllText(@".\Error.txt", string.Format("[{0}] {1}" + Environment.NewLine, Engine.Time, ex));
            }
        }

        public static void Calculate()
        {
            #region Map Annimations
            if (Engine.Time > MapClass._AnimationTime + new TimeSpan(0, 0, 0, 0, 100))
            {
                MapClass._AnimationTime = Engine.Time;
                MapClass._AnimationCount++;
            }
            #endregion
            #region Move Timer

            if (Engine.Time > MoveTime + new TimeSpan(0, 0, 0, 0, 100))
            {
                MoveTime = Engine.Time;
                CanMove = true;
            }
            else
                CanMove = false;
            #endregion
            User.Operate();
            for (int I = Player.PlayerList.Count - 1; I >= 0; I--)
            {
                if (Math.Abs(Player.PlayerList[I].ActualPoint.X - User.Info.Location.X) > 20 ||
                    Math.Abs(Player.PlayerList[I].ActualPoint.Y - User.Info.Location.Y) > 20)
                {
                    Player.PlayerList[I].Remove();
                    continue;
                }
                Player.PlayerList[I].Operate();
                Player.PlayerList[I].TryStartAction();
            }
            for (int I = Monster.MonsterList.Count - 1; I >= 0; I--)
            {
                if (Math.Abs(Monster.MonsterList[I].ActualPoint.X - User.Info.Location.X) > 20 ||
                    Math.Abs(Monster.MonsterList[I].ActualPoint.Y - User.Info.Location.Y) > 20)
                {
                    Monster.MonsterList[I].Remove();
                    continue;
                }
                Monster.MonsterList[I].Operate();
                Monster.MonsterList[I].TryStartAction();
            }
            for (int I = Magic.MagicList.Count - 1; I >= 0; I--) Magic.MagicList[I].Operate();
            //NPC
        }
        public static void CheckMouse()
        {
            bool Check = true;
            if (FindTarget() && Main.ShiftDown == false) return;
            if (Player.TargetPlayer != null && Main.ShiftDown && !Player.TargetPlayer.Info.Dead)
                if (Math.Abs(User.Info.Location.X - Player.TargetPlayer.ActualPoint.X) <= 1 && Math.Abs(User.Info.Location.Y - Player.TargetPlayer.ActualPoint.Y) <= 1)
                    if (User.CanNextAction() && Engine.Time > User.LastAttack + User.ASpeed - new TimeSpan(0, 0, 0, 0, User.Info.TotalStats.Advanced.ASpeed))
                    {
                        User.Info.Dir = Functions.GetDirection(User.Info.Location, Player.TargetPlayer.ActualPoint);
                        User.DoAction(MirAction.Attack1);
                        Out.Attack();
                    }

            if (Monster.TargetMonster != null && !Monster.TargetMonster.Info.Dead)
                if (Math.Abs(User.Info.Location.X - Monster.TargetMonster.ActualPoint.X) <= 1 && Math.Abs(User.Info.Location.Y - Monster.TargetMonster.ActualPoint.Y) <= 1)
                    if (User.CanNextAction() && Engine.Time > User.LastAttack + User.ASpeed - new TimeSpan(0, 0, 0, 0, User.Info.TotalStats.Advanced.ASpeed))
                    {
                        User.Info.Dir = Functions.GetDirection(User.Info.Location, Monster.TargetMonster.ActualPoint);
                        User.DoAction(MirAction.Attack1);
                        Out.Attack();
                    }
            #region User Controls
            for (int I = 0; I < Window.WindowList.Count; I++)
                if (Window.WindowList[I].ContainsMouse(Graphics.T.PointToClient(Cursor.Position)))
                    Check = false;

            if (Check)
            {
                if (LeftDown && !Main.CtrlDown)
                {
                    if (User.CanNextAction())
                    {
                        Direction Dir = Functions.MouseDirection(Graphics.T.PointToClient(Cursor.Position));
                        if (Main.ShiftDown)
                        {
                            if (Engine.Time > User.LastAttack + User.ASpeed - new TimeSpan(0, 0, 0, 0, User.Info.TotalStats.Advanced.ASpeed))
                            {
                                User.Info.Dir = Dir;
                                User.DoAction(MirAction.Attack1);
                                Out.Attack();
                            }
                        }
                        else if (Main.AltDown)
                        {
                            User.Info.Dir = Dir;
                            User.DoAction(MirAction.Harvest);
                            Out.Harvest();
                        }
                        else if (Functions.CanWalk(Dir))
                        {
                            if (CanMove)
                            {
                                User.Info.Dir = Dir;
                                User.DoAction(MirAction.Walking);
                                User.Info.Location = Functions.Move(User.Info.Location, Dir, 1);
                                Out.Walk();
                            }
                        }
                        else
                        {
                            User.Info.Dir = Dir;
                            User.DoAction(MirAction.Standing);
                            User.CanDoAction = false;
                            Out.Turn();
                        }
                    }
                }
                if (RightDown || LeftDown && Main.CtrlDown)
                {
                    if (User.CanNextAction())
                    {
                        Direction Dir = Functions.MouseDirection(Graphics.T.PointToClient(Cursor.Position));
                        Point TPoint = Functions.MousePoint(Graphics.T.PointToClient(Cursor.Position));
                        if (Math.Abs(TPoint.X - Config.PlayerOffSet.X) <= 2 && Math.Abs(TPoint.Y - Config.PlayerOffSet.Y) <= 2)
                        {
                            User.Info.Dir = Dir;
                            User.DoAction(MirAction.Standing);
                            User.CanDoAction = false;
                            Out.Turn();
                            return;
                        }
                        if (User.CanRun && Functions.CanRun(Dir))
                        {
                            if (CanMove)
                            {
                                User.Info.Dir = Dir;
                                User.DoAction(MirAction.Running);
                                User.Info.Location = Functions.Move(User.Info.Location, Dir, 2);
                                Out.Run();
                            }
                        }
                        else if (Functions.CanWalk(Dir))
                        {
                            if (CanMove)
                            {
                                User.CanRun = true;
                                User.Info.Dir = Dir;
                                User.DoAction(MirAction.Walking);
                                User.Info.Location = Functions.Move(User.Info.Location, Dir, 1);
                                Out.Walk();
                            }
                        }
                        else
                        {
                            User.Info.Dir = Dir;
                            User.DoAction(MirAction.Standing);
                            User.CanDoAction = false;
                            Out.Turn();
                        }
                    }
                }
            }
            #endregion


            if (Player.TargetPlayer != null && Main.ShiftDown && !Player.TargetPlayer.Info.Dead)
                if (Math.Abs(User.Info.Location.X - Player.TargetPlayer.ActualPoint.X) > 1 || Math.Abs(User.Info.Location.Y - Player.TargetPlayer.ActualPoint.Y) > 1)
                    if (User.CanNextAction() && CanMove)
                    {
                        Direction Dir = Functions.GetDirection(User.Info.Location, Player.TargetPlayer.ActualPoint);
                        if (Functions.CanWalk(Dir))
                        {
                            User.Info.Dir = Dir;
                            User.DoAction(MirAction.Walking);
                            User.Info.Location = Functions.Move(User.Info.Location, User.Info.Dir, 1);
                            Out.Walk();
                        }
                        else
                        {
                            User.Info.Dir = Dir;
                            User.DoAction(MirAction.Standing);
                            User.CanDoAction = false;
                            Out.Turn();
                        }
                    }

            if (Monster.TargetMonster != null && !Monster.TargetMonster.Info.Dead)
                if (Math.Abs(User.Info.Location.X - Monster.TargetMonster.ActualPoint.X) > 1 || Math.Abs(User.Info.Location.Y - Monster.TargetMonster.ActualPoint.Y) > 1)
                    if (User.CanNextAction() && CanMove)
                    {
                        Direction Dir = Functions.GetDirection(User.Info.Location, Monster.TargetMonster.ActualPoint);
                        if (Functions.CanWalk(Dir))
                        {
                            User.Info.Dir = Dir;
                            User.DoAction(MirAction.Walking);
                            User.Info.Location = Functions.Move(User.Info.Location, User.Info.Dir, 1);
                            Out.Walk();
                        }
                        else
                        {
                            User.Info.Dir = Dir;
                            User.DoAction(MirAction.Standing);
                            User.CanDoAction = false;
                            Out.Turn();
                        }
                    }
        }
        public static bool FindTarget()
        {
            for (int I = 0; I < Window.WindowList.Count; I++)
                if (Window.WindowList[I].ContainsMouse(Graphics.T.PointToClient(Cursor.Position)))
                    return false;

            Monster.HoverMonster = null;
            Player.HoverPlayer = null;
            if (LeftDown)
            {
                Monster.TargetMonster = null;
                Player.TargetPlayer = null;
            }

            for (int I = 0; I < Monster.MonsterList.Count; I++)
            {
                Monster M = Monster.MonsterList[I];
                if (M.MouseOver(Graphics.T.PointToClient(Cursor.Position)))
                {
                    Monster.HoverMonster = M;
                    if (LeftDown) Monster.TargetMonster = M;
                    return false;
                }
            }
            for (int I = 0; I < Player.PlayerList.Count; I++)
            {
                Player P = Player.PlayerList[I];
                if (P.MouseOver(Graphics.T.PointToClient(Cursor.Position)))
                {
                    Player.HoverPlayer = P;
                    if (LeftDown) Player.TargetPlayer = P;
                    return true;
                }
            }
            return false;
        }
        public static void RenderLight()
        {
            if (User.Info.Dead) return;
            if (Graphics.L == null || Graphics.L.Count == 0) return;
            Point TPoint = Config.PlayerOffSet;

            TPoint.X *= 48; TPoint.X += 12;
            TPoint.Y *= 32; TPoint.Y += 8;
            int L = User.Info.TotalStats.Advanced.Light;

            if (L < 0) L = 0;
            if (L > Graphics.L.Count - 1) L = Graphics.L.Count - 1;
            TPoint.X -= (L + 1) * 65 / 2;
            TPoint.Y -= (L + 1) * 50 / 2;
            if (Config.DrawMethod2)
                Graphics.S.Draw2D(Graphics.L[L], Point.Empty, 0, TPoint, Color.White.ToArgb());
            else
                Graphics.S.Draw(Graphics.L[L], Vector3.Empty, new Vector3(TPoint.X, TPoint.Y, 0), Color.White.ToArgb());
            #region Monsters
            for (int I = 0; I < Monster.MonsterList.Count; I++)
            {
                Monster M = Monster.MonsterList[I];
                if ((L = M.Info.Light) == 0) continue;
                if (L > Graphics.L.Count - 1) L = Graphics.L.Count - 1;
                TPoint = M.Info.Location;
                TPoint.Offset(-User.Info.Location.X, -User.Info.Location.Y);
                TPoint.Offset(Config.PlayerOffSet);
                TPoint.X *= 48; TPoint.X += 12;
                TPoint.Y *= 32; TPoint.Y += 8;
                TPoint.Offset(MapClass.OffSet);
                TPoint.Offset(M.MovementOffSet());
                TPoint.X -= (L + 1) * 65 / 2;
                TPoint.Y -= (L + 1) * 50 / 2;
                if (Config.DrawMethod2)
                    Graphics.S.Draw2D(Graphics.L[L], Point.Empty, 0, TPoint, Color.White.ToArgb());
                else
                    Graphics.S.Draw(Graphics.L[L], Vector3.Empty, new Vector3(TPoint.X, TPoint.Y, 0), Color.White.ToArgb());
            }
            #endregion
            #region Players
            for (int I = 0; I < Player.PlayerList.Count; I++)
            {
                Player P = Player.PlayerList[I];
                if ((L = P.Info.Light) == 0) continue;
                if (L > Graphics.L.Count - 1) L = Graphics.L.Count - 1;
                TPoint = P.Info.Location;
                TPoint.Offset(-User.Info.Location.X, -User.Info.Location.Y);
                TPoint.Offset(Config.PlayerOffSet);
                TPoint.X *= 48; TPoint.X += 12;
                TPoint.Y *= 32; TPoint.Y += 8;
                TPoint.Offset(MapClass.OffSet);
                TPoint.Offset(P.MovementOffSet());
                TPoint.X -= (L + 1) * 65 / 2;
                TPoint.Y -= (L + 1) * 50 / 2;
                if (Config.DrawMethod2)
                    Graphics.S.Draw2D(Graphics.L[L], Point.Empty, 0, TPoint, Color.White.ToArgb());
                else
                    Graphics.S.Draw(Graphics.L[L], Vector3.Empty, new Vector3(TPoint.X, TPoint.Y, 0), Color.White.ToArgb());
            }
            #endregion
            #region Magic
            for (int I = 0; I < Magic.MagicList.Count; I++)
            {
                Magic M = Magic.MagicList[I];
                if ((L = M.Light) <= 0) continue;
                if (L > Graphics.L.Count - 1) L = Graphics.L.Count - 1;
                if (L > Graphics.L.Count - 1) L = Graphics.L.Count - 1;
                TPoint = M.Location;
                TPoint.Offset(-User.Info.Location.X, -User.Info.Location.Y);
                TPoint.Offset(Config.PlayerOffSet);
                TPoint.X *= 48; TPoint.X += 12;
                TPoint.Y *= 32; TPoint.Y += 8;
                TPoint.Offset(MapClass.OffSet);
                TPoint.X -= (L + 1) * 65 / 2;
                TPoint.Y -= (L + 1) * 50 / 2;
                if (Config.DrawMethod2)
                    Graphics.S.Draw2D(Graphics.L[L], Point.Empty, 0, TPoint, M.LightColour.ToArgb());
                else
                    Graphics.S.Draw(Graphics.L[L], Vector3.Empty, new Vector3(TPoint.X, TPoint.Y, 0), M.LightColour.ToArgb());
            }

            #endregion
            int MinX = 0, MinY = 0, MaxX = 0, MaxY = 0;
            #region View Range

            if ((MinX = User.Info.Location.X - Config.PlayerOffSet.X - 15) < 0) MinX = 0;
            if ((MaxX = User.Info.Location.X + Config.PlayerOffSet.X + 15) >= MapClass.MapSize.Width) MaxX = MapClass.MapSize.Width - 1;

            if ((MinY = User.Info.Location.Y - Config.PlayerOffSet.Y - 15) < 0) MinY = 0;
            if ((MaxY = User.Info.Location.Y + Config.PlayerOffSet.Y + 15) >= MapClass.MapSize.Height) MaxY = MapClass.MapSize.Height - 1;
            #endregion

            
            for (int X = MinX; X < MaxX; X++)
                for (int Y = MinY; Y < MaxY; Y++)
                {
                    if (MapClass.Type == MapType.Mir2  && MapClass.M2CellInfo[X, Y].Light > 20) continue;
                    if (MapClass.Type == MapType.Mir3  && MapClass.M3CellInfo[X, Y].Light > 20) continue;
                    if (MapClass.Type == MapType.Mir2)
                    {
                        L = MapClass.M2CellInfo[X, Y].Light * 3;
                    }
                    else if (MapClass.Type == MapType.Mir3)
                    {
                        L = MapClass.M3CellInfo[X, Y].Light * 3;
                    }
                    if (L > 0)
                    {
                        if (L > Graphics.L.Count - 1) L = Graphics.L.Count - 1;
                        if (L > Graphics.L.Count - 1) L = Graphics.L.Count - 1;
                        TPoint = new Point(X, Y);
                        TPoint.Offset(-User.Info.Location.X -1, -User.Info.Location.Y -2);
                        TPoint.Offset(Config.PlayerOffSet);
                        TPoint.X *= 48; TPoint.X += 12;
                        TPoint.Y *= 32; TPoint.Y += 8;
                        TPoint.Offset(MapClass.OffSet);
                        TPoint.X -= (L + 1) * 65 / 2;
                        TPoint.Y -= (L + 1) * 50 / 2;
                        if (Config.DrawMethod2)
                            Graphics.S.Draw2D(Graphics.L[L], Point.Empty, 0, TPoint, Color.White.ToArgb());
                        else
                            Graphics.S.Draw(Graphics.L[L], Vector3.Empty, new Vector3(TPoint.X, TPoint.Y, 0), Color.White.ToArgb());
                    }
                }
        }


        public static void LoadGame(object o)
        {
            ClearIntro();
            GC.Collect();
            Cleaned = true;
            if (Loaded)
                CurrentStage = GameStage.Game;
        }
        public static void ClearIntro()
        {
            for (int i = 1; i < 56; i++)
                CLib.Interface.Dispose(i);

            for (int i = 60; i < 188; i++)
                CLib.Interface.Dispose(i);

            for (int i = 251; i < 283; i++)
                CLib.Interface.Dispose(i);
        }
    }
}
