﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Clash_Library;

namespace Clash_Client
{
    static class Functions
    {
        public static Queue<QuestionInfo> QuestionList = new Queue<QuestionInfo>();

        public static int GetPointerLocation(string str)
        {
            using (System.Drawing.Font TextFont = new System.Drawing.Font("Cambria", 9.5f, FontStyle.Regular))
            {
                if (str == null) return 0;
                if (str.Length == 0) return 0;
                return TextRenderer.MeasureText(str, TextFont).Width - 7;
            }
        }
        public static int GetTextPointer(String str, int MousePoint)
        {
            using (System.Drawing.Font TextFont = new System.Drawing.Font("Cambria", 9.5f, FontStyle.Regular))
            {
                if (str == null) return 0;
                if (TextRenderer.MeasureText(str, TextFont).Width <= MousePoint) return str.Length;

                string tempString = "";
                int characterSize = 0;
                for (int I = 0; I < str.Length; I++)
                {
                    characterSize = TextRenderer.MeasureText(str[I].ToString(), TextFont).Width - 7;
                    tempString += str[I];
                    if (TextRenderer.MeasureText(tempString, TextFont).Width - 7 >= MousePoint)
                    {
                        if (TextRenderer.MeasureText(tempString, TextFont).Width - 7 - (characterSize / 2) >= MousePoint)
                            return I;
                        else return I + 1;
                    }
                }
                return str.Length;
            }

        }
        public static Size GetStringSize(string str)
        {
            using (System.Drawing.Font TextFont = new System.Drawing.Font("Cambria", 9.5f, FontStyle.Regular))
            {
                if (str == null) return Size.Empty;
                if (str.Length == 0) return Size.Empty;
                Size tempSize = TextRenderer.MeasureText(str, TextFont);
                tempSize.Width -= 7;
                return tempSize;
            }
        }

        public static void ShowMessage(string Message)
        {
            IntroScenes.MessageDialogVars.Window.Text = Message;
            IntroScenes.MessageDialogVars.Window.Visible = true;
        }
        public static void AskQuestion(string Message, DefaultDelegate Extra, DefaultDelegate Yes, DefaultDelegate No, bool ShowYes)
        {
            QuestionList.Enqueue(new QuestionInfo(Message, Extra, Yes, No, ShowYes));

        }

        public static Direction MouseDirection(Point MousePoint)
        {
            PointF A, B, C;
            float AC, BC, AB;
            C = new PointF(Config.CScreenSize.Width / 2, Config.CScreenSize.Height / 2 );
            A = new PointF(C.X, 0);
            B = MousePoint;
            BC = (float)Distance(C, B);
            AC = BC;
            B.Y -= C.Y;
            C.Y += BC;
            B.Y += BC;
            AB = (float)Distance(B, A);
            Double X = (AC * AC + BC * BC - AB * AB) / (2 * AC * BC);
            Double Angle = Math.Acos(X);

            Angle *= 180 / Math.PI;

            if (MousePoint.X < C.X) Angle = 360 - Angle;
            Angle += 22.5F;
            if (Angle > 360) Angle -= 360;

            return  (Direction)(Angle / 45);
        }
        public static Point MousePoint(Point MousePoint)
        {
            return new Point(MousePoint.X / 48, MousePoint.Y / 32);
        }

        private static double Distance(PointF P1, PointF P2)
        {
            double X, Y;
            X = P2.X - P1.X;
            Y = P2.Y - P1.Y;
            return Math.Sqrt(X * X + Y * Y);
        }
        public static bool CanWalk(Direction D)
        {
            Point TPoint = Move(User.Info.Location, D, 1);
            return MapClass.EmptyCell(TPoint);
        }
        public static bool CanRun(Direction D)
        {
            Point TPoint = Move(User.Info.Location, D, 2);
            return CanWalk(D) && MapClass.EmptyCell(TPoint);
        }
        public static Point Move(Point P, Direction D, int A)
        {
            switch (D)
            {
                case Direction.Up:
                    P.Offset(0, -A);
                    break;
                case Direction.Up_Right:
                    P.Offset(A, -A);
                    break;
                case Direction.Right:
                    P.Offset(A, 0);
                    break;
                case Direction.Down_Right:
                    P.Offset(A, A);
                    break;
                case Direction.Down:
                    P.Offset(0, A);
                    break;
                case Direction.Down_Left:
                    P.Offset(-A, A);
                    break;
                case Direction.Left:
                    P.Offset(-A, 0);
                    break;
                case Direction.Up_Left:
                    P.Offset(-A, -A);
                    break;
            }
            return P;
        }

        public static Direction GetDirection(Point Old, Point New)
        {
            Direction TempD = Direction.Up;

            if (New.X < Old.X) // Left
            {
                TempD = Direction.Left;
                if (New.Y < Old.Y) // Up
                    TempD = Direction.Up_Left;
                else
                    if (New.Y > Old.Y)
                        TempD = Direction.Down_Left;
            }
            else if (New.X > Old.X) //Right
            {
                TempD = Direction.Right;
                if (New.Y < Old.Y) // Up
                    TempD = Direction.Up_Right;
                else
                    if (New.Y > Old.Y)
                        TempD = Direction.Down_Right;
            }
            else
            {
                if (New.Y < Old.Y)
                    TempD = Direction.Up;
                else
                    if (New.Y > Old.Y)
                        TempD = Direction.Down;
            }
            return TempD;
        }
        public static void AddToBag(Item A)
        {
            for (int I = 6; I < User.Info.BagItems.Length; I++)
                if (User.Info.BagItems[I] == null)
                {
                    User.Info.BagItems[I] = A;
                    return;
                }

            for (int I = 0; I < 6; I++)
                if (User.Info.BagItems[I] == null)
                {
                    User.Info.BagItems[I] = A;
                    return;
                }
        }
    }


    class QuestionInfo
    {
        public string Msg;
        public DefaultDelegate DExtra, YesButton, NoButton;
        public bool YesVisible;
        public QuestionInfo(string Message, DefaultDelegate Extra, DefaultDelegate Yes, DefaultDelegate No, bool ShowYes)
        {
            Msg = Message;
            DExtra = Extra;
            YesButton = Yes;
            YesVisible = ShowYes;
            NoButton = No;
        }
        public void Show()
        {
            IntroScenes.QuestionDialogVars.Window.Visible = true;
            IntroScenes.QuestionDialogVars.Window.Text = Msg;
            IntroScenes.QuestionDialogVars.Window.DrawAfter = DExtra;
            IntroScenes.QuestionDialogVars.Yes.Function = YesButton;
            if (YesVisible) IntroScenes.QuestionDialogVars.Yes.Visible = true;
            else IntroScenes.QuestionDialogVars.Yes.Visible = false;
            IntroScenes.QuestionDialogVars.DialogTime = Engine.Time;
            IntroScenes.QuestionDialogVars.No.Function = NoButton;
            IntroScenes.QuestionDialogVars.No.Visible = true;
        }
    }

}
