﻿using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using Microsoft.DirectX.Direct3D;


namespace Clash_Client
{
    static class Graphics
    {
        public static bool DeviceLost;
        public static Device D;
        public static Sprite S;        
        public static Control T;
        public static PresentParameters P;
        public static Texture GTexture, MTexture, HTexture, LTexture;
        public static Surface GSurface, LSurface;
        public static Microsoft.DirectX.Direct3D.Font DXNormalFont;
        public static bool Blending;
        public static List<Texture> L;

        public static void Create(Form Target)
        {
            DeviceLost = false;
            T = Target;

            P = new PresentParameters();
            P.BackBufferFormat = Manager.Adapters[0].CurrentDisplayMode.Format;
            P.BackBufferWidth = Config.CScreenSize.Width;
            P.BackBufferHeight = Config.CScreenSize.Height;
            P.SwapEffect = SwapEffect.Discard;
            P.PresentationInterval = PresentInterval.Immediate;
            P.Windowed = Config.WindowMode;
            
            
            if (Config.UseHardware)
                D = new Device(0, DeviceType.Hardware, T, CreateFlags.HardwareVertexProcessing, P);
            else
                D = new Device(0, DeviceType.Hardware, T, CreateFlags.SoftwareVertexProcessing, P);
            S = new Sprite(D);
            LoadTextures();

            D.DeviceLost += new System.EventHandler(D_DeviceLost);
            D.DeviceReset += new System.EventHandler(D_DeviceReset);
            D.DeviceResizing += new System.ComponentModel.CancelEventHandler(D_DeviceResizing);
        }

        static void D_DeviceResizing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            //e.Cancel = true;
        }

        static void D_DeviceReset(object sender, System.EventArgs e)
        {
            LoadTextures();
        }
        static void D_DeviceLost(object sender, System.EventArgs e)
        {
            DeviceLost = true;
        }



        public static void SetBlend(bool Value)
        {
            if (Value == Blending) return;
            else Blending = Value;

            Graphics.S.End();
            if (Blending)
            {
                Graphics.S.Begin(SpriteFlags.DoNotSaveState);
                Graphics.D.RenderState.AlphaBlendEnable = true;
                Graphics.D.RenderState.SourceBlend = Blend.One;
                Graphics.D.RenderState.DestinationBlend = Blend.One;
            }
            else
                Graphics.S.Begin(SpriteFlags.AlphaBlend);
        }
        public static void SetFade(float F)
        {
            S.Flush();
            D.RenderState.AlphaBlendEnable = true;
            D.RenderState.SourceBlend = Blend.BlendFactor;
            D.RenderState.DestinationBlend = Blend.InvBlendFactor;
            D.RenderState.AlphaSourceBlend = Blend.SourceAlpha;
            if (F >= 1)
                D.RenderState.BlendFactor = Color.FromArgb(255, 255, 255, 255);
            else
                D.RenderState.BlendFactor = Color.FromArgb((int)(255 * F), (int)(255 * F), (int)(255 * F), (int)(255 * F));
            //S.Flush();
        }

        private static void LoadTextures()
        {
            using (Stream stream = CreateTextureStream(Color.White, Config.CScreenSize))
            {
                GTexture = TextureLoader.FromStream(D, stream, Config.CScreenSize.Width, Config.CScreenSize.Height, 1, Usage.RenderTarget, Format.A8B8G8R8, Pool.Default, Filter.None, Filter.None, 0);
                stream.Seek(0, SeekOrigin.Begin);
                MTexture = TextureLoader.FromStream(D, stream, Config.CScreenSize.Width, Config.CScreenSize.Height, 1, Usage.RenderTarget, Format.A8B8G8R8, Pool.Default, Filter.None, Filter.None, 0);
                stream.Seek(0, SeekOrigin.Begin);
                LTexture = TextureLoader.FromStream(D, stream, Config.CScreenSize.Width, Config.CScreenSize.Height, 1, Usage.RenderTarget, Format.A8B8G8R8, Pool.Default, Filter.None, Filter.None, 0);
                stream.Seek(0, SeekOrigin.Begin);
                HTexture = TextureLoader.FromStream(D, stream, Config.CScreenSize.Width, Config.CScreenSize.Height, 1, Usage.RenderTarget, Format.A8B8G8R8, Pool.Default, Filter.None, Filter.None, 0);
            }

            GenerateLights();

            using (System.Drawing.Font F = new System.Drawing.Font("Cambria", 10, FontStyle.Regular))
                DXNormalFont = new Microsoft.DirectX.Direct3D.Font(D, F);

            GSurface = D.GetBackBuffer(0, 0, BackBufferType.Mono);  
            LSurface = LTexture.GetSurfaceLevel(0);
        }
        private static void GenerateLights()
        {
            L = new List<Texture>();
            for (int I = 1; I < 15; I++)
                using (MemoryStream MStream = new MemoryStream())
                using (Bitmap B = new Bitmap(65 * I, 50 * I, System.Drawing.Imaging.PixelFormat.Format32bppArgb))
                using (System.Drawing.Graphics G = System.Drawing.Graphics.FromImage(B))
                using (System.Drawing.Drawing2D.GraphicsPath P = new System.Drawing.Drawing2D.GraphicsPath())
                {
                    P.AddEllipse(new Rectangle(0, 0, 65 * I, 50 * I));
                    using (System.Drawing.Drawing2D.PathGradientBrush Br = new System.Drawing.Drawing2D.PathGradientBrush(P))
                    {
                        G.Clear(Color.Black);
                        G.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
                        Br.SurroundColors = new Color[] { Color.Black };
                        Br.CenterColor = Color.White;
                        G.FillPath(Br, P);
                        G.Save();
                        B.Save(MStream, System.Drawing.Imaging.ImageFormat.Png);
                        MStream.Position = 0;
                        L.Add(TextureLoader.FromStream(D, MStream, 65 * I, 50 * I, 1, Usage.RenderTarget, Format.A8B8G8R8, Pool.Default, Filter.None, Filter.None, 0));
                    }
                }
        }
        private static MemoryStream CreateTextureStream(Color _Colour, Size S)
        {
            using (MemoryStream mStream = new MemoryStream())
            {
                using (Bitmap bitMap = new Bitmap(S.Width, S.Height, System.Drawing.Imaging.PixelFormat.Format16bppRgb555))
                {
                    using (System.Drawing.Graphics G = System.Drawing.Graphics.FromImage(bitMap))
                    {
                        using (SolidBrush brush = new SolidBrush(_Colour))
                        {
                            G.FillRectangle(brush, new Rectangle(new Point(0, 0), bitMap.Size));
                            G.Save();
                            bitMap.Save(mStream, System.Drawing.Imaging.ImageFormat.Png);
                            mStream.Position = 0;
                            return new MemoryStream(mStream.ToArray());
                        }
                    }
                }
            }
        }

        public static void AttemptRecovery()
        {
            try
            {
                S.End();
            }
            catch
            { }

            try
            {
                D.EndScene();
            }
            catch
            { }
        }
        public static void AttemptReset()
        {
            int Result;
            D.CheckCooperativeLevel(out Result);
            try
            {
                switch ((ResultCode)Result)
                {
                    case ResultCode.DeviceNotReset:
                        D.Reset(P);
                        S = new Sprite(D);
                        break;
                    case ResultCode.DeviceLost:
                        break;
                    case ResultCode.Success:
                        DeviceLost = false;
                        break;
                }
            }
            catch
            { }
        }   
    }
}
