﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using Clash_Library;

namespace Clash_Client
{
    static class MapClass
    {
        public static Size MapSize = Size.Empty;
        public static Mir2CellInfo[,] M2CellInfo = null;
        public static Mir3CellInfo[,] M3CellInfo = null;
        public static Mir3TileInfo[,] M3TileInfo = null;
        public static Point OffSet;
        public static int _AnimationCount;
        public static DateTime _AnimationTime;
        public static List<object>[,] MapObjects;

        public static MapType Type
        {
            get { return User.Info.MapInfo.Type; }
            set { User.Info.MapInfo.Type = value; }
        }
        public static string MapName
        {
            get { return User.Info.MapInfo.MapDescription; }
            set { User.Info.MapInfo.MapDescription = value; }
        }
        public static string FileName
        {
            get { return User.Info.MapInfo.FileName; }
            set { User.Info.MapInfo.FileName = value; }
        }

        public static void LoadMap()
        {
            if (Type == MapType.Mir2)
                LoadMir2Map();
            else
                LoadMir3Map();
            MapObjects = new List<object>[MapSize.Width, MapSize.Height];
        }
        private static void LoadMir2Map()
        {
            byte[] buffer; int OffSet;

            if (File.Exists(@".\Maps\" + FileName + ".map"))
                try
                {
                    buffer = File.ReadAllBytes(@".\Maps\" + FileName + ".map");
                    MapSize = new Size(BitConverter.ToInt16(buffer, 0), BitConverter.ToInt16(buffer, 2));
                    M2CellInfo = new Mir2CellInfo[MapSize.Width, MapSize.Height];
                    OffSet = 52;
                    for (int x = 0; x < MapSize.Width; x++)
                        for (int y = 0; y < MapSize.Height; y++)
                        {
                            MapClass.M2CellInfo[x, y] = new Mir2CellInfo(
                                BitConverter.ToInt16(buffer, OffSet),
                                BitConverter.ToInt16(buffer, OffSet + 2),
                                BitConverter.ToInt16(buffer, OffSet + 4),
                                buffer[OffSet + 6],
                                buffer[OffSet + 7],
                                buffer[OffSet + 8],
                                buffer[OffSet + 9],
                                buffer[OffSet + 10],
                                buffer[OffSet + 11]);
                            OffSet += 12;
                        }
                }
                catch
                {
                }
        }
        private static void LoadMir3Map()
        {
            byte[] buffer; int OffSet;

            try
            {
                if (File.Exists(@".\Maps\" + FileName + ".map"))
                {
                    buffer = File.ReadAllBytes(@".\Maps\" + FileName + ".map");
                    MapSize = new Size(BitConverter.ToInt16(buffer, 22), BitConverter.ToInt16(buffer, 24));
                    M3TileInfo = new Mir3TileInfo[MapSize.Width / 2, MapSize.Height / 2];
                    M3CellInfo = new Mir3CellInfo[MapSize.Width, MapSize.Height];
                    OffSet = 28;
                    for (int X = 0; X < MapSize.Width / 2; X++)
                        for (int Y = 0; Y < MapSize.Height / 2; Y++)
                        {
                            M3TileInfo[X, Y] = new Mir3TileInfo(buffer[OffSet], BitConverter.ToUInt16(buffer, OffSet + 1));
                            OffSet += 3;
                            if (M3TileInfo[X, Y].ImageIndex != 65535)
                            {
                            }
                            if (M3TileInfo[X, Y].FileIndex != 255)
                            {
                            }
                        }

                    for (int X = 0; X < MapSize.Width; X++)
                        for (int Y = 0; Y < MapSize.Height; Y++)
                        {
                            M3CellInfo[X, Y] = new Mir3CellInfo(buffer[OffSet], buffer[OffSet + 1], buffer[OffSet + 2], BitConverter.ToUInt16(buffer, OffSet + 3),
                                BitConverter.ToUInt16(buffer, OffSet + 5), BitConverter.ToUInt16(buffer, OffSet + 7), BitConverter.ToUInt16(buffer, OffSet + 9), buffer[OffSet + 11], BitConverter.ToUInt16(buffer, OffSet + 12));
                            OffSet += 14;
                        }
                }
            }
            catch
            {

            }
        }

        public static void DrawTiles()        
        {
            int MinX = 0, MinY = 0, MaxX = 0, MaxY = 0, Draw, ImageIndex, filenumber;
            Point tempPoint;
            Graphics.D.Clear(Microsoft.DirectX.Direct3D.ClearFlags.Target, Color.Black, 0, 0);
            #region View Range
            if ((MinX = User.Info.Location.X - Config.PlayerOffSet.X - 4) < 0) MinX = 0;
            if ((MaxX = User.Info.Location.X + Config.PlayerOffSet.X + 4) >= MapSize.Width) MaxX = MapSize.Width - 1;

            if ((MinY = User.Info.Location.Y - Config.PlayerOffSet.Y - 6) < 0) MinY = 0;
            if ((MaxY = User.Info.Location.Y + Config.PlayerOffSet.Y + 4) >= MapSize.Height) MaxY = MapSize.Height - 1;
            #endregion
            #region Mir 2
            if (Type == MapType.Mir2)
            {
                #region Big Tiles
                for (int x = MinX; x <= MaxX; x++)
                    for (int y = MinY; y < MaxY; y++)
                    {
                        Draw = MapClass.M2CellInfo[x, y].BackImage & 0x7FFF;
                        if (x % 2 == 0 && y % 2 == 0)
                        {
                            tempPoint = new Point((x - User.Info.Location.X + Config.PlayerOffSet.X) * 48, (y - User.Info.Location.Y + Config.PlayerOffSet.Y) * 32);
                            tempPoint.Offset(OffSet);
                            CLib.Tile.Draw(Draw - 1, tempPoint, false); ;
                        }
                    }
                #endregion
                #region Small Tiles
                for (int x = MinX; x <= MaxX; x++)
                    for (int y = MinY; y <= MaxY; y++)
                    {
                        Draw = MapClass.M2CellInfo[x, y].MidImage;
                        if (Draw-- > 0)
                        {
                            tempPoint = new Point((x - User.Info.Location.X + Config.PlayerOffSet.X) * 48, (y - User.Info.Location.Y + Config.PlayerOffSet.Y) * 32);
                            tempPoint.Offset(OffSet);
                            CLib.SmallTile.Draw(Draw, tempPoint, false);
                        }
                    }
                #endregion
            }
            #endregion
            #region Mir 3
            else
            {
                #region Big Tiles
                for (int X = MinX; X < MaxX; X++)
                    for (int Y = MinY; Y < MaxY; Y++)
                    {
                        if (X % 2 == 0 && Y % 2 == 0)
                        {
                            filenumber = M3TileInfo[X / 2, Y / 2].FileIndex;
                            ImageIndex = M3TileInfo[X / 2, Y / 2].ImageIndex;

                            if (filenumber != 255 && ImageIndex != 65535)
                            {

                                tempPoint = new Point((X - User.Info.Location.X + Config.PlayerOffSet.X) * 48, (Y - User.Info.Location.Y + Config.PlayerOffSet.Y) * 32);
                                tempPoint.Offset(OffSet);
                                if (filenumber < CLib.Mir3Objects.Length && CLib.Mir3Objects[filenumber] != null)
                                    CLib.Mir3Objects[filenumber].Draw(ImageIndex +1, tempPoint, false);
                            }
                        }
                    }
                #endregion
            }
            #endregion
        }
        public static void DrawObjects()
        {
            int MinX = 0, MinY = 0, MaxX = 0, MaxY = 0, Draw, ImageIndex, filenumber;
            byte AnimationFrame, AnimationTick, FileIndex;
            Point tempPoint; Size tempSize; byte[] buffer;
            #region View Range
            if ((MinX = User.Info.Location.X - Config.PlayerOffSet.X - 4) < 0) MinX = 0;
            if ((MaxX = User.Info.Location.X + Config.PlayerOffSet.X + 4) >= MapSize.Width) MaxX = MapSize.Width - 1;

            if ((MinY = User.Info.Location.Y - Config.PlayerOffSet.Y - 4) < 0) MinY = 0;
            if ((MaxY = User.Info.Location.Y + Config.PlayerOffSet.Y + 40) >= MapSize.Height) MaxY = MapSize.Height - 1;
            #endregion
            #region Mir2
            if (Type == MapType.Mir2)
            {
                #region Objects
                for (int Y = MinY; Y <= MaxY; Y++)
                {
                    for (int X = MinX; X <= MaxX; X++)
                    {

                        Draw = MapClass.M2CellInfo[X, Y].FrontImage & 0x7FFF;
                        if (Draw-- > 0)
                        {

                            AnimationFrame = (byte)(MapClass.M2CellInfo[X, Y].AniFrame & 0x7F);
                            if (AnimationFrame > 0)
                            {
                                AnimationTick = MapClass.M2CellInfo[X, Y].AniTick;
                                Draw += (_AnimationCount % (AnimationFrame + (AnimationFrame * AnimationTick))) / (1 + AnimationTick);
                            }

                            FileIndex = MapClass.M2CellInfo[X, Y].Area;
                            tempSize = CLib.Objects[FileIndex].GetImageSize(Draw);


                            tempPoint = new Point((X - User.Info.Location.X + Config.PlayerOffSet.X) * 48, ((Y - User.Info.Location.Y + Config.PlayerOffSet.Y) * 32) - tempSize.Height + 32);
                            tempPoint.Offset(OffSet);

                            if (Draw >= 2723 && Draw <= 2732)
                            {
                                tempPoint.Offset(CLib.Objects[FileIndex].GetImageOffSet(Draw));
                                if (tempPoint.Y <= Config.CScreenSize.Height)
                                    CLib.Objects[FileIndex].DrawBlend(Draw, tempPoint, false, Color.White);
                            }
                            else if (tempPoint.Y <= Config.CScreenSize.Height)
                                CLib.Objects[FileIndex].Draw(Draw, tempPoint, false);
                        }
                        if (MapObjects[X, Y] != null)
                            for (int I = MapObjects[X, Y].Count - 1; I >= 0; I--)
                            {
                                object o = MapObjects[X, Y][I];
                                switch (o.GetType().Name)
                                {
                                    case "Player":
                                        ((Player)o).Draw();
                                        break;
                                    case "Monster":
                                        ((Monster)o).Draw();
                                        break;
                                }
                            }
                        if (X == User.Info.Location.X && Y == User.Info.Location.Y)
                        {
                            User.Draw();
                        }
                    }
                }
                #endregion
            }
            else
            #endregion
            #region Mir3
            {
                #region Objects
                for (int Y = MinY; Y <= MaxY; Y++)
                {
                    for (int X = MinX; X <= MaxX; X++)
                    {
                        filenumber = M3CellInfo[X, Y].FileIndex;
                        if (filenumber != 65535 && filenumber != 255)
                        {
                            buffer = BitConverter.GetBytes(filenumber);
                            if (buffer[1] > 0 && buffer[1] != 255)
                            {
                                ImageIndex = M3CellInfo[X, Y].Obj1 +1;
                                if (ImageIndex > 0 && CLib.Mir3Objects[buffer[1]] != null)
                                {
                                    tempSize = CLib.Mir3Objects[buffer[1]].GetImageSize(ImageIndex);
                                    tempPoint = new Point((X - User.Info.Location.X + Config.PlayerOffSet.X) * 48, ((Y - User.Info.Location.Y + Config.PlayerOffSet.Y) * 32) - tempSize.Height + 32);
                                    tempPoint.Offset(OffSet);
                                    CLib.Mir3Objects[buffer[1]].Draw(ImageIndex , tempPoint, false);
                                }
                            }
                        }
                    }
                    for (int X = MinX; X <= MaxX; X++)
                    {
                        filenumber = M3CellInfo[X, Y].FileIndex;
                        if (filenumber != 65535 && filenumber != 255)
                        {
                            buffer = BitConverter.GetBytes(filenumber);
                            if (buffer[0] > 0 && buffer[0] != 255)
                            {
                                ImageIndex = M3CellInfo[X, Y].Obj2 +1;
                                if (ImageIndex > 0 && CLib.Mir3Objects[buffer[0]] != null)
                                {
                                    tempSize = CLib.Mir3Objects[buffer[0]].GetImageSize(ImageIndex);
                                    tempPoint = new Point((X - User.Info.Location.X + Config.PlayerOffSet.X) * 48, ((Y - User.Info.Location.Y + Config.PlayerOffSet.Y) * 32) - tempSize.Height + 32);
                                    tempPoint.Offset(OffSet);

                                    CLib.Mir3Objects[buffer[0]].Draw(ImageIndex , tempPoint, false);
                                }
                            }
                        }
                        if (MapObjects[X, Y] != null)
                            for (int I = MapObjects[X, Y].Count - 1; I >= 0; I--)
                            {
                                object o = MapObjects[X, Y][I];
                                switch (o.GetType().Name)
                                {
                                    case "Player":
                                        ((Player)o).Draw();
                                        break;
                                    case "Monster":
                                        ((Monster)o).Draw();
                                        break;
                                }
                            }
                        if (X == User.Info.Location.X && Y == User.Info.Location.Y)
                            User.Draw();
                        else if (X == User.MovePoint.X && Y == User.MovePoint.Y)
                            User.Draw();
     
                    }
                }

                #endregion
            }
            #endregion
        }
        public static void AddObject(object o, Point P)
        {
            if (!new Rectangle(Point.Empty, MapSize).Contains(P)) return;

            if (MapObjects[P.X, P.Y] == null) MapObjects[P.X, P.Y] = new List<object>();
            else if (MapObjects[P.X, P.Y].Contains(o)) return;
            MapObjects[P.X, P.Y].Insert(0,o);
        }
        public static void RemoveObject(object o, Point P)
        {
            if (!new Rectangle(Point.Empty, MapSize).Contains(P)) return;

            if (MapObjects[P.X, P.Y] == null) return;
            MapObjects[P.X, P.Y].Remove(o);
            if (MapObjects[P.X, P.Y].Count == 0) MapObjects[P.X, P.Y] = null;
        }
        public static bool EmptyCell(Point P)
        {
            if (new Rectangle(Point.Empty, MapSize).Contains(P))
            {
                if (Type == MapType.Mir2 && (MapClass.M2CellInfo[P.X, P.Y].BackImage & 0x8000) + (MapClass.M2CellInfo[P.X, P.Y].FrontImage & 0x8000) != 0)
                    return false;
                else if (Type == MapType.Mir3 && MapClass.M3CellInfo[P.X, P.Y].Flag != 3)
                    return false;

                if (MapObjects[P.X, P.Y] != null)
                    for (int I = MapObjects[P.X, P.Y].Count - 1; I >= 0; I--)
                    {
                        object o = MapObjects[P.X, P.Y][I];
                        switch (o.GetType().Name)
                        {
                            case "Player":
                                if (((Player)o).Info.Dead || ((Player)o).Info.CurrentHP <= 0) continue;
                                else return false;
                            case "Monster":
                                if (((Monster)o).Info.Dead || ((Monster)o).Info.CurrentHP <= 0) continue;
                                else return false;
                        }
                    }

                return true;
            }
            else return false;
        }
    }
    #region Tile Classes
    public class Mir2CellInfo
    {
        #region Fields

        private short _backImage;
        private short _midImage;
        private short _frontImage;
        private byte _doorIndex;
        private byte _doorOffset;
        private byte _aniFrame;
        private byte _AniTick;
        private byte _area;
        private byte _light;

        #endregion
        #region Properties

        public short BackImage
        {
            get { return _backImage; }
        }
        public short MidImage
        {
            get { return _midImage; }
        }
        public short FrontImage
        {
            get { return _frontImage; }
        }
        public byte DoorIndex
        {
            get { return _doorIndex; }
        }
        public byte DoorOffset
        {
            get { return _doorOffset; }
        }
        public byte AniFrame
        {
            get { return _aniFrame; }
        }
        public byte AniTick
        {
            get { return _AniTick; }
        }
        public byte Area
        {
            get { return _area; }
        }
        public byte Light
        {
            get { return _light; }
        }

        #endregion
        public Mir2CellInfo(short b, short m, short f, byte di, byte dos, byte af, byte at, byte a, byte l)
        {
            _backImage = b;
            _midImage = m;
            _frontImage = f;
            _doorIndex = di;
            _doorOffset = dos;
            _aniFrame = af;
            _AniTick = at;
            _area = a;
            _light = l;
        }
    }
    public class Mir3CellInfo
    {
        #region Fields

        private byte _Flag;
        private byte _ObjAni1;
        private byte _ObjAni2;
        private ushort _fileIndex;
        private ushort _obj1;
        private ushort _obj2;
        private ushort _doorIdx;
        private byte _doorOffset;
        private ushort _light;

        #endregion
        #region Properties

        public byte Flag
        {
            get { return _Flag; }
        }
        public byte ObjAni1
        {
            get { return _ObjAni1; }
        }
        public byte ObjAni2
        {
            get { return _ObjAni2; }
        }
        public ushort FileIndex
        {
            get { return _fileIndex; }
        }
        public ushort Obj1
        {
            get { return _obj1; }
        }
        public ushort Obj2
        {
            get { return _obj2; }
        }
        public ushort DoorIdx
        {
            get { return _doorIdx; }
        }
        public byte DoorOffset
        {
            get { return _doorOffset; }
        }
        public ushort Light
        {
            get { return _light; }
        }

        #endregion

        public Mir3CellInfo(byte fl, byte ob1A, byte ob2A, ushort file, ushort ob1, ushort ob2, ushort door, byte dooroff, ushort lig)
        {
            _Flag = fl;
            _ObjAni1 = ob1A;
            _ObjAni2 = ob2A;
            _fileIndex = file;
            _obj1 = ob1;
            _obj2 = ob2;
            _doorIdx = door;
            _doorOffset = dooroff;
            _light = lig;
        }
    }
    public class Mir3TileInfo
    {
        #region Fields

        private byte _FileIndex;
        private ushort _ImageIndex;

        #endregion
        #region Properties

        public byte FileIndex
        {
            get { return _FileIndex; }
        }
        public ushort ImageIndex
        {
            get { return _ImageIndex; }
        }

        #endregion

        public Mir3TileInfo(byte b, ushort img)
        {
            _FileIndex = b;
            _ImageIndex = img;
        }
    }

    #endregion
}
