﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using System.Text;
using Clash_Library;
using C = Client_Packets;

namespace Clash_Server
{
    public class Accounts
    {
        #region Fields
        public static List<AccountIndex> AccountList;
        #endregion

        public static void Load()
        {
            string AccPath = Config.AccountPath;
            if (!Directory.Exists(AccPath)) Directory.CreateDirectory(AccPath);
            AccPath = Path.Combine(AccPath, @"AIndex.CIF");
            if (File.Exists(AccPath))
            {
                try
                {
                    BinaryFormatter BF = new BinaryFormatter();
                    FileStream FStream = File.OpenRead(AccPath);
                    AccountList = (List<AccountIndex>)BF.Deserialize(FStream);
                    FStream.Close();
                }
                catch (Exception ex)
                { Main.MessageList.Enqueue(ex.ToString()); }
            }

            if (AccountList == null) AccountList = new List<AccountIndex>();
            Characters.Load();
        }
        public static void Save()
        {
            string AccPath = Config.AccountPath;
            if (!Directory.Exists(AccPath)) Directory.CreateDirectory(AccPath);
            AccPath = Path.Combine(AccPath, @"AIndex.CIF");
            if (AccountList != null)
            {
                try
                {
                    using (FileStream FStream = File.Create(AccPath))
                        new BinaryFormatter().Serialize(FStream, AccountList);
                }
                catch (Exception ex)
                { Main.MessageList.Enqueue(ex.ToString()); }
            }
            Characters.Save();
        }
        public static bool AccountByID(string AccountID, out AccountIndex AI)
        {
            lock (AccountList)
            {
                for (int I = 0; I < AccountList.Count; I++)
                    if (AccountList[I].AccountID.CompareTo(AccountID) == 0)
                    {
                        AI = AccountList[I];
                        return true;
                    }
                AI = null;
                return false;
            }
        }
        public static bool AccountByIndex(int Index, out AccountIndex AI)
        {
            lock (AccountList)
            {
                if (AccountList[Index].Index == Index)
                {
                    AI = AccountList[Index];
                    return true;
                }

                for (int I = 0; I < AccountList.Count; I++)
                    if (AccountList[I].Index == Index)
                    {
                        AI = AccountList[I];
                        return true;
                    }

                AI = null;
                return false;
            }
        }
        public static bool AccountUsed(string AccountID)
        {
            for (int I = 0; I < AccountList.Count; I++)
                if (AccountList[I].AccountID.CompareTo(AccountID) == 0)
                    return true;

            return false;
        }
        public static bool NewAccount(C.NewAccount Packet, out AccountIndex AI)
        {
            lock (AccountList)
            {
                AI = null;
                if (AccountUsed(Packet.AccountID)) return false;
                AI = new AccountIndex(Packet);
                AccountList.Add(AI);
                return true;
            }
        }

    }
    [Serializable]
    public class AccountIndex
    {
        #region Fields
        public int Index;
        public string AccountID;
        private byte[] Password;
        public bool Banned;
        public DateTime ExpiryDate;
        public int WrongPassCount;
        public string BanReason;
        public int LastCharID;
        [NonSerialized]
        public Account Account;
        #endregion
        #region Constructor
        public AccountIndex(C.NewAccount Packet)
        {
            Index = Accounts.AccountList.Count;
            AccountID = Packet.AccountID;
            Password = new MD5CryptoServiceProvider().ComputeHash(ASCIIEncoding.ASCII.GetBytes(Packet.Password));
            Banned = false;
            ExpiryDate = DateTime.MinValue;
            WrongPassCount = 0;
            LastCharID = 0;
            BanReason = null;
        }
        #endregion
        #region Methods
        public bool LogIn(string Pass)
        {
            byte[] PassHash = new MD5CryptoServiceProvider().ComputeHash(ASCIIEncoding.ASCII.GetBytes(Pass));
            if (string.Compare(Convert.ToBase64String(Password), Convert.ToBase64String(PassHash)) == 0)
            {
                LoadAccount();
                Account.Connected = true;
                return true;
            }
            else
            {
                WrongPassCount++;
                return false;
            }
        }
        public bool ChangePassword(C.ChangePass Packet)
        {
            byte[] PassHash = new MD5CryptoServiceProvider().ComputeHash(ASCIIEncoding.ASCII.GetBytes(Packet.CPassword));
            if (string.Compare(Convert.ToBase64String(Password), Convert.ToBase64String(PassHash)) == 0)
            {
                Password = new MD5CryptoServiceProvider().ComputeHash(ASCIIEncoding.ASCII.GetBytes(Packet.NPassword));
                return true;
            }
            else
            {
                WrongPassCount++;
                return false;
            }
        }
        private void LoadAccount()
        {
            if (Account == null)
            {
                string AccPath = Path.Combine(Config.AccountPath, Index.ToString());
                if (!Directory.Exists(AccPath)) Directory.CreateDirectory(AccPath);
                if (File.Exists(Path.Combine(AccPath, @"Account.CAF")))
                {
                    try
                    {
                        BinaryFormatter BF = new BinaryFormatter();
                        FileStream FStream = File.OpenRead(Path.Combine(AccPath, @"Account.CAF"));
                        Account = (Account)BF.Deserialize(FStream);
                        FStream.Close();
                    }
                    catch (Exception ex)
                    { Main.MessageList.Enqueue(ex.ToString()); }
                }

            }
        }
        [OnSerialized]
        public void SaveAccount(StreamingContext context)
        {
            string AccPath = Path.Combine(Config.AccountPath, Index.ToString());
            if (!Directory.Exists(AccPath)) Directory.CreateDirectory(AccPath);
            if (Account != null)
                try
                {
                    BinaryFormatter BF = new BinaryFormatter();
                    FileStream FStream = File.Create(Path.Combine(AccPath, @"Account.CAF"));
                    BF.Serialize(FStream, Account);
                    FStream.Close();
                    if (!Account.Connected)
                        Account = null;
                }
                catch (Exception ex)
                { Main.MessageList.Enqueue(ex.ToString()); }
        }
        #endregion
    }

    [Serializable]
    public class Account
    {
        #region Fields
        public int Index;
        public string Name, DoB, Q1, A1, Q2, A2, Phone, Mobile, Email;
        public string CreatorIP;
        public int BanCount;
        public DateTime CreationDate;
        public int LastChar;
        public List<Player> CharacterList;
        public List<Item> StorageItems;
        public long Gold;
        [NonSerialized]
        public bool Connected;
        #endregion
        #region Constructor
        public Account(int Idx, C.NewAccount Packet, string IPAdd)
        {
            Index = Idx;
            Name = Packet.Name;
            DoB = Packet.DateOfBirth;
            Q1 = Packet.Question1;
            A1 = Packet.Answer1;
            Q2 = Packet.Question2;
            A2 = Packet.Answer2;
            Phone = Packet.Phone;
            Mobile = Packet.Mobile;
            Email = Packet.Email;
            CreatorIP = IPAdd;
            CreationDate = DateTime.Now;
            CharacterList = new List<Player>();
            StorageItems = new List<Item>();
            LastChar = 0;
            BanCount = 0;
            Gold = 0;

            string AccPath = Path.Combine(Config.AccountPath, Index.ToString());
            if (!Directory.Exists(AccPath)) Directory.CreateDirectory(AccPath);
            try
            {
                BinaryFormatter BF = new BinaryFormatter();
                using (FileStream FStream = File.Create(Path.Combine(AccPath, @"Account.CAF")))
                    BF.Serialize(FStream, this);
            }
            catch (Exception ex)
            { Main.MessageList.Enqueue(ex.ToString()); }
        }
        #endregion
    }
}
