﻿using System;
using System.Collections.Generic;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;

namespace Clash_Server
{
    static class Characters
    {
        #region Fields
        public static List<CharacterIndex> CharacterList;
        #endregion
        #region Methods
        public static void Load()
        {
            string AccPath = Config.AccountPath;
            if (!Directory.Exists(AccPath)) Directory.CreateDirectory(AccPath);
            AccPath = Path.Combine(AccPath, @"CIndex.CIF");
            if (File.Exists(AccPath))
            {
                try
                {
                    using (FileStream FStream = File.OpenRead(AccPath))
                        CharacterList = (List<CharacterIndex>)new BinaryFormatter().Deserialize(FStream);
                }
                catch (Exception ex)
                { Main.MessageList.Enqueue(ex.ToString()); }
            }

            if (CharacterList == null) CharacterList = new List<CharacterIndex>();


        }
        public static void Save()
        {
            string AccPath = Config.AccountPath;
            if (!Directory.Exists(AccPath)) Directory.CreateDirectory(AccPath);
            AccPath = Path.Combine(AccPath, @"CIndex.CIF");
            if (CharacterList != null)
            {
                try
                {
                    using (FileStream FStream = File.Create(AccPath))
                        new BinaryFormatter().Serialize(FStream, CharacterList);
                }
                catch (Exception ex)
                { Main.MessageList.Enqueue(ex.ToString()); }
            }
        }
        public static bool CharacterUsed(string Name, int Idx)
        {
            CharacterIndex CI;
            for (int I = 0; I < CharacterList.Count; I++)
            {
                CI = CharacterList[I];
                if (string.Compare(CI.CharacterName, Name) == 0)
                {
                    if ((CI.Deleted && Config.AllowReuseName) || (CI.AccountIndex == Idx && Config.AllowReuseNameOnAcc))
                        continue;
                    return true;
                }
            }

            return false;
        }
        public static bool NewCharacter(int AccountIdx, string Name, out CharacterIndex CI)
        {
            lock (CharacterList)
            {
                CI = null;
                if (CharacterUsed(Name, AccountIdx))
                    return false;

                CI = new CharacterIndex(AccountIdx, Name);
                CharacterList.Add(CI);
                return true;
            }
        }
        public static bool CharacterByIndex(int Index, out CharacterIndex CI)
        {
            lock (CharacterList)
            {
                if (CharacterList[Index].Index == Index)
                {
                    CI = CharacterList[Index];
                    return true;
                }

                for (int I = 0; I < CharacterList.Count; I++)
                    if (CharacterList[I].Index == Index)
                    {
                        CI = CharacterList[I];
                        return true;
                    }

                CI = null;
                return false;
            }
        }
        #endregion
    }

    [Serializable]
    public class CharacterIndex
    {
        public int Index, AccountIndex;
        public string CharacterName;
        public bool Deleted;
        public bool Banned;
        public DateTime ExpiryDate;
        public string BanReason;
        public CharacterIndex(int AccountIdx, string Name)
        {
            Index = Characters.CharacterList.Count;
            AccountIndex = AccountIdx;
            CharacterName = Name;
            Deleted = false;
            Banned = false;
            ExpiryDate = DateTime.MinValue;
            BanReason = null;
        }
    }
}
