﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using Clash_Library;
using System.Net.Sockets;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text.RegularExpressions;
using System.Threading;
using System.Reflection;
using S = Server_Packets;
using C = Client_Packets;

namespace Clash_Server
{
    public enum ConnectionStage { Login, CharacterSelect, Game, Exit }
    class MirClient
    {
        private int SessionID;
        private TcpClient Client;
        private Queue<Packet> OPList, IPList;
        private BinaryReader BReader;
        private NetworkStream NStream;
        private BinaryWriter BWriter;
        private DateTime TimeConnected, LastCommunication;
        private AccountIndex AI;
        private Player Player;
        public ConnectionStage Stage;
        private string IPAddress;
        [NonSerialized]
        List<int> IIndexList;
        

        public MirClient(TcpClient _client, int _sessionID)
        {
            Stage = ConnectionStage.Login;
            SessionID = _sessionID;
            Client = _client;
            NStream = Client.GetStream();
            TimeConnected = DateTime.Now;
            LastCommunication = DateTime.Now;
            IPAddress = Client.Client.RemoteEndPoint.ToString().Split(':')[0];
            BReader = new BinaryReader(NStream);
            OPList = new Queue<Packet>();
            IPList = new Queue<Packet>();
            BWriter = new BinaryWriter(NStream);
            IIndexList = new List<int>();
            ThreadPool.QueueUserWorkItem(Process);
        }

        private void Process(object o)
        {
            Packet P;
            string S;
            while (Client != null && Client.Connected)
            {

                try
                {

                    if (NStream.DataAvailable)
                    {
                        S = BReader.ReadString();
                        P = Activator.CreateInstance(Type.GetType(string.Format("Client_Packets.{0},Clash Library", S))) as Packet;
                        if (P != null)
                            P.Read(BReader);
                        IPList.Enqueue(P);
                        LastCommunication = DateTime.Now;
                    }


                    while (IPList != null && IPList.Count > 0)
                        Read(IPList.Dequeue());

                    while (OPList != null && OPList.Count > 0)
                    {
                        P = OPList.Dequeue();
                        if (P != null)
                            lock (BWriter)
                            {
                                BWriter.Write(P.GetType().Name);
                                P.WriteTo(BWriter);
                            }
                    }

                    if (DateTime.Now > LastCommunication + new TimeSpan(0, 0, 20))
                    {
                      //SendDisconnect(2);
                      //  break;
                    }
                }
                catch (Exception ex)
                {
                    Main.MessageList.Enqueue(string.Format("IP:{0}, {1}", IPAddress, ex));
                }


                Thread.Sleep(1);
            }
            DisconnectUser();
        }
        private void SendData(Packet Ob)
        {
            OPList.Enqueue(Ob);
        }
        private void Read(Packet P)
        {
            try
            {
                if (Stage == ConnectionStage.Game)
                {
                    if (P.GetType().Name != "Disconnect" && P.GetType().Name != "KAlive")
                    {
                        Player.Read(P);
                        return;
                    }
                }
                GetType().GetMethod(P.GetType().Name).Invoke(this, new Object[] { P });
            }
            catch (Exception ex)
            {
                Main.MessageList.Enqueue(string.Format("IP:{0}, Packet {1} \n {2}", IPAddress, P.GetType().Name, ex));
            }
        }

        public void SendDisconnect(byte _Reason)
        {
            OPList.Clear();
            try
            {
                lock (BWriter)
                {
                    BWriter.Write("Disconnect");
                    new S.Disconnect(_Reason).WriteTo(BWriter);
                }
            }
            catch (IOException)
            { }
            finally
            {
                DisconnectUser();
            }
        }

        private void DisconnectUser()
        {
            if (Stage == ConnectionStage.Exit) return;

            switch (Stage)
            {
                case  ConnectionStage.Game:
                    if (Player != null) Player.LogOut();
                    Player = null;
                    goto case ConnectionStage.CharacterSelect;
                case ConnectionStage.CharacterSelect:
                    AI.Account.Connected = false;
                    AI = null;
                    goto case ConnectionStage.Login;
                case ConnectionStage.Login:
                    if (NStream != null) NStream.Dispose();
                    if (BWriter != null) BWriter.Dispose();
                    if (BReader != null) BReader.Dispose();
                    NStream = null;
                    BWriter = null;
                    BReader = null;
                    OPList = null;
                    IPList = null;
                    IIndexList = null;
                    break;
            }


            Stage = ConnectionStage.Exit;
        }

        public void Disconnect(C.Disconnect Packet)
        {
            DisconnectUser();
        }
        public void KAlive(C.KAlive Packet)
        { }

        #region Login
        public void Login(C.Login Packet)
        {
            try
            {
                if (!Config.AllowLogin)
                {
                    SendData(new S.Login(0));
                    return;
                }

                if (string.IsNullOrEmpty(Packet.AccountName))
                {
                    SendData(new S.Login(1)); //Empty or Null AccountID
                    return;
                }

                if (string.IsNullOrEmpty(Packet.Password))
                {
                    SendData(new S.Login(2)); //Empty or Null Password
                    return;
                }


                if (!Accounts.AccountByID(Packet.AccountName, out AI))
                {
                    SendData(new S.Login(3));//AccountID NotFound.
                    return;
                }

                if (AI.Banned && DateTime.Now < AI.ExpiryDate)
                {
                    SendData(new S.Login(4, AI.BanReason, AI.ExpiryDate));//Account Banned.
                    return;
                }


                if (AI.Banned)
                {
                    AI.Banned = false;
                    AI.BanReason = null;
                }


                if (!AI.LogIn(Packet.Password))
                {
                    SendData(new S.Login(5));//WrongPassword.                
                    AI = null;
                    return;
                }


                SendData(new S.Login(6, AI.WrongPassCount)); //Success;  
                Stage = ConnectionStage.CharacterSelect;


                SendCharacterList();

                for (int I = 0; I < Network.MirClients.Count; I++)
                {
                    MirClient MC = Network.MirClients[I];
                    if (MC.AI != null && MC.AI.Index == AI.Index && MC.SessionID != SessionID)
                        MC.SendDisconnect(0);
                }
            }
            catch (Exception ex)
            {
                Main.MessageList.Enqueue(string.Format("IP:{0}, {1}", IPAddress, ex));
                SendData(new S.Login(7)); //Error;  
            }

        }
        public void NewAccount(C.NewAccount Packet)
        {
            try
            {
                if (!Config.AllowNewAccounts)
                {
                    SendData(new S.NewAccount(0)); //New Accounts - Disabled.
                    return;
                }


                if (string.IsNullOrEmpty(Packet.AccountID))
                {
                    SendData(new S.NewAccount(1)); //AccountID null or empty
                    return;
                }


                if (string.IsNullOrEmpty(Packet.Password))
                {
                    SendData(new S.NewAccount(2));  //Password null or empty
                    return;
                }

                AccountIndex AI;

                if (!Accounts.NewAccount(Packet, out AI))
                {
                    SendData(new S.NewAccount(3));  //Already Exists.
                    return;
                }
                SendData(new S.NewAccount(4));
                AI.Account = new Account(AI.Index, Packet, IPAddress);


                return;
            }
            catch (Exception ex)
            {
                Main.MessageList.Enqueue(string.Format("IP:{0}, {1}", IPAddress, ex));
                SendData(new S.NewAccount(5));
                return;
            }
        }
        public void ChangePass(C.ChangePass Packet)
        {
            try
            {
                if (!Config.AllowChangePassword) // Disabled
                {
                    SendData(new S.ChangePass(0));
                    return;
                }
                if (string.IsNullOrEmpty(Packet.AccountID)) // AccountID
                {
                    SendData(new S.ChangePass(1));
                    return;
                }
                if (string.IsNullOrEmpty(Packet.CPassword)) // Currnet Password
                {
                    SendData(new S.ChangePass(2));
                    return;
                }

                if (string.IsNullOrEmpty(Packet.NPassword)) //New Password
                {
                    SendData(new S.ChangePass(3));
                    return;
                }
                if (!Accounts.AccountByID(Packet.AccountID, out AI))
                {
                    SendData(new S.ChangePass(4));//Not Found
                    return;
                }

                if (AI.Banned && DateTime.Now < AI.ExpiryDate)
                {
                    SendData(new S.ChangePass(5, AI.BanReason, AI.ExpiryDate));
                    AI = null;
                    return;
                }

                if (AI.Banned)
                {
                    AI.Banned = false;
                    AI.BanReason = null;
                }
                if (!AI.ChangePassword(Packet))
                {
                    SendData(new S.ChangePass(4)); //Wrong Password.
                    AI = null;
                    return;
                }

                SendData(new S.ChangePass(6)); //Success.                

                AI = null;
                return;


            }
            catch (Exception ex)
            {
                Main.MessageList.Enqueue(string.Format("IP:{0}, {1}", IPAddress, ex));
                SendData(new S.ChangePass(7));
                return;
            }


        }
        public void SendCharacterList()
        {
            List<CharSelInfo> CharList = new List<CharSelInfo>();

            for (int I = 0; I < AI.Account.CharacterList.Count; I++)
            {
                Player P = AI.Account.CharacterList[I];
                if (P.CI.Deleted) continue;
                CharList.Add(new CharSelInfo(P.CI.Index, P.CI.CharacterName, P.Level, P.Class, P.Gender));
                if (CharList.Count >= 4) break;
            }

            SendData(new S.CharacterList(CharList, AI.LastCharID));
        }
        #endregion
        #region Character
        public void NewCharacter(C.NewCharacter Packet)
        {
            try
            {
                System.Text.RegularExpressions.Regex r = new System.Text.RegularExpressions.Regex(@"^[A-Za-z0-9]{3,20}$");
                int tempInt = 0;

                if (!Config.AllowNewCharacters)
                {
                    SendData(new S.NewCharacter(0));
                    return;
                }

                if (string.IsNullOrEmpty(Packet.Name))
                {
                    SendData(new S.NewCharacter(1));
                    return;
                }
                else if (!r.IsMatch(Packet.Name))
                {
                    SendData(new S.NewCharacter(2));
                    return;
                }

                if ((byte)Packet.Job < 0 || (byte)Packet.Job > 3)
                {
                    SendData(new S.NewCharacter(3));
                    return;
                }

                if ((byte)Packet.Sex < 0 || (byte)Packet.Sex > 1)
                {
                    SendData(new S.NewCharacter(4));
                    return;
                }

                if (AI.Banned)
                {
                    SendData(new S.NewCharacter(5));
                    return;
                }

                for (int I = 0; I < AI.Account.CharacterList.Count; I++)
                    if (!AI.Account.CharacterList[I].CI.Deleted)
                        tempInt++;

                if (tempInt >= 4)
                {
                    SendData(new S.NewCharacter(6));
                    return;
                }
                CharacterIndex CI;
                if (!Characters.NewCharacter(AI.Index, Packet.Name, out CI))
                {
                    SendData(new S.NewCharacter(7));
                    return;
                }

                AI.Account.CharacterList.Add(new Player(CI, Packet.Job, Packet.Sex));

                SendData(new S.NewCharacter(8, CI.Index, AI.Account.CharacterList[AI.Account.CharacterList.Count - 1].Level));
            }
            catch (Exception ex)
            {
                Main.MessageList.Enqueue(string.Format("IP:{0}, {1}", IPAddress, ex));
                SendData(new S.NewCharacter(9));
            }

        }
        public void DeleteCharacter(C.DeleteCharacter Packet)
        {
            try
            {
                if (!Config.AllowDeleteCharacter)
                {
                    SendData(new S.DeleteCharacter(0));
                    return;
                }
                if (AI.Banned)
                {
                    SendData(new S.DeleteCharacter(1));
                    return;
                }

                for (int I = 0; I < AI.Account.CharacterList.Count; I++)
                {
                    Player P = AI.Account.CharacterList[I];
                    if (P.Index == Packet.CharID)
                    {
                        P.CI.Deleted = true;
                        P.DeleteCount++;
                        SendData(new S.DeleteCharacter(2));
                        return;
                    }
                }

                SendData(new S.DeleteCharacter(3));
                return;

            }
            catch (Exception ex)
            {
                Main.MessageList.Enqueue(string.Format("IP:{0}, {1}", IPAddress, ex));
                SendData(new S.DeleteCharacter(4));
                return;
            }
        }
        public void Logout(C.Logout Packet)
        {
            if (!Config.AllowLogout || Stage != ConnectionStage.CharacterSelect)
            {
                return;
            }
            AI.Account.Connected = false;
            AI = null;
            Stage = ConnectionStage.Login;
            SendData(new S.Logout(1));
        }
        public void StartGame(C.StartGame Packet)
        {
            try
            {
                if (!Config.AllowStartGame || Engine.State != EngineState.Running)
                {
                    SendData(new S.StartGame(0));
                    return;
                }
                if (AI.Banned)
                {
                    SendData(new S.StartGame(1));
                    Logout(new C.Logout());
                    return;
                }
                for (int I = 0; I < AI.Account.CharacterList.Count; I++)
                {
                    CharacterIndex CI = AI.Account.CharacterList[I].CI;
                    if (CI.Index == Packet.CharID)
                    {
                        if (CI.Deleted) // Deleted
                        {
                            SendData(new S.StartGame(2));
                            return;
                        }
                        if (CI.Banned && DateTime.Now < CI.ExpiryDate) // Banned
                        {
                            SendData(new S.StartGame(3, CI.BanReason, CI.ExpiryDate));
                            return;
                        }
                        if (CI.Banned) CI.Banned = false;
                        AI.LastCharID = Packet.CharID;
                        SendData(new S.StartGame(4));
                        Player = AI.Account.CharacterList[I];
                        Player.LogIn(OPList, IIndexList);
                        Stage = ConnectionStage.Game;
                        return;
                    }
                }
                SendData(new S.StartGame(5));//Not Found\
            }
            catch (Exception ex)
            {
                Main.MessageList.Enqueue(string.Format("IP:{0}, {1}", IPAddress, ex));
                SendData(new S.StartGame(6)); //Error
            }
        }
        #endregion
    }
}
