﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using Clash_Library;
using Server_Packets;
using System.Threading;

namespace Clash_Server
{
    public class Map
    {
        #region Static Fields
        public static TileInfo HighWall = new TileInfo(CellAttribute.HighWall);
        public static TileInfo LowWall = new TileInfo(CellAttribute.LowWall);
        #endregion
        public MapInfo MInfo;
        public Size MapSize;
        public TileInfo[,] Tiles;
        public Random Rand;
        public List<Respawn> SpawnList;
        public List<FloorItem> FloorItems;
        public List<Player> Players;
        public List<Monster> Monsters;
        public List<Magic> Magics;
        private DateTime OperateTime;

        public Map(MapInfo MI)
        {
            MInfo = MI;
            switch (MInfo.MapType)
            {
                case MapType.Mir2:
                    if (!ReadMir2Map()) return;
                    break;
                case MapType.Mir3:
                    if (!ReadMir3Map()) return;
                    break;
            }
            FloorItems = new List<FloorItem>();
            Players = new List<Player>();
            Monsters = new List<Monster>();
            Rand = new Random(DateTime.Now.Millisecond);
            LoadRespawn();
            Engine.MapList.Add(MInfo, this);

        }
        private void LoadRespawn()
        {
            SpawnList = new List<Respawn>();
            foreach (RespawnInfo RI in MInfo.RespawnList)
                SpawnList.Add(new Respawn(RI, MapSize));
        }
        private bool ReadMir2Map()
        {
            try
            {
                string path = Path.Combine(Config.MapPath, MInfo.FileName + ".map");
                if (File.Exists(path))
                {
                    byte[] Bytes = File.ReadAllBytes(path);
                    MapSize = new Size(BitConverter.ToUInt16(Bytes, 0), BitConverter.ToUInt16(Bytes, 2));
                    Tiles = new TileInfo[MapSize.Width, MapSize.Height];
                    int OffSet = 52;

                    for (short X = 0; X < MapSize.Width; X++)
                        for (short Y = 0; Y < MapSize.Height; Y++)
                        {
                            if ((BitConverter.ToUInt16(Bytes, OffSet) & 0x8000) != 0)
                                Tiles[X, Y] = LowWall; //Can Fire Over.

                            if ((BitConverter.ToUInt16(Bytes, (OffSet += 4)) & 0x8000) != 0)
                                Tiles[X, Y] = HighWall;//Can't Fire Over.

                            if (Tiles[X, Y] == null)
                                Tiles[X, Y] = new TileInfo(CellAttribute.Walk);
                            OffSet += 8; //No other information Wanted.
                        }
                }
                else
                {
                    Main.MessageList.Enqueue(string.Format("Map {0}.map Not Found.", MInfo.FileName));
                    return false;
                }
            }
            catch (Exception ex)
            {
                Main.MessageList.Enqueue(ex.Message + " Possibly wrong file or MapType.");
                return false;
            }
            return true;
        }
        private bool ReadMir3Map()
        {
            try
            {
                string path = Path.Combine(Config.MapPath, MInfo.FileName + ".map");
                if (File.Exists(path))
                {
                    byte[] Bytes = File.ReadAllBytes(path);
                    MapSize = new Size(BitConverter.ToUInt16(Bytes, 22), BitConverter.ToUInt16(Bytes, 24));
                    Tiles = new TileInfo[MapSize.Width, MapSize.Height];
                    int OffSet = 28 + (MapSize.Width * MapSize.Height / 4) * 3;

                    for (int X = 0; X < MapSize.Width; X++)
                        for (int Y = 0; Y < MapSize.Height; Y++)
                        {
                            switch (Bytes[OffSet])
                            {
                                case 3:
                                    Tiles[X, Y] = new TileInfo(CellAttribute.Walk);
                                    break;
                                case 0:
                                    Tiles[X, Y] = LowWall;
                                    break;
                                default:
                                    Tiles[X, Y] = HighWall;
                                    break;
                            }
                            OffSet += 14;
                        }
                }
                else
                {
                    Main.MessageList.Enqueue(string.Format("Map {0}.map Not Found.", MInfo.FileName));
                    return false;
                }
            }
            catch (Exception ex)
            {
                Main.MessageList.Enqueue(ex.ToString() + " \nPossibly wrong file or MapType.");
                return false;
            }
            return true;
        }
        public void Respawn()
        {
            bool Spawned;
            try
            {
                foreach (Respawn R in SpawnList)
                {
                    if (SpawnList.Count == 4)
                    {
                        if (SpawnList[0].RInfo.Monster.Name == "StoneGargoyle")
                        { }

                    }
                    if (DateTime.Now > R.SpawnTime && R.RemainingAmount > 0)
                    {
                        TimeSpan Delay = R.RInfo.RespawnTime + new TimeSpan(0, Rand.Next(-R.RInfo.DistortionTime, R.RInfo.DistortionTime), 0);
                        if (Delay <= TimeSpan.Zero) Delay = new TimeSpan(0, 1, 0);

                        R.SpawnTime += Delay;
                        int Count = 0;
                        Point SpawnPoint;

                        while (Count < R.RemainingAmount)
                        {
                            Spawned = false;
                            for (int x = 0; x < Config.SpawnAttempts; x++)
                            {
                                SpawnPoint = new Point(Rand.Next(R.MinX, R.MaxX), Rand.Next(R.MinY, R.MaxY));
                                if (IsValidPoint(SpawnPoint))
                                {
                                    Spawned = true;
                                    switch (R.RInfo.Monster.AI)
                                    {
                                        case MonsterAI.Oma:
                                            new Oma(R.RInfo.Monster).Spawn(R, this, SpawnPoint);
                                            break;
                                        default:
                                            new Monster(R.RInfo.Monster).Spawn(R, this, SpawnPoint);
                                            break;

                                    }
                                    Interlocked.Decrement(ref R.RemainingAmount);
                                    break;
                                }
                            }
                            if (!Spawned) Count++;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Main.MessageList.Enqueue(ex.ToString());
            }
        }
        public void Operate()
        {
            for (int I = Players.Count - 1; I >= 0; I--)
                Players[I].Operate();

            for (int I = Monsters.Count - 1; I >= 0; I--)
                Monsters[I].Operate();

        }
           


        public bool IsValidPoint(Point Loc)
        {
            try
            {
                if (!new Rectangle(Point.Empty, MapSize).Contains(Loc)) return false;
                TileInfo T = Tiles[Loc.X, Loc.Y];
                return (Tiles[Loc.X, Loc.Y].Attrib == CellAttribute.Walk);
            }
            catch (Exception EX)
            { }
            return false;

        }

        public bool EmptyPoint(Point Loc)
        {
            if (!new Rectangle(Point.Empty, MapSize).Contains(Loc)) return false;
            return (Tiles[Loc.X, Loc.Y].Attrib == CellAttribute.Walk);
        }

        public List<Player> GetPlayers(Point P)
        {
            List<Player> Plays = new List<Player>();

            Point tempPoint;
            for (int I = Players.Count - 1; I >= 0; I--)
            {
                tempPoint = Players[I].Location;
                if (Math.Abs(tempPoint.X - P.X) <= Config.DataRange && Math.Abs(tempPoint.Y - P.Y) <= Config.DataRange)
                    Plays.Add(Players[I]);
            }
            return Plays;
        }
        public List<object> GetObjects(Player P)
        {
            List<object> ObjList = new List<object>();
            for (int X = P.Location.X - 15; X <= P.Location.X + 15; X++)
            {
                if (X < 0 || X >= MapSize.Width) continue;
                for (int Y = P.Location.Y - 15; Y <= P.Location.Y + 15; Y++)
                {
                    if (Y < 0 || Y >= MapSize.Height) continue;
                    if (Tiles[X, Y].Monsters != null)
                        ObjList.AddRange(Tiles[X, Y].Monsters);

                    if (Tiles[X, Y].Players != null)
                        if (P.Location == new Point(X, Y))
                        {
                            for (int I = Tiles[X, Y].Players.Count - 1; I >= 0; I--)
                            {
                                if (Tiles[X, Y].Players[I] == P) continue;
                                ObjList.Add(Tiles[X, Y].Players[I]);
                            }

                        }
                        else
                            ObjList.AddRange(Tiles[X, Y].Players);
                }
            }
            return ObjList;
        }
        public List<object> GetObjects(Point P)
        {
            if (new Rectangle(Point.Empty, MapSize).Contains(P))
            {
                List<object> List = new List<object>();
                if (Tiles[P.X, P.Y].Players != null) List.AddRange(Tiles[P.X, P.Y].Players);
                if (Tiles[P.X, P.Y].Monsters != null) List.AddRange(Tiles[P.X, P.Y].Monsters);
                return List;
            }
            return null;
        }

        public List<object> GetObjects(Point P, int Range)
        {
            List<object> ObjList = new List<object>();
            for (int X = P.X - Range; X <= P.X + Range; X++)
            {
                if (X < 0 || X >= MapSize.Width) continue;
                for (int Y = P.Y - Range; Y <= P.Y + Range; Y++)
                {
                    if (Y < 0 || Y >= MapSize.Height) continue;
                    if (Tiles[X, Y].Monsters != null)
                        ObjList.AddRange(Tiles[X, Y].Monsters);

                    if (Tiles[X, Y].Players != null)
                        ObjList.AddRange(Tiles[X, Y].Players);
                }
            }
            return ObjList;
        }
        public List<Monster> GetMonsters(Point P, int Range)
        {
            List<Monster> ObjList = new List<Monster>();
            for (int X = P.X - Range; X <= P.X + Range; X++)
            {
                if (X < 0 || X >= MapSize.Width) continue;
                for (int Y = P.Y - Range; Y <= P.Y + Range; Y++)
                {
                    if (Y < 0 || Y >= MapSize.Height) continue;
                    if (Tiles[X, Y].Monsters != null)
                        ObjList.AddRange(Tiles[X, Y].Monsters);
                }
            }
            return ObjList;
        }
        public List<Player> GetPlayers(Point P, int Range)
        {
            List<Player> ObjList = new List<Player>();
            for (int X = P.X - Range; X <= P.X + Range; X++)
            {
                if (X < 0 || X >= MapSize.Width) continue;
                for (int Y = P.Y - Range; Y <= P.Y + Range; Y++)
                {
                    if (Y < 0 || Y >= MapSize.Height) continue;
                    if (Tiles[X, Y].Players != null)
                        ObjList.AddRange(Tiles[X, Y].Players);
                }
            }
            return ObjList;
        }

        public void AddObject(Player P)
        {
            Players.Add(P);
            Tiles[P.Location.X, P.Location.Y].AddObject(P);
        }
        public void AddObject(Monster M)
        {
            Monsters.Add(M);
            Tiles[M.Location.X, M.Location.Y].AddObject(M);
        }

        public bool CheckMapMovement(Player P, Direction D, int A)
        {
            Map M;
            Point TempPoint = Functions.Move(P.Location, D, A);
            for (int I = MInfo.MoveList.Count - 1; I >= 0; I--)
                if (MInfo.MoveList[I].SourcePoint == TempPoint)
                    if (Engine.MapList.TryGetValue(MInfo.MoveList[I].DestMapInfo, out M))
                        if (M.IsValidPoint(MInfo.MoveList[I].DestinationPoint))
                        {
                            RemoveObject(P);
                            P.SendDataOut(new PlayerRemove(P.Index));
                            P.Location = MInfo.MoveList[I].DestinationPoint;
                            P.MapID = M.MInfo.Index;
                            P.Map = M;
                            M.AddObject(P);
                            P.SendData(new ChangeMap(M.MInfo, P.Location));
                            P.SendDataOut(new PlayerNew(P));
                            P.GetAllObjects();
                            return true;
                        }
                    return false;

        }
        public bool Move(Player P, Direction D, int A)
        {
            Point TempPoint = Functions.Move(P.Location, D, A);
            if (!new Rectangle(Point.Empty, MapSize).Contains(TempPoint)) return false;
            
            if (Tiles[TempPoint.X, TempPoint.Y].CanWalk())
            {
                Tiles[P.Location.X, P.Location.Y].RemoveObject(P);
                P.Location = TempPoint;
                P.Dir = D;
                Tiles[P.Location.X, P.Location.Y].AddObject(P);
                return true;
            }
            else return false;

        }
        public bool Move(Monster M, Direction D, int A)
        {
            Point TempPoint = Functions.Move(M.Location, D, A);
            if (!new Rectangle(Point.Empty, MapSize).Contains(TempPoint)) return false;

            if (Tiles[TempPoint.X, TempPoint.Y].CanWalk())
            {
                Tiles[M.Location.X, M.Location.Y].RemoveObject(M);
                M.Location = TempPoint;
                M.Dir = D;
                Tiles[M.Location.X, M.Location.Y].AddObject(M);
                return true;
            }
            else return false;

        }

        public void RemoveObject(Player P)
        {
            Players.Remove(P);
            Tiles[P.Location.X, P.Location.Y].RemoveObject(P);
        }
        public void RemoveObject(Monster M)
        {
            Monsters.Remove(M);
            Tiles[M.Location.X, M.Location.Y].RemoveObject(M);
        }

    }
    
    public class TileInfo
    {
        #region Fields
        public CellAttribute Attrib;
        public List<Player> Players;
        public List<Monster> Monsters;
        #endregion
        #region Constructor
        public TileInfo(CellAttribute _Attrib)
        {
            Attrib = _Attrib;
        }
        #endregion

        public bool CanWalk()
        {
            if (Players != null)
                for (int I = Players.Count - 1; I >= 0; I--)
                    if (!Players[I].Dead) return false;

            if (Monsters != null)
                for (int I = Monsters.Count - 1; I >= 0; I--)
                    if (!Monsters[I].Dead) return false;
            return true;
        }

        public void AddObject(Player P)
        {
            if (Players == null) Players = new List<Player>();
            Players.Add(P);
        }
        public void RemoveObject(Player P)
        {
            if (Players == null) return;
            Players.Remove(P);
            if (Players.Count == 0) Players = null;
        }

        public void AddObject(Monster P)
        {
            if (Monsters == null) Monsters = new List<Monster>();
            Monsters.Add(P);
        }
        public void RemoveObject(Monster P)
        {
            if (Monsters == null) return;
            Monsters.Remove(P);
            if (Monsters.Count == 0) Monsters = null;
        }
    }


    public class Respawn
    {
        #region Fields
        public RespawnInfo RInfo;
        public int RemainingAmount;
        public DateTime SpawnTime;
        public int MinX, MinY, MaxX, MaxY;
        #endregion
        #region Constructor
        public Respawn(RespawnInfo RI, Size MapSize)
        {
            RInfo = RI;
            RemainingAmount = RI.Amount;
            SpawnTime = DateTime.Now;
            #region Point - X
            if (RInfo.Location.X == -1)
            {
                MinX = 0;
                MaxX = MapSize.Width - 1;
            }
            else
            {
                if ((MinX = RInfo.Location.X - RInfo.Spread) < 0) MinX = 0;
                if ((MaxX = RInfo.Location.X + RInfo.Spread) >= MapSize.Width) MaxX = MapSize.Width - 1;
            }
            #endregion
            #region Point - Y
            if (RInfo.Location.Y == -1)
            {
                MinY = 0;
                MaxY = MapSize.Width - 1;
            }
            else
            {
                if ((MinY = RInfo.Location.Y - RInfo.Spread) < 0) MinY = 0;
                if ((MaxY = RInfo.Location.Y + RInfo.Spread) >= MapSize.Height) MaxY = MapSize.Height - 1;
            }
            #endregion
        }
        #endregion
    }

    public class FloorItem
    {
        #region Fields
        public Item I;
        public DateTime DropTime;
        public Player Owner;
        public Point Location;
        #endregion
        #region Constructor
        public FloorItem(Item i, Player owner, Point Loc)
        {
            I = i;
            DropTime = DateTime.Now;
            Owner = owner;
            Location = Loc;
        }
        #endregion
    }
}
