﻿using System;
using System.Collections.Generic;
using System.Drawing;
using Clash_Library;
using S = Server_Packets;
using System.Threading;

namespace Clash_Server
{
    public class ExpOwner
    {
        public Player P;
        public DateTime LastHit;
        public int Damage;
        public ExpOwner(Player p, DateTime t, int d)
        {
            P = p;
            LastHit = t;
            Damage = d;
        }
    }

    public class Monster
    {
        public int MobID;
        public Direction Dir;
        public MonsterInfo Info;
        public Respawn RInfo;
        public Point Location;
        public bool Dead;
        public AllStats TotalStats;
        public int CurrentHP, CurrentMP;
        public Map Map;
        public object Target;
        public List<ExpOwner> ExpList;

        public short ViewRange;
        public TimeSpan SearchDelay, AttackDelay;
        public DateTime ActionTime, SearchTime, DeathTime, RegenTime;

        public Monster(MonsterInfo MI)
        {
            MobID = Engine.MobID;
            Info = MI;
            Dead = false;

            CurrentHP = Info.Stats.Basic.HP;
            CurrentMP = Info.Stats.Basic.MP;
            ViewRange = 8;

            SearchDelay = new TimeSpan(0, 0, 3);

            if (Info.Stats.Advanced.ASpeed > 300)
                AttackDelay = new TimeSpan(0, 0, 0, 0, Info.Stats.Advanced.ASpeed);
            else
                AttackDelay = new TimeSpan(0, 0, 0, 0, 300);

            MakeTotalStats();
        }
        public void Spawn(Respawn R, Map M, Point Loc)
        {
            RInfo = R;
            Map = M;
            Location = Loc;
            Dir = (Direction)M.Rand.Next(8);
            Map.AddObject(this);
            SendDataOut(new S.MonsterNew(this));
        }
        public void Remove()
        {
            Map.RemoveObject(this);
            SendDataOut(new S.MonsterRemove(MobID));
        }
        public void Die()
        {
            if (Dead) return;
            Dead = true;
            if (RInfo != null)
                Interlocked.Increment(ref RInfo.RemainingAmount);
            DeathTime = Engine.Time;
            Target = null;

            Player Winner= null;
            int D = -1;
            if (ExpList != null )
                for (int I = ExpList.Count - 1; I >= 0; I--)
                {
                    if (D < ExpList[I].Damage)
                    {
                        Winner = ExpList[I].P;
                        D = ExpList[I].Damage;
                    }
                }

            if (Winner != null)
            {
                Winner.WinEXP(Info.Experience);


            }
        }
        public void Regen()
        {
            if (CanRegen())
            {
                bool Send = false;
                int I;
                RegenTime = Engine.Time + new TimeSpan(0, 0, 10);
                if (CurrentHP < TotalStats.Basic.HP)
                {
                    if ((I = (int)(TotalStats.Basic.HP / 10D)) <= 0) I = 1;
                    if (I + CurrentHP > TotalStats.Basic.HP) I = I + CurrentHP - TotalStats.Basic.HP;
                    Interlocked.Add(ref CurrentHP, I);
                    Send = true;
                }
                if (CurrentMP < TotalStats.Basic.MP)
                {
                    if ((I = (int)(TotalStats.Basic.MP / 10D)) <= 0) I = 1;
                    if (I + CurrentMP > TotalStats.Basic.MP) I = I + CurrentMP - TotalStats.Basic.MP;
                    Interlocked.Add(ref CurrentMP, I);
                    Send = true;
                }

                if (Send)
                    SendHealth();

            }

        }
        public void SendHealth()
        {
            SendDataOut(new S.MonsterRegen(MobID, CurrentHP, CurrentMP));
        }

        public List<Rectangle> GetMovementRect(Direction D, int Dist, Point OldPoint)
        {
            Rectangle HR = Rectangle.Empty, VR = Rectangle.Empty;
            int R = Config.DataRange;

            #region Bounds
            switch (D)
            {
                case Direction.Up:
                    VR = new Rectangle(OldPoint, new Size(R * 2 + 1, Dist));
                    VR.Offset(-R, -R - Dist);
                    VR = Rectangle.Intersect(VR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
                case Direction.Up_Left:
                    VR = new Rectangle(OldPoint, new Size(R * 2 + 1, Dist));
                    VR.Offset(-R - Dist, -R - Dist);
                    VR = Rectangle.Intersect(VR, new Rectangle(Point.Empty, Map.MapSize));

                    HR = new Rectangle(OldPoint, new Size(Dist, R * 2 + 1 - Dist));
                    HR.Offset(-R - Dist, -R);
                    HR = Rectangle.Intersect(HR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
                case Direction.Up_Right:
                    VR = new Rectangle(OldPoint, new Size(R * 2 + 1, Dist));
                    VR.Offset(-R + Dist, -R - Dist);
                    VR = Rectangle.Intersect(VR, new Rectangle(Point.Empty, Map.MapSize));

                    HR = new Rectangle(OldPoint, new Size(Dist, R * 2 + 1 - Dist));
                    HR.Offset(R, -R);
                    HR = Rectangle.Intersect(HR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
                case Direction.Right:
                    HR = new Rectangle(OldPoint, new Size(Dist, R * 2 + 1));
                    HR.Offset(R, -R);
                    HR = Rectangle.Intersect(HR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
                case Direction.Left:
                    HR = new Rectangle(OldPoint, new Size(Dist, R * 2 + 1));
                    HR.Offset(-R - Dist, -R);
                    HR = Rectangle.Intersect(HR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
                case Direction.Down:
                    VR = new Rectangle(OldPoint, new Size(R * 2 + 1, Dist));
                    VR.Offset(-R, R);
                    VR = Rectangle.Intersect(VR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
                case Direction.Down_Left:
                    VR = new Rectangle(OldPoint, new Size(R * 2 + 1, Dist));
                    VR.Offset(-R - Dist, R);
                    VR = Rectangle.Intersect(VR, new Rectangle(Point.Empty, Map.MapSize));

                    HR = new Rectangle(OldPoint, new Size(Dist, R * 2 + 1 - Dist));
                    HR.Offset(-R - Dist, R + Dist);
                    HR = Rectangle.Intersect(HR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
                case Direction.Down_Right:
                    VR = new Rectangle(OldPoint, new Size(R * 2 + 1, Dist));
                    VR.Offset(-R + Dist, R);
                    VR = Rectangle.Intersect(VR, new Rectangle(Point.Empty, Map.MapSize));

                    HR = new Rectangle(OldPoint, new Size(Dist, R * 2 + 1 - Dist));
                    HR.Offset(R, R + Dist);
                    HR = Rectangle.Intersect(HR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
            }
            #endregion
            List<Rectangle> RList = new List<Rectangle>();
            if (!VR.IsEmpty) RList.Add(VR);
            if (!HR.IsEmpty) RList.Add(HR);
            return RList;
        }
        public List<Point> GetMovementPoints(Direction D, int Dist, Point OldPoint)
        {
            Rectangle OldRect, NewRect;
            OldPoint.Offset(-15, -15);
            OldRect = new Rectangle(OldPoint, new Size(31, 31));

            OldPoint = Functions.Move(OldPoint, D, Dist);
            NewRect = new Rectangle(OldPoint, new Size(31, 31));

            OldRect = Rectangle.Intersect(OldRect, new Rectangle(Point.Empty, Map.MapSize));
            NewRect = Rectangle.Intersect(NewRect, new Rectangle(Point.Empty, Map.MapSize));
            OldRect.Intersect(NewRect);

            List<Point> Ps = GetPoints(NewRect);

            for (int I = Ps.Count - 1; I >= 0; I--)
                if (OldRect.Contains(Ps[I])) Ps.RemoveAt(I);
                else
                { }

            return Ps;
        }
        public List<Point> GetPoints(Rectangle R)
        {
            List<Point> Points = new List<Point>();
            for (int X = R.Left; X < R.Right; X++)
                for (int Y = R.Top; Y < R.Bottom; Y++)
                    Points.Add(new Point(X, Y));
            return Points;
        }
        public List<Point> GetPoints(List<Rectangle> RList)
        {
            List<Point> Points = new List<Point>();
            foreach (Rectangle R in RList)
                for (int X = R.Left; X < R.Right; X++)
                    for (int Y = R.Top; Y < R.Bottom; Y++)
                        Points.Add(new Point(X, Y));
            return Points;
        }

        public int PStruck(AllStats Stats)
        {
            if (Dead) return 0;


            int Dmg = TotalStats.PhysicalStruck(Stats, Map.Rand);

            if (Dmg <= 0) return 0;

            Interlocked.Add(ref CurrentHP, -Dmg);
            Element E = Element.None;
            if (Stats.Elemental.FireA > 0) E |= Element.Fire;
            if (Stats.Elemental.LightningA > 0) E |= Element.Lightning;
            if (Stats.Elemental.IceA > 0) E |= Element.Ice;
            if (Stats.Elemental.WindA > 0) E |= Element.Wind;
            if (Stats.Elemental.HolyA > 0) E |= Element.Holy;
            if (Stats.Elemental.DarkA > 0) E |= Element.Dark;
            if (Stats.Elemental.IllusionA > 0) E |= Element.Illusion;

            SendDataOut(new S.MonsterStruck(MobID, CurrentHP, E, Dir));
            if (CurrentHP <= 0) Die();
            return Dmg;
        }
        public virtual void Attack()
        {
            if (!CanAttack()) return;
            ActionTime = Engine.Time + AttackDelay;
            if (Target as Player != null)
            {
                Dir = Functions.GetDirection(Location, ((Player)Target).Location);
                ((Player)Target).PStruck(TotalStats);
            }
            else
            {
                Dir = Functions.GetDirection(Location, ((Monster)Target).Location);
                ((Monster)Target).PStruck(TotalStats);
            }

            SendDataOut(new S.MonsterAttack(Dir, MobID));
        }
        public void SendNewMonster()
        {
            List<Point> Points = GetMovementPoints(Dir, 1, Functions.Move(Location, Dir, -1));

            S.MonsterNew N = new S.MonsterNew(this);
            for (int I = 0; I < Map.Players.Count; I++)
                if (Points.Contains(Map.Players[I].Location))
                    Map.Players[I].SendData(N);
        }

        public void Move(Point P)
        {
            if (!CanMove()) return;
            Direction D = Functions.GetDirection(Location, P);
            if (Map.Move(this, D, 1))
            {
                SendDataOut(new S.MonsterWalk(MobID, Location));
                ActionTime = Engine.Time + Info.MoveSpeed;
                SendNewMonster();
                return;
            }
            else  if (Map.Rand.Next(2) == 0)
            {
                for (int I = 0; I <= 7; I++)
                {
                    D++;
                    if ((byte)D > 7) D = (Direction)0;
                    if (Map.Move(this, D, 1))
                    {
                        SendDataOut(new S.MonsterWalk(MobID, Location));
                        ActionTime = Engine.Time + Info.MoveSpeed;
                        SendNewMonster();
                        return;
                    }
                }
            }
            else
            {
                for (int I = 7; I >= 0; I--)
                {
                    D--;
                    if ((byte)D < 0) D = (Direction)7;
                    if (Map.Move(this, D, 1))
                    {
                        SendDataOut(new S.MonsterWalk(MobID, Location));
                        ActionTime = Engine.Time + Info.MoveSpeed;
                        SendNewMonster();
                        return;
                    }
                }
            }
        }
        public void Wander()
        {
            if (!CanMove()) return;
            if (Map.Rand.Next(10) == 0)
            Move(Functions.Move(Location, (Direction)Map.Rand.Next(7), 1));
            ActionTime = Engine.Time + Info.MoveSpeed;
        }
        public void FindTarget()
        {
            if (!Info.Behaviour.HasFlag(MonsterBehaviour.Passive) && Engine.Time > SearchTime)
            {
                SearchTime = Engine.Time + SearchDelay;
                GetTarget();
            }
        }
        public bool ProperTarget(Player P)
        {
            if (P.Dead || P.Exiting)
            {
                P = null;
                return false;
            }
            else
            {
                Target = P;
                P = null;
                return true;
            }
        }
        public bool ProperTarget(Monster M)
        {
            return false;
        }
        public void AddToList(Player P, int Dmage)
        {
            if (ExpList == null)
            {
                ExpList = new List<ExpOwner>();
                ExpList.Add(new ExpOwner(P, Engine.Time, Dmage));
            }
            else
            {
                for (int I = ExpList.Count - 1; I >= 0; I--)
                {
                    if (ExpList[I].P == P)
                    {
                        ExpList[I].Damage += Dmage;
                        ExpList[I].LastHit = Engine.Time;
                        return;
                    }
                }
                ExpList.Add(new ExpOwner(P, Engine.Time, Dmage));
            }
        }

        public void GetTarget()
        {
            if (Map.Players.Count == 0) return;
            Point tempPoint = Location;
            int Skip = 0;

            for (int R = 0; R < ViewRange; R++)
                for (int Y = -R; Y < R; Y++)
                {
                    if (tempPoint.Y + Y < 0 || tempPoint.Y + Y >= Map.MapSize.Height) continue;
                    if (Math.Abs(Y) == R) Skip = 0;
                    else Skip = R * 2 - 1;
                    for (int X = -R; X < R; X++)
                    {
                        if (tempPoint.X + X < 0 || tempPoint.X + X >= Map.MapSize.Width) continue;

                        if (Map.Tiles[tempPoint.X + X, tempPoint.Y + Y].Players != null)
                            for (int I = Map.Tiles[tempPoint.X + X, tempPoint.Y + Y].Players.Count - 1; I >= 0; I--)
                                if (ProperTarget(Map.Tiles[tempPoint.X + X, tempPoint.Y + Y].Players[I])) return;

                        if (Map.Tiles[tempPoint.X + X, tempPoint.Y + Y].Monsters != null)
                            for (int I = Map.Tiles[tempPoint.X + X, tempPoint.Y + Y].Monsters.Count - 1; I >= 0; I--)
                                if (ProperTarget(Map.Tiles[tempPoint.X + X, tempPoint.Y + Y].Monsters[I])) return;
                    }
                }
        }
        public void SendDataOut(Packet P)
        {
            List<Player> TempList = Map.GetPlayers(Location);

            foreach (Player Play in TempList)
                Play.SendData(P);

            TempList = null;
        }
        public void MakeTotalStats()
        {
            TotalStats = new AllStats();
            TotalStats += Info.Stats;
        }
        public virtual void Operate()
        {
            Player P = null;
            Monster M = null;

            if (ExpList != null)
            {
                for (int I = ExpList.Count - 1; I >= 0; I--)
                {
                    if (ExpList[I] != null)
                    {
                        if (Engine.Time > ExpList[I].LastHit + new TimeSpan(0, 0, 8))
                            ExpList.RemoveAt(I);
                    }
                }
                if (ExpList.Count == 0) ExpList = null;

            }


            if (Dead)
            {
                if (Engine.Time > DeathTime + new TimeSpan(0, 2, 0))
                    Remove();
                return;
            }
            if (Target != null)
                if ((P = Target as Player) != null)
                {
                    if (P.Dead || P.Exiting)
                    {
                        P = null;
                        Target = null;
                    }
                    else if (!Functions.InRange(Location, P.Location, 20))
                    {
                        P = null;
                        Target = null;
                    }
                }
                else if ((M = Target as Monster) != null)
                    if (M.Dead)
                    {
                        M = null;
                        Target = null;
                    }
                    else if (!Functions.InRange(Location, M.Location, 20))
                    {
                        M = null;
                        Target = null;
                    }
            
            if (Target == null && !Info.Behaviour.HasFlag(MonsterBehaviour.Passive))
            {

                FindTarget();
                if (Target != null)
                    if ((P = Target as Player) == null)
                        M = Target as Monster;
            }
            if (Target != null)
            {
                Point TPoint = P != null ? P.Location : M != null ? M.Location : Point.Empty;

                if (Functions.InRange(Location, TPoint, 1))
                    Attack();
                else
                {
                    Move(TPoint);
                }
            }
            else
                Wander();

            Regen();
        }

        public bool CanRegen()
        {
            return !Dead && Engine.Time > RegenTime;
        }
        public bool CanMove()
        {
            return !Dead && Engine.Time > ActionTime && Info.Behaviour.HasFlag(MonsterBehaviour.CanMove);
        }
        public bool CanAttack()
        {
            return !Dead && Engine.Time > ActionTime;
        }
    }

    public class Oma : Monster
    {
        public DateTime NextHeal;
        public Oma(MonsterInfo MI)
            : base(MI)
        { }

        public void Heal()
        {
            bool Send = false;
            List<Monster> MobList = Map.GetMonsters(Location, 1);

            if (TotalStats.Basic.SC > 0 || TotalStats.Basic.SC2 > 0)
            {
                int HealAmount = 0;
                for (int I = MobList.Count - 1; I >= 0; I--)
                {
                    if (MobList[I].Dead || MobList[I].CurrentHP >= MobList[I].TotalStats.Basic.HP) continue;
                    Send = true;
                    ActionTime = Engine.Time + AttackDelay;
                    NextHeal = Engine.Time + new TimeSpan(0, 0, 8);
                    if (TotalStats.Basic.SC >= TotalStats.Basic.SC2)
                        HealAmount = TotalStats.Basic.SC;
                    else
                        HealAmount = Map.Rand.Next(TotalStats.Basic.SC, TotalStats.Basic.SC2 + 1);

                    Interlocked.Add(ref MobList[I].CurrentHP,
                        MobList[I].CurrentHP + HealAmount > MobList[I].TotalStats.Basic.HP ?
                        MobList[I].CurrentHP + HealAmount - MobList[I].TotalStats.Basic.HP : HealAmount);
                    MobList[I].SendHealth();
                }
                if (Send)
                    SendDataOut(new S.MonsterSpell(MobID, 0, Dir));
            }
        }

        public override void Operate()
        {
            if (CanAttack() && Engine.Time > NextHeal)
                Heal();

            base.Operate();
        }
    }

    public class OmaFighter : Monster
    {
        public DateTime NextFireWall;

        public void FireWall()
        {

        }

        public OmaFighter(MonsterInfo MI)
            : base(MI)
        { }
        public override void Operate()
        {
            if (CanAttack() && Engine.Time > NextFireWall)
                FireWall();

            base.Operate();
        }


    }

}
