﻿using System;
using System.Collections.Generic;
using System.Drawing;
using Clash_Library;
using S = Server_Packets;
using C = Client_Packets;
using System.Runtime.Serialization;

namespace Clash_Server
{
    [Serializable]
     public class Player
    {
        public int Index;
        public int Level;
        public Class Class;
        public Gender Gender;
        public Point Location;
        public Item[] BagItems;
        public Item[] Equipment;
        public Direction Dir;
        public int MapID;
        public byte Hair;
        public bool Dead, Dev;
        public AllStats BaseStats;

        public int CurrentHP, CurrentMP;
        public long Exp;

        [NonSerialized]
        public long MaxEXP;

        [NonSerialized]
        public AllStats TotalStats;
        [NonSerialized]
        public Map Map;

        #region Misc Fields
        [NonSerialized]
        public AccountIndex AI;
        public int DeleteCount, BanCount;
        [NonSerialized]
        Queue<Packet> OPList;
        [NonSerialized]
        public bool Exiting;
        [NonSerialized]
        List<int> IIndexList;
        [NonSerialized]
        DateTime MoveTime, AttackTime, RegenTime;
        #endregion
        
        [NonSerialized]
        public CharacterIndex CI;

        public Player(CharacterIndex _CI, Class job, Gender sex)
        {
            CI = _CI;
            Index = CI.Index;
            Class = job;
            Gender = sex;
            Level = 50;
            Location = Config.StartPoint;
            MapID = Config.StartMapID;
            BagItems = new Item[46];
            if (CI.CharacterName == "Jamie") Dev = true;
            else Dev = false;
            Equipment = new Item[Enum.GetNames(typeof(ItemLocation)).Length];
            Hair = 1;
            BanCount = 0;
            Dead = false;
            Dir = Direction.Up;
            BaseStats = new AllStats();
            BaseStats.Advanced.Light = 3;
            AddToBag(new Item(Engine.SData.ItemList[317], Config.ItemID));

            switch (Class)
            {
                case Clash_Library.Class.Warrior:
                    BaseStats.Basic.HP = 300;
                    BaseStats.Basic.MP = 100;
                    BaseStats.Basic.Accuracy = 50;
                    BaseStats.Basic.Agility = 3;
                    BaseStats.Basic.DC = 2;
                    BaseStats.Basic.DC2 = 5;
                    break;
                case Clash_Library.Class.Wizard:
                    BaseStats.Basic.HP = 100;
                    BaseStats.Basic.MP = 300;
                    BaseStats.Basic.Accuracy = 5;
                    BaseStats.Basic.Agility = 5;
                    BaseStats.Basic.MC2 = 5;

                    break;
                case Clash_Library.Class.Taoist:
                    BaseStats.Basic.HP = 200;
                    BaseStats.Basic.MP = 200;
                    BaseStats.Basic.Accuracy = 6;
                    BaseStats.Basic.Agility = 5;
                    BaseStats.Basic.DC2 = 2;
                    BaseStats.Basic.SC2 = 5;

                    break;
                case Clash_Library.Class.Assassin:
                    BaseStats.Basic.HP = 230;
                    BaseStats.Basic.MP = 170;
                    BaseStats.Basic.Accuracy = 7;
                    BaseStats.Basic.Agility = 7;
                    BaseStats.Basic.DC = 3;
                    BaseStats.Basic.DC2 = 3;

                    break;
            }
            CurrentHP = BaseStats.Basic.HP;
            CurrentMP = BaseStats.Basic.MP;

            Accounts.AccountByIndex(CI.AccountIndex, out AI);
            MakeTotalStats();
        }
        public void LogIn(Queue<Packet> PL, List<int> IList)
        {
            
            if (Dead)
            {
                Dead = false;
                CurrentHP = TotalStats.Basic.HP;
                CurrentMP = TotalStats.Basic.MP;
                MapID = Config.StartMapID;
                Location = Config.StartPoint;
            } 
            MakeTotalStats();
            OPList = PL;
            Exiting = false;
            IIndexList = IList;
            if (Level >= Config.MaxExpList.Count)
                MaxEXP = Config.MaxExpList[Config.MaxExpList.Count - 1];
            else
                MaxEXP = Config.MaxExpList[Level];
            #region Set Location
            MapInfo MI = null;


            if (MapID >= 0 && MapID < Engine.SData.MapList.Count && Engine.SData.MapList[MapID].Index == MapID)
                MI = Engine.SData.MapList[MapID];
            else for (int I = 0; I < Engine.MapList.Count; I++)
                    if (Engine.SData.MapList[I].Index == MapID)
                    {
                        MI = Engine.SData.MapList[I];
                        break;
                    }

            if (MI == null)
                for (int I = 0; I < Engine.MapList.Count; I++)
                    if (Engine.SData.MapList[I].Index == Config.StartMapID)
                    {
                        MapID = Config.StartMapID;
                        MI = Engine.SData.MapList[I];
                        Location = Config.StartPoint;
                        break;
                    }

            if (Engine.MapList.TryGetValue(MI, out Map) && Map.IsValidPoint(Location))
                Map.AddObject(this);
            else
            {
                for (int I = 0; I < Engine.MapList.Count; I++)
                    if (Engine.SData.MapList[I].Index == Config.StartMapID)
                    {
                        MapID = Config.StartMapID;
                        MI = Engine.SData.MapList[I];
                        Location = Config.StartPoint;
                        break;
                    }
                if (Engine.MapList.TryGetValue(MI, out Map) && Map.IsValidPoint(Location))
                    Map.AddObject(this);
                else
                {
                    // Disconnect
                }
            }
            #endregion
            SendLoginItems();
            SendData(new S.UserInfo(this));
            SendDataOut(new S.PlayerNew(this));
            GetAllObjects();
        }
        public void LogOut()
        { 
            Map.RemoveObject(this);
            Exiting = true;
            SendDataOut(new S.PlayerRemove(Index));
        }
        public void SendData(Packet P)
        {
            OPList.Enqueue(P);
        }
        public void SendDataOut(Packet P)
        {
            List<Player> TempList = Map.GetPlayers(Location);

            foreach (Player Play in TempList)
                if (Play != this)
                    Play.SendData(P);

            TempList = null;
        }
        public void Read(Packet P)
        {
            try
            {
                GetType().GetMethod(P.GetType().Name).Invoke(this, new Object[] { P });
            }
            catch (Exception ex)
            {
                Main.MessageList.Enqueue(string.Format("Character:{0}, Packet {1} \n{2}", CI.CharacterName, P.GetType().Name, ex));
            }
        }

        public void GetAllObjects()
        {
            try
            {
                List<object> ObjList = Map.GetObjects(this);
                if (ObjList.Count > 0)
                    SendData(new S.NewObjects(ObjList));
            }
            catch (Exception ex)
            { Main.MessageList.Enqueue(CI.CharacterName + "/n" + ex.ToString()); }
        }
        public void GetNewObjects(Direction D, int Dist, Point OldPoint)
        {
            List<object> ObjList = new List<object>();
            List<Point> Points = GetMovementPoints(D, Dist, OldPoint);

            foreach (Point P in Points)
            {
                if (Map.Tiles[P.X, P.Y].Players != null)
                    ObjList.AddRange(Map.Tiles[P.X, P.Y].Players);
                if (Map.Tiles[P.X, P.Y].Monsters != null)
                    ObjList.AddRange(Map.Tiles[P.X, P.Y].Monsters);
            }

            if (ObjList.Count > 0)
                SendData(new S.NewObjects(ObjList));

            S.PlayerNew N = new S.PlayerNew(this);
            for (int I = 0; I < Map.Players.Count; I++)
                if (Points.Contains(Map.Players[I].Location))
                    Map.Players[I].SendData(N);
        }
        public List<Rectangle> GetMovementRect(Direction D, int Dist, Point OldPoint)
        {
            Rectangle HR = Rectangle.Empty, VR = Rectangle.Empty;
            int R = Config.DataRange;

            #region Bounds
            switch (D)
            {
                case Direction.Up:
                    VR = new Rectangle(OldPoint, new Size(R * 2 + 1, Dist));
                    VR.Offset(-R, -R - Dist);
                    VR = Rectangle.Intersect(VR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
                case Direction.Up_Left:
                    VR = new Rectangle(OldPoint, new Size(R * 2 + 1, Dist));
                    VR.Offset(-R - Dist, -R - Dist);
                    VR = Rectangle.Intersect(VR, new Rectangle(Point.Empty, Map.MapSize));

                    HR = new Rectangle(OldPoint, new Size(Dist, R * 2 + 1 - Dist));
                    HR.Offset(-R - Dist, -R);
                    HR = Rectangle.Intersect(HR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
                case Direction.Up_Right:
                    VR = new Rectangle(OldPoint, new Size(R * 2 + 1, Dist));
                    VR.Offset(-R + Dist, -R - Dist);
                    VR = Rectangle.Intersect(VR, new Rectangle(Point.Empty, Map.MapSize));

                    HR = new Rectangle(OldPoint, new Size(Dist, R * 2 + 1 - Dist));
                    HR.Offset(R, -R);
                    HR = Rectangle.Intersect(HR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
                case Direction.Right:
                    HR = new Rectangle(OldPoint, new Size(Dist, R * 2 + 1));
                    HR.Offset(R, -R);
                    HR = Rectangle.Intersect(HR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
                case Direction.Left:
                    HR = new Rectangle(OldPoint, new Size(Dist, R * 2 + 1));
                    HR.Offset(-R - Dist, -R);
                    HR = Rectangle.Intersect(HR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
                case Direction.Down:
                    VR = new Rectangle(OldPoint, new Size(R * 2 + 1, Dist));
                    VR.Offset(-R, R);
                    VR = Rectangle.Intersect(VR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
                case Direction.Down_Left:
                    VR = new Rectangle(OldPoint, new Size(R * 2 + 1, Dist));
                    VR.Offset(-R - Dist, R);
                    VR = Rectangle.Intersect(VR, new Rectangle(Point.Empty, Map.MapSize));

                    HR = new Rectangle(OldPoint, new Size(Dist, R * 2 + 1 - Dist));
                    HR.Offset(-R - Dist, R + Dist);
                    HR = Rectangle.Intersect(HR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
                case Direction.Down_Right:
                    VR = new Rectangle(OldPoint, new Size(R * 2 + 1, Dist));
                    VR.Offset(-R + Dist, R);
                    VR = Rectangle.Intersect(VR, new Rectangle(Point.Empty, Map.MapSize));

                    HR = new Rectangle(OldPoint, new Size(Dist, R * 2 + 1 - Dist));
                    HR.Offset(R, R + Dist);
                    HR = Rectangle.Intersect(HR, new Rectangle(Point.Empty, Map.MapSize));
                    break;
            }
            #endregion
            List<Rectangle> RList = new List<Rectangle>();
            if (!VR.IsEmpty) RList.Add(VR);
            if (!HR.IsEmpty) RList.Add(HR);
            return RList;
        }
        public List<Point> GetMovementPoints(Direction D, int Dist, Point OldPoint)
        {
            Rectangle OldRect, NewRect;
            OldPoint.Offset(-15, -15);
            OldRect = new Rectangle(OldPoint, new Size(31, 31));

            OldPoint = Functions.Move(OldPoint, D, Dist);
            NewRect = new Rectangle(OldPoint, new Size(31, 31));

            OldRect = Rectangle.Intersect(OldRect, new Rectangle(Point.Empty, Map.MapSize));
            NewRect = Rectangle.Intersect(NewRect, new Rectangle(Point.Empty, Map.MapSize));
            OldRect.Intersect(NewRect);

            List<Point> Ps = GetPoints(NewRect);

            for (int I = Ps.Count - 1; I >= 0; I--)
                if (OldRect.Contains(Ps[I])) Ps.RemoveAt(I);
                else
                { }

            return Ps;
        }
        public List<Point> GetPoints(Rectangle R)
        {
            List<Point> Points = new List<Point>();
            for (int X = R.Left; X < R.Right; X++)
                for (int Y = R.Top; Y < R.Bottom; Y++)
                    Points.Add(new Point(X, Y));
            return Points;
        }
        public List<Point> GetPoints(List<Rectangle> RList)
        {
            List<Point> Points = new List<Point>();
            foreach (Rectangle R in RList)
                for (int X = R.Left; X < R.Right; X++)
                    for (int Y = R.Top; Y < R.Bottom; Y++)
                        Points.Add(new Point(X, Y));
            return Points;
        }



        public void PStruck(AllStats Stats)
        {
            if (Dead) return;
            
            int Dmg = TotalStats.PhysicalStruck(Stats, Map.Rand);
            if (Dmg <= 0) return;

            System.Threading.Interlocked.Add(ref CurrentHP, -Dmg);
            Element E = Element.None;

            if (Stats.Elemental.FireA > 0) E |= Element.Fire;
            if (Stats.Elemental.LightningA > 0) E |= Element.Lightning;
            if (Stats.Elemental.IceA > 0) E |= Element.Ice;
            if (Stats.Elemental.WindA > 0) E |= Element.Wind;
            if (Stats.Elemental.HolyA > 0) E |= Element.Holy;
            if (Stats.Elemental.DarkA > 0) E |= Element.Dark;
            if (Stats.Elemental.IllusionA > 0) E |= Element.Illusion;
            SendData(new S.Struck(CurrentHP, E));
            SendDataOut(new S.PlayerStruck(Index, CurrentHP, E, Dir));

            if (CurrentHP <= 0)
                Dead = true;
        }
        public void MStruck(AllStats Stats)
        {
            if (Dead) return;

            int Dmg = TotalStats.PhysicalStruck(Stats, Map.Rand);
            if (Dmg <= 0) return;

            System.Threading.Interlocked.Add(ref CurrentHP, -Dmg);
            Element E = Element.None;

            if (Stats.Elemental.FireA > 0) E |= Element.Fire;
            if (Stats.Elemental.LightningA > 0) E |= Element.Lightning;
            if (Stats.Elemental.IceA > 0) E |= Element.Ice;
            if (Stats.Elemental.WindA > 0) E |= Element.Wind;
            if (Stats.Elemental.HolyA > 0) E |= Element.Holy;
            if (Stats.Elemental.DarkA > 0) E |= Element.Dark;
            if (Stats.Elemental.IllusionA > 0) E |= Element.Illusion;
            SendData(new S.Struck(CurrentHP, E));
            SendDataOut(new S.PlayerStruck(Index, CurrentHP, E, Dir));

            if (CurrentHP <= 0)
                Dead = true;
        }
        public void MStruck(int Dmg)
        {

        }


        public void Operate()
        {
            Regen();
        }
        public void WinEXP(long Amount)
        {
            Exp += Amount;
            SendData(new S.WinEXP(Amount));
            if (Exp >= MaxEXP)
            {
                Exp -= MaxEXP;
                Level++;
                if (Level >= Config.MaxExpList.Count)
                    MaxEXP = Config.MaxExpList[Config.MaxExpList.Count - 1];
                else
                    MaxEXP = Config.MaxExpList[Level];

                SendData(new S.LevelUp(MaxEXP));
            }

        }

        public void Regen()
        {
            if (CanRegen())
            {
                bool Send = false;
                int I;
                RegenTime = Engine.Time + new TimeSpan(0, 0, 1);
                if (CurrentHP < TotalStats.Basic.HP)
                {
                   if ((I = (int)(TotalStats.Basic.HP / (12D/10D * 100D))) <= 0) I = 1;
                    if (I + CurrentHP > TotalStats.Basic.HP) I -= I + CurrentHP - TotalStats.Basic.HP;
                    System.Threading.Interlocked.Add(ref CurrentHP, I);
                    Send = true;
                }
                if (CurrentMP < TotalStats.Basic.MP)
                {
                    if ((I = (int)(TotalStats.Basic.MP / (12D / 10D * 100D))) <= 0) I = 1;
                    if (I + CurrentMP > TotalStats.Basic.MP) I -= I + CurrentMP - TotalStats.Basic.MP;
                    System.Threading.Interlocked.Add(ref CurrentMP, I);
                    Send = true;
                }

                if (Send)
                {
                    SendDataOut(new S.PlayerRegen(Index, CurrentHP, CurrentMP));
                    SendData(new S.Regen(CurrentHP, CurrentMP));
                }
            }
        }


        #region Actions
        public void Attack(C.Attack Packet)
        {
            Monster M;
            Player P;

            
            if (CanAttack())
            {
                Dir = Packet.Dir;
                List<object> ObjList = Map.GetObjects(Functions.Move(Location, Dir, 1));
                if (ObjList != null)
                    for (int I = ObjList.Count - 1; I >= 0; I--)
                    {
                        if ( (P = ObjList[I] as Player) != null)
                        {
                            if (P.Dead) continue;
                            P.PStruck(TotalStats);
                        }
                        else if ((M = ObjList[I] as Monster) != null)
                        {
                            if (M.Dead) continue;
                            M.AddToList(this, 0);
                            int D = M.PStruck(TotalStats);
                            M.AddToList(this, D);

                            if (M.Target == null)
                                M.Target = this;
                            else if (Map.Rand.Next(5) == 0)
                                M.Target = this;
                        }
                    }
                SendDataOut(new S.PlayerAttack(Index, Dir));
            }
        }
        public void Turn(C.Turn Packet)
        {
            if (CanMove())
            {
                if (Map.CheckMapMovement(this, Dir, 0)) return;
                Dir = Packet.Dir;
                SendDataOut(new S.PlayerTurn(Index, Dir));
            }
            else
                SendData(new S.MoveFail(Location, Dir));

        }
        public void Walk(C.Walk Packet)
        {
            if (CanMove())
            {
                if (Map.CheckMapMovement(this, Packet.Dir, 1)) return;
                if (Map.Move(this, Packet.Dir, 1))
                {
                    GetNewObjects(Dir, 1, Functions.Move(Location, Dir, -1));
                    SendDataOut(new S.PlayerWalk(Index, Location));
                }
                else
                    SendData(new S.MoveFail(Location, Dir));
            }
            else
                SendData(new S.MoveFail(Location, Dir));

        }
        public void Run(C.Run Packet)
        {
            if (CanMove())
            {
                if (Map.CheckMapMovement(this, Packet.Dir, 2)) return;
                if (Map.Move(this, Packet.Dir, 2))
                {
                    GetNewObjects(Dir, 2, Functions.Move(Location, Dir, -2));
                    SendDataOut(new S.PlayerRun(Index, Location));
                }
                else
                    SendData(new S.MoveFail(Location, Dir));
            }
            else
                SendData(new S.MoveFail(Location, Dir));
        }
        public void Revive(C.Revive Packet)
        {
            if (Dead)
            {
                Dead = false;
                CurrentHP = TotalStats.Basic.HP;
                SendDataOut(new S.PlayerRemove(Index));
                Map.RemoveObject(this);
                Location = Config.StartPoint;
                MapID = Config.StartMapID;
                Map = Engine.MapList[Engine.SData.MapList[Config.StartMapID]];
                SendData(new S.ChangeMap(Map.MInfo, Location));

                Map.AddObject(this);
                SendData(new S.Revive(Location));
                SendDataOut(new S.PlayerNew(this));
                GetAllObjects();
            }
        }
        public void Chat(C.Chat Packet)
        {
            int I = -1;
            if (int.TryParse(Packet.Text,out I))
            {
                if (I < 0) return;
                if (I < Engine.SData.MapList.Count)
                {
                   Map T = Engine.MapList[Engine.SData.MapList[I]];
                   Point P = Location;
                    for (int X = 0; X < 20; X++)
                    {
                        P = new Point(T.Rand.Next(T.MapSize.Width), T.Rand.Next(T.MapSize.Height));
                        if (T.IsValidPoint(P))
                        {
                            break;
                        }
                        if (X >= 18) return;
                    }

                    Map.RemoveObject(this);
                    SendDataOut(new S.PlayerRemove(Index));
                    Location = P;
                    MapID = I;
                    Map = T;
                    Map.AddObject(this);
                    SendData(new S.ChangeMap(Map.MInfo, Location));
                    SendDataOut(new S.PlayerNew(this));
                    GetAllObjects();
                }

            }
            return;
        }
        #endregion

        #region Items
        public void SendLoginItems()
        {
            List<ItemInfo> IIList = new List<ItemInfo>();
            for (int I = BagItems.Length - 1; I >= 0; I--)
            {
                if (BagItems[I] == null) continue;
                if (IIndexList.Contains(BagItems[I].Index)) continue;
                IIList.Add(BagItems[I].IInfo);
                IIndexList.Add(BagItems[I].Index);
            }
            for (int I = Equipment.Length - 1; I >= 0; I--)
            {
                if (Equipment[I] == null) continue;
                if (IIndexList.Contains(Equipment[I].Index)) continue;
                IIList.Add(Equipment[I].IInfo);
                IIndexList.Add(Equipment[I].Index);
            }
            SendData(new S.ItemList(IIList));
        }
        private void AddToBag(Item I)
        {
            for (int A = 6; A < BagItems.Length; A++)
                if (BagItems[A] == null)
                {
                    BagItems[A] = I;
                    return;
                }
            for (int A = 0; A < 6; A++)
                if (BagItems[A] == null)
                {
                    BagItems[A] = I;
                    return;
                }

        }
        public void ChangeBagSlot(C.ChangeBagSlot Packet)
        {
            Item TempI;
            TempI = BagItems[Packet.OldIndex];
            BagItems[Packet.OldIndex] = BagItems[Packet.NewIndex];
            BagItems[Packet.NewIndex] = TempI;
            TempI = null;
        }
        public void EquipItem(C.EquipItem Packet)
        {
            Item TItem;
            int A;
            for (int I = 0; I < BagItems.Length; I++)
            {
                TItem = BagItems[I];
                if (TItem != null && TItem.UniqueID == Packet.ItemID)
                {
                    #region Requirement
                    if ((A = TItem.IInfo.ReqAmount) > 0)
                        switch (TItem.IInfo.ReqType)
                        {
                            case Requirement.Level:
                                if (A > Level)
                                    return; //Level too Low                                
                                break;
                            case Requirement.DC:
                                if (A > TotalStats.Basic.DC2)
                                    return;//Level DC Low                                
                                break;
                            case Requirement.MC:
                                if (A > TotalStats.Basic.MC2)
                                    return; //Level MC Low
                                break;
                            case Requirement.SC:
                                if (A > TotalStats.Basic.SC2)
                                    return; // Level SC Low
                                break;
                        }

                    if (TItem.IInfo.RequiredClass != 0)
                    {
                        ReqClass C = (ReqClass)Enum.Parse(typeof(ReqClass), Class.ToString());
                        if ((TItem.IInfo.RequiredClass & C) == 0)
                            return; //Wrong Class
                    }

                    //Gender
                    if (TItem.IInfo.RequiredGender != 0)
                    {
                        ReqGender G = (ReqGender)Enum.Parse(typeof(ReqGender), Gender.ToString());
                        if ((TItem.IInfo.RequiredGender & G) != 0)
                            return; //Wrong Gender
                    }
                    #endregion

                    if (Equipment[(byte)Packet.Loc] != null)
                    {
                        if (Equipment[(byte)Packet.Loc].IInfo.Behaviour.HasFlag(ItemBehaviour.Stuck)) return; //Cannot Remove
                        BagItems[I] = Equipment[(byte)Packet.Loc];
                        Equipment[(byte)Packet.Loc] = TItem;
                    }
                    else
                    {
                        BagItems[I] = null;
                        Equipment[(byte)Packet.Loc] = TItem;
                    }

                    SendData(new S.EquipItem(Packet.Loc, Packet.ItemID));

                    MakeTotalStats();
                    ChangeState();
                    return;
                }
            }
        }
        public bool CheckBagRoom(int Spaces)
        {
            for (int I = 0; I < BagItems.Length; I++)
            {
                if (--Spaces <= 0) return true;
            }
            return false;
        }
        public void UnEquipItem(C.UnEquipItem Packet)
        {
            if (!CheckBagRoom(1)) return;

            for (int I = 0; I < Equipment.Length; I++)
            {
                if (Equipment[I] != null && Equipment[I].UniqueID == Packet.ItemID)
                {
                    if (Equipment[I].IInfo.Behaviour.HasFlag(ItemBehaviour.Stuck)) return;
                    else SendData(new S.UnEquipItem(Equipment[I].UniqueID));

                    AddToBag(Equipment[I]);
                    Equipment[I] = null;

                    MakeTotalStats();
                    ChangeState();
                    return;
                }
            }
        }
        public void ChangeState()
        {
            S.PlayerChanged P = new S.PlayerChanged();
            P.Sex = Gender;
            P.MaxHP = TotalStats.Basic.HP;
            P.MaxMP = TotalStats.Basic.MP;
            P.Hair = Hair;
            if (Equipment[(byte)ItemLocation.Armour] != null)
                P.Armor = Equipment[(byte)ItemLocation.Armour].IInfo.Shape;
            if (Equipment[(byte)ItemLocation.Weapon] != null)
                P.Weapon = Equipment[(byte)ItemLocation.Weapon].IInfo.Shape;
            P.PlayerID = Index;
            SendDataOut(P);
        }
        #endregion

        public bool CanRegen()
        {
            return !Dead && Engine.Time > RegenTime;
        }
        public bool CanMove()
        {
            return !Dead && DateTime.Now > MoveTime + new TimeSpan(0, 0, 0, 0, 500);
        }
        public bool CanAttack()
        {
            return !Dead && DateTime.Now > AttackTime + new TimeSpan(0, 0, 0, 0, 800);
        }


        [OnDeserialized]
        public void LoadCI(StreamingContext context)
        {
            Characters.CharacterByIndex(Index, out CI);
            Accounts.AccountByIndex(CI.AccountIndex, out AI);
            UpdateItems();
            MakeTotalStats();
        }
        public void MakeTotalStats()
        {
            TotalStats = new AllStats();
            TotalStats += BaseStats;
            foreach (Item E in Equipment)
                if (E != null)
                    TotalStats += E.TotalStats;
        }
        private void UpdateItems()
        {
            foreach (Item I in BagItems)
            {
                if (I == null) continue;
                if (I.Index == Engine.SData.ItemList[I.Index].Index)
                {
                    I.IInfo = Engine.SData.ItemList[I.Index];
                    I.UpdateStats();
                }
                else
                    for (int X = 0; X < Engine.SData.ItemList.Count; X++)
                        if (I.Index == Engine.SData.ItemList[X].Index)
                        {
                            I.IInfo = Engine.SData.ItemList[X];
                            I.UpdateStats();
                            break;
                        }
            }
            foreach (Item I in Equipment)
            {
                if (I == null) continue;
                if (I.Index == Engine.SData.ItemList[I.Index].Index)
                {
                    I.IInfo = Engine.SData.ItemList[I.Index];
                    I.UpdateStats();
                }
                else
                    for (int X = 0; X < Engine.SData.ItemList.Count; X++)
                        if (I.Index == Engine.SData.ItemList[X].Index)
                        {
                            I.IInfo = Engine.SData.ItemList[X];
                            I.UpdateStats();
                            break;
                        }
            }
        }
    }
}
