﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using Clash_Library;
using System.Threading;


namespace Clash_Server
{
    public enum EngineState { Stopped, Loading, Running, Stopping, Crashed, Failed }
    
    static class Engine
    {
        private static EngineState state = EngineState.Stopped;

        public static EngineState State
        {
            get { return Engine.state; }
            set
            {
                Engine.state = value;
            }
        }
        public static ServerData SData;
        public static Dictionary<MapInfo, Map> MapList;
        private static int _ObjectID = 0;
        public static DateTime Time;

        #region Properties
        public static int MobID
        {
            get { return Interlocked.Increment(ref Engine._ObjectID); }
        }
        #endregion

        public static void Load(object o)
        {
            try
            {
                if (State == EngineState.Stopped)
                {
                    Main.MessageList.Enqueue("Server Starting.");
                    State = EngineState.Loading;
                    BinaryFormatter BF = new BinaryFormatter();
                    if (File.Exists(Path.Combine(Config.DataFolder, @"Data.CL")))
                        using (FileStream FStream = new FileStream(Path.Combine(Config.DataFolder, @"Data.CL"), FileMode.Open))
                            SData = (ServerData)BF.Deserialize(FStream);

                    if (SData == null)
                        throw new Exception("Data File Not Found.");

                    LoadMaps();
                    if (!Network.Load())
                        return;

                    State = EngineState.Running;
                    Main.MessageList.Enqueue("Server Loaded Successfully.");
                    EngineLoop();
                }
            }
            catch (Exception ex)
            {
                State = EngineState.Crashed;
                Main.MessageList.Enqueue(ex.ToString());
            }
        }
        private static void LoadMaps()
        {
            Main.MessageList.Enqueue(string.Format("Loading Maps {0}.", SData.MapList.Count));
            MapList = new Dictionary<MapInfo, Map>();
            foreach (MapInfo MInfo in SData.MapList)
                new Map(MInfo);
            Main.MessageList.Enqueue("Maps Loaded.");
        }

        private static void EngineLoop()
        {
            DateTime LastSpawn = DateTime.MinValue;
            while (State == EngineState.Running)
            {
                try
                {
                    if (Engine.Time > LastSpawn + new TimeSpan(0, 1, 0))
                    {
                        LastSpawn = Engine.Time;
                        foreach (Map M in MapList.Values)
                            M.Respawn();
                    }

                    foreach (Map M in MapList.Values)
                        M.Operate();

                    Time = DateTime.Now;
                    Thread.Sleep(1);
                }
                catch (Exception ex)
                { Main.MessageList.Enqueue(ex.ToString()); }
            }
        }
    }
}
