﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Clash_Library;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;

namespace ClashManager
{
    public partial class MapForm : Form
    {
        MapInfo CurrentMap = new MapInfo();
        MovementInfo CurrentMove = new MovementInfo();
        RespawnInfo CurrentSpawn = new RespawnInfo();
        RespawnInfo CopySpawn = null;


        public MapForm()
        {
            InitializeComponent();
        }

        private void RemoveMap_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to remove this Map?", "Remove Map", MessageBoxButtons.YesNo) == System.Windows.Forms.DialogResult.Yes)
            {
                MapInfo DelMap = CurrentMap;
                Main.SData.MapList.Remove(DelMap);
                MapListBox.Items.Remove(DelMap);
                for (int I = 0; I < Main.SData.MapList.Count; I++)
                    Main.SData.MapList[I].Index = I;
                MapListBox.Items.Clear();
                MapListBox.Items.AddRange(Main.SData.MapList.ToArray());
                DestMapID.Items.Clear();
                DestMapID.Items.AddRange(Main.SData.MapList.ToArray());
                UpdateControls();

                foreach (MapInfo M in Main.SData.MapList)
                    for (int I = M.MoveList.Count - 1; I >= 0; I--)
                    {
                        if (M.MoveList[I].DestMapInfo == DelMap)
                            M.MoveList.RemoveAt(I);
                    }
            }
        }

        private void CopyMap_Click(object sender, EventArgs e)
        {
            using (MemoryStream MStream = new MemoryStream())
            {
                BinaryFormatter BF = new BinaryFormatter();
                BF.Serialize(MStream, CurrentMap);
                MStream.Position = 0;
                CurrentMap = (MapInfo)BF.Deserialize(MStream);
                CurrentMap.Index = MapListBox.Items.Count;
                Main.SData.MapList.Add(CurrentMap);
                MapListBox.Items.Add(CurrentMap);
                DestMapID.Items.Add(CurrentMap);
                MapListBox.SelectedIndex = MapListBox.Items.Count - 1;
                UpdateControls();
            }
        }

        private void AddMap_Click(object sender, EventArgs e)
        {
            CurrentMap = new MapInfo();
            CurrentMap.Index = MapListBox.Items.Count;
            Main.SData.MapList.Add(CurrentMap);
            MapListBox.Items.Add(CurrentMap);
            MapListBox.SelectedIndex = MapListBox.Items.Count - 1;
            DestMapID.Items.Add(CurrentMap);
            UpdateControls();
        }

        private void Index_TextChanged(object sender, EventArgs e)
        {
            if (int.TryParse(Index.Text, out CurrentMap.Index))
                Index.BackColor = Color.Green;
            else
                Index.BackColor = Color.Red;
            UpdateName();
        }

        private void FileName_TextChanged(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(FileName.Text))
                FileName.BackColor = Color.Red;
            else
                FileName.BackColor = Color.Green;
            CurrentMap.FileName = FileName.Text;
        }

        private void MapName_TextChanged(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(MapName.Text))
                MapName.BackColor = Color.Red;
            else
                MapName.BackColor = Color.Green;
            CurrentMap.MapDescription = MapName.Text;
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            CurrentMap.MapType = MapType.Mir2;
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            CurrentMap.MapType = MapType.Mir3;
        }



        private void MapForm_Load(object sender, EventArgs e)
        {
            MapListBox.Items.AddRange(Main.SData.MapList.ToArray());
            MapFlags.Items.AddRange(Enum.GetNames(typeof(Clash_Library.MapFlags)));
            DestMapID.Items.AddRange(Main.SData.MapList.ToArray());
            MonsterList.Items.AddRange(Main.SData.MonsterList.ToArray());            
        }

        private void SaveMaps_Click(object sender, EventArgs e)
        {
            if (!Directory.Exists(@".\Data\")) Directory.CreateDirectory(@".\Data\");
            BinaryFormatter BF = new BinaryFormatter();
            using (FileStream FStream = new FileStream(@".\Data\Data.CL", FileMode.Create))
                BF.Serialize(FStream, Main.SData);
        }

        private void MapFlags_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (e.NewValue == CheckState.Checked)
                CurrentMap.Flags += (int)Enum.GetValues(typeof(Clash_Library.MapFlags)).GetValue(e.Index);
            else
                CurrentMap.Flags -= (int)Enum.GetValues(typeof(Clash_Library.MapFlags)).GetValue(e.Index);
        }



        private void UpdateName()
        {
            int I = MapListBox.SelectedIndex;
            if (I == -1) return;
            MapListBox.SelectedIndexChanged -= MapListBox_SelectedIndexChanged;
            MapListBox.Items[I] = CurrentMap;
            MapListBox.SelectedItem = I;
            MapListBox.SelectedIndexChanged += new EventHandler(MapListBox_SelectedIndexChanged);            
        }

        private void MapListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            CurrentMap = (MapInfo)MapListBox.SelectedItem;
            if (CurrentMap == null) CurrentMap = new MapInfo();
            UpdateControls();
            MovementList.SelectedIndex = MovementList.Items.Count - 1;
            SpawnList.SelectedIndex = SpawnList.Items.Count - 1;
        }
        private void UpdateControls()
        {
            Index.Text = CurrentMap.Index.ToString();
            FileName.Text = CurrentMap.FileName;
            MapName.Text = CurrentMap.MapDescription;
            if (CurrentMap.MapType == MapType.Mir2)
                radioButton1.Checked = true;
            else
                radioButton2.Checked = true;

            MapFlags.ItemCheck -= MapFlags_ItemCheck;
            for (int I = 0; I < Enum.GetValues(typeof(Clash_Library.MapFlags)).Length; I++)
                if ((CurrentMap.Flags & (Clash_Library.MapFlags)Enum.GetValues(typeof(Clash_Library.MapFlags)).GetValue(I)) != 0)
                    MapFlags.SetItemChecked(I, true);
                else MapFlags.SetItemChecked(I, false);
            MapFlags.ItemCheck += new ItemCheckEventHandler(MapFlags_ItemCheck);

            MovementList.Items.Clear();
            MovementList.Items.AddRange(CurrentMap.MoveList.ToArray());
            SpawnList.Items.Clear();
            SpawnList.Items.AddRange(CurrentMap.RespawnList.ToArray());
        }

        private void MapForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = true;
            this.Hide();
        }

        private void AddMove_Click(object sender, EventArgs e)
        {
            CurrentMove = new MovementInfo();
            CurrentMap.MoveList.Add(CurrentMove);
            MovementList.Items.Add(CurrentMove);
            MovementList.SelectedIndex = MovementList.Items.Count - 1;
        }

        private void RemoveMove_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to remove this Movement?", "Remove Movement", MessageBoxButtons.YesNo) == System.Windows.Forms.DialogResult.Yes)
            {
                CurrentMap.MoveList.Remove(CurrentMove);
                MovementList.Items.Remove(CurrentMove);
            }

        }

        private void MovementList_SelectedIndexChanged(object sender, EventArgs e)
        {
            CurrentMove = (MovementInfo)MovementList.SelectedItem;
            if (CurrentMove == null) CurrentMove = new MovementInfo();
            UpdateMControls();
        }

        private void UpdateMControls()
        {
            SourceX.Text = CurrentMove.SourcePoint.X.ToString();
            SourceY.Text = CurrentMove.SourcePoint.Y.ToString();
            if (CurrentMove.DestMapInfo != null)
                DestMapID.SelectedIndex = CurrentMove.DestMapInfo.Index;
            else
                DestMapID.SelectedIndex = -1;
            DestX.Text = CurrentMove.DestinationPoint.X.ToString();
            DestY.Text = CurrentMove.DestinationPoint.Y.ToString();
        }

        private void Index_Leave(object sender, EventArgs e)
        {
            UpdateName();
        }

        private void MapName_Leave(object sender, EventArgs e)
        {

            UpdateName();
        }

        private void SourceX_TextChanged(object sender, EventArgs e)
        {
            int X;
            if (int.TryParse(SourceX.Text, out X))
                SourceX.BackColor = Color.Green;
            else
                SourceX.BackColor = Color.Red;
            CurrentMove.SourcePoint.X = X;
        }

        private void SourceY_TextChanged(object sender, EventArgs e)
        {
            int Y;
            if(int.TryParse(SourceY.Text, out Y))
                SourceY.BackColor = Color.Green;
            else
                SourceY.BackColor = Color.Red;
            CurrentMove.SourcePoint.Y = Y;
        }

        private void DestY_TextChanged(object sender, EventArgs e)
        {

            int Y;
            if (int.TryParse(DestY.Text, out Y))
                DestY.BackColor = Color.Green;
            else
                DestY.BackColor = Color.Red;
            CurrentMove.DestinationPoint.Y = Y;
        }

        private void DestX_TextChanged(object sender, EventArgs e)
        {
            int X;
            if (int.TryParse(DestX.Text, out X))
                DestX.BackColor = Color.Green;
            else
                DestX.BackColor = Color.Red;
            CurrentMove.DestinationPoint.X = X;
        }

        private void DestMapID_SelectedIndexChanged(object sender, EventArgs e)
        {
            CurrentMove.DestMapInfo = (MapInfo)DestMapID.SelectedItem;
            if (DestMapID.SelectedIndex == -1)
                DestMapID.BackColor = Color.Red;
            else
                DestMapID.BackColor = Color.Green;

        }

        private void SourceX_Leave(object sender, EventArgs e)
        {
            UpdateMList();
        }

        private void SourceY_Leave(object sender, EventArgs e)
        {
            UpdateMList();
        }

        private void DestX_Leave(object sender, EventArgs e)
        {
            UpdateMList();
        }

        private void DestY_Leave(object sender, EventArgs e)
        {
            UpdateMList();
        }

        private void DestMapID_Leave(object sender, EventArgs e)
        {
            UpdateMList();
        }
        private void UpdateMList()
        {
            int I = MovementList.SelectedIndex;
            if (I == -1) return;
            MovementList.SelectedIndexChanged -= MovementList_SelectedIndexChanged;
            MovementList.Items[I] = CurrentMove;
            MovementList.SelectedIndex = I;
            MovementList.SelectedIndexChanged += new EventHandler(MovementList_SelectedIndexChanged);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (File.Exists(@"D:\Maps.txt"))
            {
                string[] Lines = File.ReadAllLines(@"D:\Maps.txt");
                foreach (string Line in Lines)
                {
                    CurrentMap = new MapInfo();
                    string[] S = Line.Split('	');
                    CurrentMap.Index = int.Parse(S[0]);
                    CurrentMap.FileName = S[1];
                    CurrentMap.MapDescription = S[2];
                    if ("No" == S[4])
                        CurrentMap.MapType = MapType.Mir2;
                    else
                        CurrentMap.MapType = MapType.Mir3;
                    Main.SData.MapList.Add(CurrentMap);
                }
                MapListBox.Items.AddRange(Main.SData.MapList.ToArray());
                DestMapID.Items.AddRange(Main.SData.MapList.ToArray());
            }
            if (File.Exists(@"D:\Move.txt"))
            {
                string[] Lines = File.ReadAllLines(@"D:\Move.txt");
                foreach (string Line in Lines)
                {
                    CurrentMove = new MovementInfo();
                    string[] S = Line.Split('	');
                    CurrentMove.SourcePoint = new Point(int.Parse(S[1]), int.Parse(S[2]));
                    CurrentMove.DestMapInfo = (MapInfo)MapListBox.Items[int.Parse(S[3])];
                    CurrentMove.DestinationPoint = new Point(int.Parse(S[4]), int.Parse(S[5]));
                    foreach (MapInfo M in Main.SData.MapList)
                        if (M.Index == int.Parse(S[0]))
                        {
                            M.MoveList.Add(CurrentMove);
                            break;
                        }
                }
                MapListBox.Items.AddRange(Main.SData.MapList.ToArray());
                DestMapID.Items.AddRange(Main.SData.MapList.ToArray());
            }
        }

        private void AddSpawn_Click(object sender, EventArgs e)
        {
            CurrentSpawn = new RespawnInfo();
            CurrentMap.RespawnList.Add(CurrentSpawn);
            SpawnList.Items.Add(CurrentSpawn);
            SpawnList.SelectedIndex = SpawnList.Items.Count - 1;
        }

        private void RemoveSpawn_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to remove this Respawn?", "Remove Respawn", MessageBoxButtons.YesNo) == System.Windows.Forms.DialogResult.Yes)
            {
                CurrentMap.RespawnList.Remove(CurrentSpawn);
                SpawnList.Items.Remove(CurrentSpawn);
            }
        }

        private void MonsterList_SelectedIndexChanged(object sender, EventArgs e)
        {
            CurrentSpawn.Monster = (MonsterInfo)MonsterList.SelectedItem;
        }

        private void SpawnAmount_TextChanged(object sender, EventArgs e)
        {
            if (int.TryParse(SpawnAmount.Text, out CurrentSpawn.Amount))
                SpawnAmount.BackColor = Color.Green;
            else
                SpawnAmount.BackColor = Color.Red;
        }

        private void SpawnRadius_TextChanged(object sender, EventArgs e)
        {
            if (int.TryParse(SpawnRadius.Text, out CurrentSpawn.Spread))
                SpawnRadius.BackColor = Color.Green;
            else
                SpawnRadius.BackColor = Color.Red;
        }

        private void SpawnX_TextChanged(object sender, EventArgs e)
        {
            int X;
            if (int.TryParse(SpawnX.Text, out X))
                SpawnX.BackColor = Color.Green;
            else
                SpawnX.BackColor = Color.Red;
            CurrentSpawn.Location.X = X;
        }

        private void SpawnY_TextChanged(object sender, EventArgs e)
        {
            int Y;
            if (int.TryParse(SpawnY.Text, out Y))
                SpawnY.BackColor = Color.Green;
            else
                SpawnY.BackColor = Color.Red;
            CurrentSpawn.Location.Y = Y;
        }

        private void SpawnDelay_TextChanged(object sender, EventArgs e)
        {
            int D;
            if (int.TryParse(SpawnDelay.Text, out D))
                SpawnDelay.BackColor = Color.Green;
            else
                SpawnDelay.BackColor = Color.Red;
            CurrentSpawn.RespawnTime = new TimeSpan(0, D, 0);
        }

        private void SpawnDistort_TextChanged(object sender, EventArgs e)
        {
            int D;
            if (int.TryParse(SpawnDistort.Text, out D))
                SpawnDistort.BackColor = Color.Green;
            else
                SpawnDistort.BackColor = Color.Red;
            CurrentSpawn.DistortionTime = D;
        }

        private void MonsterList_Leave(object sender, EventArgs e)
        {
            UpdateSpawn();
        }

        private void SpawnAmount_Leave(object sender, EventArgs e)
        {
            UpdateSpawn();
        }

        private void SpawnRadius_Leave(object sender, EventArgs e)
        {
            UpdateSpawn();
        }

        private void SpawnX_Leave(object sender, EventArgs e)
        {
            UpdateSpawn();
        }

        private void SpawnY_Leave(object sender, EventArgs e)
        {
            UpdateSpawn();
        }

        private void SpawnDelay_Leave(object sender, EventArgs e)
        {
            UpdateSpawn();
        }

        private void SpawnDistort_Leave(object sender, EventArgs e)
        {
            UpdateSpawn();

        }
        private void UpdateSpawn()
        {
            int I = SpawnList.SelectedIndex;
            if (I == -1) return;
            SpawnList.SelectedIndexChanged -= SpawnList_SelectedIndexChanged;
            SpawnList.Items[I] = CurrentSpawn;
            SpawnList.SelectedIndex = I;
            SpawnList.SelectedIndexChanged += new EventHandler(SpawnList_SelectedIndexChanged);
        }

        private void SpawnList_SelectedIndexChanged(object sender, EventArgs e)
        {
            CurrentSpawn = (RespawnInfo)SpawnList.SelectedItem;
            if (CurrentSpawn == null) CurrentSpawn = new RespawnInfo();
            UpdateSControls();

        }
        private void UpdateSControls()
        {
            if (CurrentSpawn.Monster == null)
                MonsterList.Text = "";
            else 
                MonsterList.Text = CurrentSpawn.Monster.ToString();
            SpawnAmount.Text = CurrentSpawn.Amount.ToString();
            SpawnRadius.Text = CurrentSpawn.Spread.ToString();

            SpawnX.Text = CurrentSpawn.Location.X.ToString();
            SpawnY.Text = CurrentSpawn.Location.Y.ToString();

            SpawnDelay.Text = CurrentSpawn.RespawnTime.TotalMinutes.ToString();
            SpawnDistort.Text = CurrentSpawn.DistortionTime.ToString(); 
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if (!File.Exists(@"D:\Spawns.txt")) return;
            string[] Lines = File.ReadAllLines(@"D:\Spawns.txt");
            foreach (string S in Lines)
            {
                string[] Line = S.Split('	');
                CurrentSpawn = new RespawnInfo();
                if (!string.IsNullOrEmpty(Line[1])) 
                    foreach (MonsterInfo M in Main.SData.MonsterList)
                    if (M.Index == int.Parse(Line[1]))
                    {
                        CurrentSpawn.Monster = M;
                        break;
                    }

                if (!string.IsNullOrEmpty(Line[2])) CurrentSpawn.RespawnTime = new TimeSpan(0, int.Parse(Line[2]), 0);
                if (!string.IsNullOrEmpty(Line[3])) CurrentSpawn.Amount = int.Parse(Line[3]);

                if (!string.IsNullOrEmpty(Line[5]) && !string.IsNullOrEmpty(Line[6])) CurrentSpawn.Location = new Point(int.Parse(Line[5]), int.Parse(Line[6]));
                if (!string.IsNullOrEmpty(Line[7])) CurrentSpawn.Spread = int.Parse(Line[7]);



                if (!string.IsNullOrEmpty(Line[8]))  CurrentSpawn.DistortionTime = int.Parse(Line[8]);
                if (!string.IsNullOrEmpty(Line[4])) 
                    foreach (MapInfo M in Main.SData.MapList)
                    if (M.Index == int.Parse(Line[4]))
                    {
                        M.RespawnList.Add(CurrentSpawn);
                        break;
                    }
            }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            using (MemoryStream MStream = new MemoryStream())
            {
                BinaryFormatter BF = new BinaryFormatter();
                BF.Serialize(MStream, CurrentSpawn);
                MStream.Position = 0;
                CopySpawn = (RespawnInfo)BF.Deserialize(MStream);
            }

            if (CopySpawn != null)
            {
                button3.Enabled = true;
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            CurrentMap.RespawnList.Add(CopySpawn);
            SpawnList.Items.Add(CopySpawn);
            SpawnList.SelectedIndex = SpawnList.Items.Count - 1;
            button3.Enabled = false;
        }
    }
}
