﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Clash_Library;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;

namespace ClashManager
{
    public partial class MonsterForm : Form
    {
        MonsterInfo CurrentMonster = new MonsterInfo();
        DropInfo CurrentDrop = new DropInfo();
        public MonsterForm()
        {
            InitializeComponent();
        }

        private void HP_TextChanged(object sender, EventArgs e)
        {
            if (int.TryParse(HP.Text, out CurrentMonster.Stats.Basic.HP))
                HP.BackColor = Color.Green;
            else
                HP.BackColor = Color.Red;
        }

        private void GoldRate_TextChanged(object sender, EventArgs e)
        {
            int tempInt;
            if (int.TryParse(GoldRate.Text, out tempInt))
            {
                if (tempInt != 0)
                    CurrentMonster.GoldRate = 1D / tempInt;
                else
                    CurrentMonster.GoldRate = 0;
                GoldRate.BackColor = Color.Green;
            }
            else
            {
                CurrentMonster.GoldRate = 0;
                GoldRate.BackColor = Color.Red;
            }

        }

        private void ColButton_Click(object sender, EventArgs e)
        {
            if (ColourDiag.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                NameColour.Text = ColourDiag.Color.ToString();
                CurrentMonster.NameColour = ColourDiag.Color;
            }
        }

        private void MonsterForm_Load(object sender, EventArgs e)
        {
            MonsterListBox.Items.AddRange(Main.SData.MonsterList.ToArray());
            AI.Items.AddRange(Enum.GetNames(typeof(Clash_Library.MonsterAI)));
            DropList.Items.AddRange(CurrentMonster.DropList.ToArray());
            ItemList.Items.AddRange(Main.SData.ItemList.ToArray());
        }

        private void Index_TextChanged(object sender, EventArgs e)
        {

            if (int.TryParse(Index.Text, out CurrentMonster.Index))
                Index.BackColor = Color.Green;
            else
                Index.BackColor = Color.Red;
        }

        private void MonsterName_TextChanged(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(MonsterName.Text))
                MonsterName.BackColor = Color.Green;
            else
                MonsterName.BackColor = Color.Red;
            CurrentMonster.Name = MonsterName.Text;
        }

        private void AI_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(AI.Text))
            {
                CurrentMonster.AI = (MonsterAI)Enum.Parse(typeof(MonsterAI), AI.Text);
                AI.BackColor = Color.Green;
            }
            else
                AI.BackColor = Color.Red;
        }

        private void Image_TextChanged(object sender, EventArgs e)
        {
            if (int.TryParse(Image.Text, out CurrentMonster.Image))
                Image.BackColor = Color.Green;
            else
                Image.BackColor = Color.Red;
        }

        private void SpellImage_TextChanged(object sender, EventArgs e)
        {
            if (int.TryParse(SpellImage.Text, out CurrentMonster.SpellImage))
                SpellImage.BackColor = Color.Green;
            else
                SpellImage.BackColor = Color.Red;

        }

        private void Experience_TextChanged(object sender, EventArgs e)
        {
            if (long.TryParse(Experience.Text, out CurrentMonster.Experience))
                Experience.BackColor = Color.Green;
            else
                Experience.BackColor = Color.Red;
        }

        private void Level_TextChanged(object sender, EventArgs e)
        {
            if (int.TryParse(Level.Text, out CurrentMonster.Level))
                Level.BackColor = Color.Green;
            else
                Level.BackColor = Color.Red;
        }

        private void AddMonster_Click(object sender, EventArgs e)
        {
            CurrentMonster = new MonsterInfo();
            CurrentMonster.Index = MonsterListBox.Items.Count;
            Main.SData.MonsterList.Add(CurrentMonster);
            MonsterListBox.Items.Add(CurrentMonster);
            MonsterListBox.SelectedIndex = MonsterListBox.Items.Count - 1;
        }

        private void CopyMonster_Click(object sender, EventArgs e)
        {
            using (MemoryStream MStream = new MemoryStream())
            {
                BinaryFormatter BF = new BinaryFormatter();
                BF.Serialize(MStream, CurrentMonster);
                MStream.Position = 0;
                CurrentMonster = (MonsterInfo)BF.Deserialize(MStream);
                CurrentMonster.Index = MonsterListBox.Items.Count;
                Main.SData.MonsterList.Add(CurrentMonster);
                MonsterListBox.Items.Add(CurrentMonster);
                MonsterListBox.SelectedIndex = MonsterListBox.Items.Count - 1;
            }
        }

        private void AC_TextChanged(object sender, EventArgs e)
        {
            if (short.TryParse(AC.Text, out CurrentMonster.Stats.Basic.AC))
                AC.BackColor = Color.Green;
            else
                AC.BackColor = Color.Red;
        }

        private void MAC_TextChanged(object sender, EventArgs e)
        {
            if (short.TryParse(MAC.Text, out CurrentMonster.Stats.Basic.MAC))
                MAC.BackColor = Color.Green;
            else
                MAC.BackColor = Color.Red;
        }

        private void MP_TextChanged(object sender, EventArgs e)
        {
            if (int.TryParse(MP.Text, out CurrentMonster.Stats.Basic.MP))
                MP.BackColor = Color.Green;
            else
                MP.BackColor = Color.Red;
        }

        private void AC2_TextChanged(object sender, EventArgs e)
        {
            if (short.TryParse(AC2.Text, out CurrentMonster.Stats.Basic.AC2))
                AC2.BackColor = Color.Green;
            else
                AC2.BackColor = Color.Red;
        }

        private void MAC2_TextChanged(object sender, EventArgs e)
        {
            if (short.TryParse(MAC2.Text, out CurrentMonster.Stats.Basic.MAC2))
                MAC2.BackColor = Color.Green;
            else
                MAC2.BackColor = Color.Red;
        }

        private void Accuracy_TextChanged(object sender, EventArgs e)
        {
            if (short.TryParse(Accuracy.Text, out CurrentMonster.Stats.Basic.Accuracy))
                Accuracy.BackColor = Color.Green;
            else
                Accuracy.BackColor = Color.Red;
        }

        private void Agility_TextChanged(object sender, EventArgs e)
        {
            if (short.TryParse(Agility.Text, out CurrentMonster.Stats.Basic.Agility))
                Agility.BackColor = Color.Green;
            else
                Agility.BackColor = Color.Red;
        }

        private void SC_TextChanged(object sender, EventArgs e)
        {
            if (short.TryParse(SC.Text, out CurrentMonster.Stats.Basic.SC))
                SC.BackColor = Color.Green;
            else
                SC.BackColor = Color.Red;
        }

        private void MC_TextChanged(object sender, EventArgs e)
        {
            if (short.TryParse(MC.Text, out CurrentMonster.Stats.Basic.MC))
                MC.BackColor = Color.Green;
            else
                MC.BackColor = Color.Red;
        }

        private void DC_TextChanged(object sender, EventArgs e)
        {
            if (short.TryParse(DC.Text, out CurrentMonster.Stats.Basic.DC))
                DC.BackColor = Color.Green;
            else
                DC.BackColor = Color.Red;
        }

        private void DC2_TextChanged(object sender, EventArgs e)
        {
            if (short.TryParse(DC2.Text, out CurrentMonster.Stats.Basic.DC2))
                DC2.BackColor = Color.Green;
            else
                DC2.BackColor = Color.Red;
        }

        private void MC2_TextChanged(object sender, EventArgs e)
        {
            if (short.TryParse(MC2.Text, out CurrentMonster.Stats.Basic.MC2))
                MC2.BackColor = Color.Green;
            else
                MC2.BackColor = Color.Red;
        }

        private void SC2_TextChanged(object sender, EventArgs e)
        {
            if (short.TryParse(SC2.Text, out CurrentMonster.Stats.Basic.SC2))
                SC2.BackColor = Color.Green;
            else
                SC2.BackColor = Color.Red;
        }

        private void Remove_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to remove this Monster?", "Remove Monster", MessageBoxButtons.YesNo) == System.Windows.Forms.DialogResult.Yes)
            {
                MonsterInfo DelMob = CurrentMonster;
                Main.SData.MonsterList.Remove(DelMob);
                MonsterListBox.Items.Remove(DelMob);
                for (int I = 0; I < Main.SData.MonsterList.Count; I++)
                    Main.SData.MonsterList[I].Index = I;
                //Adjust Respawns
                MonsterListBox.Items.Clear();
                MonsterListBox.Items.AddRange(Main.SData.MonsterList.ToArray());

                foreach (MapInfo M in Main.SData.MapList)
                    for (int I = M.RespawnList.Count -1; I >= 0; I--)
                    {
                        if (M.RespawnList[I].Monster == DelMob)
                            M.RespawnList.RemoveAt(I);
                    }
            }
        }

        private void SaveMonster_Click(object sender, EventArgs e)
        {
            if (!Directory.Exists(@".\Data\")) Directory.CreateDirectory(@".\Data\");
            BinaryFormatter BF = new BinaryFormatter();
            using (FileStream FStream = new FileStream(@".\Data\Data.CL", FileMode.Create))
                BF.Serialize(FStream, Main.SData);
        }

        private void MonsterListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            tabControl1.Enabled = true;
            CurrentMonster = (MonsterInfo)MonsterListBox.SelectedItem;
            if (CurrentMonster == null) CurrentMonster = new MonsterInfo();
            UpdateControls();
            DropList.SelectedIndex = DropList.Items.Count - 1;
        }

        private void UpdateControls()
        {

            Index.Text = CurrentMonster.Index.ToString();
            MonsterName.Text = CurrentMonster.Name;
            AI.Text = CurrentMonster.AI.ToString();
            Image.Text = CurrentMonster.Image.ToString();
            SpellImage.Text = CurrentMonster.SpellImage.ToString();
            NameColour.Text = CurrentMonster.NameColour.ToString();

            Experience.Text = CurrentMonster.Experience.ToString();
            Level.Text = CurrentMonster.Level.ToString();


            HP.Text = CurrentMonster.Stats.Basic.HP.ToString();
            MP.Text = CurrentMonster.Stats.Basic.MP.ToString();
            AC.Text = CurrentMonster.Stats.Basic.AC.ToString();
            AC2.Text = CurrentMonster.Stats.Basic.AC2.ToString();
            MAC.Text = CurrentMonster.Stats.Basic.MAC.ToString();
            MAC2.Text = CurrentMonster.Stats.Basic.MAC2.ToString();

            DC.Text = CurrentMonster.Stats.Basic.DC.ToString();
            DC2.Text = CurrentMonster.Stats.Basic.DC2.ToString();
            MC.Text = CurrentMonster.Stats.Basic.MC.ToString();
            MC2.Text = CurrentMonster.Stats.Basic.MC2.ToString();
            SC.Text = CurrentMonster.Stats.Basic.SC.ToString();
            SC2.Text = CurrentMonster.Stats.Basic.SC2.ToString();


            Accuracy.Text = CurrentMonster.Stats.Basic.Accuracy.ToString();
            Agility.Text = CurrentMonster.Stats.Basic.Agility.ToString();

            ASpeed.Text = CurrentMonster.Stats.Advanced.ASpeed.ToString();
            Luck.Text = CurrentMonster.Stats.Advanced.Luck.ToString();
            Light.Text = CurrentMonster.Stats.Advanced.Light.ToString();
            MR.Text = CurrentMonster.Stats.Advanced.MR.ToString();
            PR.Text = CurrentMonster.Stats.Advanced.PR.ToString();
            CR.Text = CurrentMonster.Stats.Advanced.CR.ToString();
            CA.Text = CurrentMonster.Stats.Advanced.CA.ToString();
            PA.Text = CurrentMonster.Stats.Advanced.PA.ToString();


            FireA.Text = CurrentMonster.Stats.Elemental.FireA.ToString();
            LightningA.Text = CurrentMonster.Stats.Elemental.LightningA.ToString();
            IceA.Text = CurrentMonster.Stats.Elemental.IceA.ToString();
            WindA.Text = CurrentMonster.Stats.Elemental.WindA.ToString();
            DarkA.Text = CurrentMonster.Stats.Elemental.DarkA.ToString();
            HolyA.Text = CurrentMonster.Stats.Elemental.HolyA.ToString();
            IllusionA.Text = CurrentMonster.Stats.Elemental.IllusionA.ToString();
            FireD.Text = CurrentMonster.Stats.Elemental.FireD.ToString();
            LightningD.Text = CurrentMonster.Stats.Elemental.LightningD.ToString();
            IceD.Text = CurrentMonster.Stats.Elemental.IceD.ToString();
            WindD.Text = CurrentMonster.Stats.Elemental.WindD.ToString();
            DarkD.Text = CurrentMonster.Stats.Elemental.DarkD.ToString();
            HolyD.Text = CurrentMonster.Stats.Elemental.HolyD.ToString();
            IllusionD.Text = CurrentMonster.Stats.Elemental.IllusionD.ToString();
            Gold.Text = CurrentMonster.Gold.ToString();
            if (CurrentMonster.GoldRate != 0)
                GoldRate.Text = (1 / CurrentMonster.GoldRate).ToString();
            else
                GoldRate.Text = "Never";

            CoolEye.Value = CurrentMonster.Behaviour.CoolEye;
            Undead.Checked = CurrentMonster.Behaviour.Undead;
            Tameable.Checked = CurrentMonster.Behaviour.Tameable;
            Moveable.Checked = CurrentMonster.Behaviour.Moveable;
            Turnable.Checked = CurrentMonster.Behaviour.Turnable;
            CanMove.Checked = CurrentMonster.Behaviour.CanMove;
            Passive.Checked = CurrentMonster.Behaviour.Passive;
            HideName.Checked = CurrentMonster.Behaviour.HideName;
            DropList.Items.Clear();
            DropList.Items.AddRange(CurrentMonster.DropList.ToArray());
            textBox1.Text = CurrentMonster.MoveSpeed.TotalMilliseconds.ToString();
        }

        private void ASpeed_TextChanged(object sender, EventArgs e)
        {
            if (short.TryParse(ASpeed.Text, out CurrentMonster.Stats.Advanced.ASpeed))
                ASpeed.BackColor = Color.Green;
            else
                ASpeed.BackColor = Color.Red;
        }

        private void Luck_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(Luck.Text, out CurrentMonster.Stats.Advanced.Luck))
                Luck.BackColor = Color.Green;
            else
                Luck.BackColor = Color.Red;
        }

        private void Light_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(Light.Text, out CurrentMonster.Stats.Advanced.Light))
                Light.BackColor = Color.Green;
            else
                Light.BackColor = Color.Red;
        }

        private void MR_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(MR.Text, out CurrentMonster.Stats.Advanced.MR))
                MR.BackColor = Color.Green;
            else
                MR.BackColor = Color.Red;
        }

        private void PR_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(PR.Text, out CurrentMonster.Stats.Advanced.PR))
                PR.BackColor = Color.Green;
            else
                PR.BackColor = Color.Red;
        }

        private void CR_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(CR.Text, out CurrentMonster.Stats.Advanced.CR))
                CR.BackColor = Color.Green;
            else
                CR.BackColor = Color.Red;
        }

        private void PA_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(PA.Text, out CurrentMonster.Stats.Advanced.PA))
                PA.BackColor = Color.Green;
            else
                PA.BackColor = Color.Red;
        }

        private void CA_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(CA.Text, out CurrentMonster.Stats.Advanced.CA))
                CA.BackColor = Color.Green;
            else
                CA.BackColor = Color.Red;
        }

        private void FireA_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(FireA.Text, out CurrentMonster.Stats.Elemental.FireA))
                FireA.BackColor = Color.Green;
            else
                FireA.BackColor = Color.Red;
        }

        private void LightningA_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(LightningA.Text, out CurrentMonster.Stats.Elemental.LightningA))
                LightningA.BackColor = Color.Green;
            else
                LightningA.BackColor = Color.Red;
        }

        private void IceA_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(FireA.Text, out CurrentMonster.Stats.Elemental.IceA))
                IceA.BackColor = Color.Green;
            else
                IceA.BackColor = Color.Red;
        }

        private void WindA_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(WindA.Text, out CurrentMonster.Stats.Elemental.WindA))
                WindA.BackColor = Color.Green;
            else
                WindA.BackColor = Color.Red;
        }

        private void DarkA_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(DarkA.Text, out CurrentMonster.Stats.Elemental.DarkA))
                DarkA.BackColor = Color.Green;
            else
                DarkA.BackColor = Color.Red;
        }

        private void HolyA_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(HolyA.Text, out CurrentMonster.Stats.Elemental.HolyA))
                HolyA.BackColor = Color.Green;
            else
                HolyA.BackColor = Color.Red;
        }

        private void IllusionA_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(IllusionA.Text, out CurrentMonster.Stats.Elemental.IllusionA))
                IllusionA.BackColor = Color.Green;
            else
                IllusionA.BackColor = Color.Red;
        }

        private void IllusionD_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(IllusionD.Text, out CurrentMonster.Stats.Elemental.IllusionD))
                IllusionD.BackColor = Color.Green;
            else
                IllusionD.BackColor = Color.Red;
        }

        private void HolyD_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(HolyD.Text, out CurrentMonster.Stats.Elemental.HolyD))
                HolyD.BackColor = Color.Green;
            else
                HolyD.BackColor = Color.Red;
        }

        private void DarkD_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(DarkD.Text, out CurrentMonster.Stats.Elemental.DarkD))
                DarkD.BackColor = Color.Green;
            else
                DarkD.BackColor = Color.Red;
        }

        private void WindD_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(WindD.Text, out CurrentMonster.Stats.Elemental.WindD))
                WindD.BackColor = Color.Green;
            else
                WindD.BackColor = Color.Red;
        }

        private void IceD_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(IceD.Text, out CurrentMonster.Stats.Elemental.IceD))
                IceD.BackColor = Color.Green;
            else
                IceD.BackColor = Color.Red;
        }

        private void LightningD_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(LightningD.Text, out CurrentMonster.Stats.Elemental.LightningD))
                LightningD.BackColor = Color.Green;
            else
                LightningD.BackColor = Color.Red;
        }

        private void FireD_TextChanged(object sender, EventArgs e)
        {
            if (sbyte.TryParse(FireD.Text, out CurrentMonster.Stats.Elemental.FireD))
                FireD.BackColor = Color.Green;
            else
                FireD.BackColor = Color.Red;
        }

        private void Gold_TextChanged(object sender, EventArgs e)
        {
            if (long.TryParse(Gold.Text, out CurrentMonster.Gold))
                Gold.BackColor = Color.Green;
            else
                Gold.BackColor = Color.Red;
        }

        private void LightningD_TextChanged_1(object sender, EventArgs e)
        {
            if (sbyte.TryParse(LightningD.Text, out CurrentMonster.Stats.Elemental.LightningD))
                LightningD.BackColor = Color.Green;
            else
                LightningD.BackColor = Color.Red;
        }

        private void Undead_CheckedChanged(object sender, EventArgs e)
        {
            CurrentMonster.Behaviour.Undead = Undead.Checked;
        }

        private void CanMove_CheckedChanged(object sender, EventArgs e)
        {
            CurrentMonster.Behaviour.CanMove = CanMove.Checked;
        }

        private void Passive_CheckedChanged(object sender, EventArgs e)
        {
            CurrentMonster.Behaviour.Passive = Passive.Checked;
        }

        private void Tameable_CheckedChanged(object sender, EventArgs e)
        {
            CurrentMonster.Behaviour.Tameable = Tameable.Checked;
        }

        private void Moveable_CheckedChanged(object sender, EventArgs e)
        {
            CurrentMonster.Behaviour.Moveable = Moveable.Checked;
        }

        private void HideName_CheckedChanged(object sender, EventArgs e)
        {
            CurrentMonster.Behaviour.HideName = HideName.Checked;
        }

        private void Turnable_CheckedChanged(object sender, EventArgs e)
        {

            CurrentMonster.Behaviour.Turnable = Turnable.Checked;
        }

        private void CoolEye_ValueChanged(object sender, EventArgs e)
        {
            CurrentMonster.Behaviour.CoolEye = (byte)CoolEye.Value;
        }

        private void DropList_SelectedIndexChanged(object sender, EventArgs e)
        {
            CurrentDrop = (DropInfo)DropList.SelectedItem;
            if (CurrentDrop.Item == null)
                ItemList.Text = "";
                else
            ItemList.Text = CurrentDrop.Item.ToString();
            UpdateDControls();
        }
        private void UpdateDControls()
        {
            if (CurrentDrop.Item == null)
                ItemList.Text = "";
            else
                ItemList.Text = CurrentDrop.Item.ToString();
            DropChance.Text = (1 / CurrentDrop.Chance).ToString();
            DropAmount.Text = CurrentDrop.Amount.ToString();
            DropEnabled.Checked = CurrentDrop.Enabled;
        }

        private void ItemList_SelectedIndexChanged(object sender, EventArgs e)
        {
            CurrentDrop.Item = (ItemInfo)ItemList.SelectedItem;
        }

        private void DropChance_TextChanged(object sender, EventArgs e)
        {
            int tempInt;
            if (int.TryParse(DropChance.Text, out tempInt))
            {
                if (tempInt != 0)
                    CurrentDrop.Chance = 1D / tempInt;
                else
                    CurrentDrop.Chance = 0;
                DropChance.BackColor = Color.Green;
            }
            else
            {
                CurrentDrop.Chance = 0;
                DropChance.BackColor = Color.Red;
            }

        }

        private void DropAmount_TextChanged(object sender, EventArgs e)
        {
            if (ushort.TryParse(DropAmount.Text, out CurrentDrop.Amount))
                DropAmount.BackColor = Color.Green;
            else
                DropAmount.BackColor = Color.Red;
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            CurrentDrop.Enabled = DropEnabled.Checked;
        }

        private void AddDrop_Click(object sender, EventArgs e)
        {
            CurrentDrop = new DropInfo();
            CurrentMonster.DropList.Add(CurrentDrop);
            DropList.Items.Add(CurrentDrop);
            DropList.SelectedIndex = DropList.Items.Count -1;
        }

        private void DropChance_Leave(object sender, EventArgs e)
        {
            UpdateDrop();
        }

        private void ItemList_Leave(object sender, EventArgs e)
        {
            UpdateDrop();
        }

        private void DropAmount_Leave(object sender, EventArgs e)
        {
            UpdateDrop();
        }

        private void DropEnabled_Leave(object sender, EventArgs e)
        {
            UpdateDrop();
        }

        private void Index_Leave(object sender, EventArgs e)
        {
            UpdateName();
        }

        private void MonsterName_Leave(object sender, EventArgs e)
        {

            UpdateName();
        }
        private void UpdateName()
        {
            int I = MonsterListBox.SelectedIndex;
            if (I == -1) return;
            MonsterListBox.SelectedIndexChanged -= MonsterListBox_SelectedIndexChanged;
            MonsterListBox.Items[I] = CurrentMonster;
            MonsterListBox.SelectedItem = I;
            MonsterListBox.SelectedIndexChanged += new EventHandler(MonsterListBox_SelectedIndexChanged);
        }
        private void UpdateDrop()
        {
            int I = DropList.SelectedIndex;
            if (I == -1) return;
            DropList.SelectedIndexChanged -= DropList_SelectedIndexChanged;
            DropList.Items[I] = CurrentDrop;
            DropList.SelectedItem = I;
            DropList.SelectedIndexChanged += new EventHandler(DropList_SelectedIndexChanged);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (!File.Exists(@"D:\Monsters.txt")) return;
            string[] Lines = File.ReadAllLines(@"D:\Monsters.txt");
            foreach (string S in Lines)
            {
                CurrentMonster = new MonsterInfo();
                string[] Line = S.Split('	');
                CurrentMonster.Index = int.Parse(Line[0]);
                CurrentMonster.Name = Line[1];
                if (!string.IsNullOrEmpty(Line[5])) CurrentMonster.Level = int.Parse(Line[5]);
                if (!string.IsNullOrEmpty(Line[6])) CurrentMonster.Experience = long.Parse(Line[6]);
                CurrentMonster.Behaviour.Undead = bool.Parse(Line[7]);
                if (!string.IsNullOrEmpty(Line[8])) CurrentMonster.Behaviour.CoolEye = byte.Parse(Line[8]);
                if (!string.IsNullOrEmpty(Line[9])) CurrentMonster.Stats.Basic.HP = int.Parse(Line[9]);
                if (!string.IsNullOrEmpty(Line[10])) CurrentMonster.Stats.Basic.MP = int.Parse(Line[10]);

                if (!string.IsNullOrEmpty(Line[11])) CurrentMonster.Stats.Basic.AC = short.Parse(Line[11]);
                if (!string.IsNullOrEmpty(Line[12])) CurrentMonster.Stats.Basic.AC2 = short.Parse(Line[12]);
                if (!string.IsNullOrEmpty(Line[13])) CurrentMonster.Stats.Basic.MAC = short.Parse(Line[13]);
                if (!string.IsNullOrEmpty(Line[14])) CurrentMonster.Stats.Basic.MAC2 = short.Parse(Line[14]);

                if (!string.IsNullOrEmpty(Line[15])) CurrentMonster.Stats.Basic.DC = short.Parse(Line[15]);
                if (!string.IsNullOrEmpty(Line[16])) CurrentMonster.Stats.Basic.DC2 = short.Parse(Line[16]);
                if (!string.IsNullOrEmpty(Line[17])) CurrentMonster.Stats.Basic.MC = short.Parse(Line[17]);
                if (!string.IsNullOrEmpty(Line[18])) CurrentMonster.Stats.Basic.MC2 = short.Parse(Line[18]);
                if (!string.IsNullOrEmpty(Line[19])) CurrentMonster.Stats.Basic.SC = short.Parse(Line[19]);
                if (!string.IsNullOrEmpty(Line[20])) CurrentMonster.Stats.Basic.SC2 = short.Parse(Line[20]);

                if (!string.IsNullOrEmpty(Line[21])) CurrentMonster.Stats.Basic.Accuracy = short.Parse(Line[21]);
                if (!string.IsNullOrEmpty(Line[22])) CurrentMonster.Stats.Basic.Agility = short.Parse(Line[22]);

                if (!string.IsNullOrEmpty(Line[23])) CurrentMonster.Stats.Advanced.CA = sbyte.Parse(Line[23]);
                if (!string.IsNullOrEmpty(Line[24])) CurrentMonster.Stats.Advanced.PA = sbyte.Parse(Line[24]);
                if (!string.IsNullOrEmpty(Line[25])) CurrentMonster.Stats.Advanced.MR = sbyte.Parse(Line[25]);
                if (!string.IsNullOrEmpty(Line[26])) CurrentMonster.Stats.Advanced.CR = sbyte.Parse(Line[26]);
                if (!string.IsNullOrEmpty(Line[27])) CurrentMonster.Stats.Advanced.PR = sbyte.Parse(Line[27]);
                if (!string.IsNullOrEmpty(Line[28])) CurrentMonster.MoveSpeed = new TimeSpan(0, 0, 0, 0, short.Parse(Line[28]));
                if (!string.IsNullOrEmpty(Line[29])) CurrentMonster.AttackSpeed = new TimeSpan(0, 0, 0, 0, short.Parse(Line[29]));

                if (!string.IsNullOrEmpty(Line[29])) CurrentMonster.Behaviour.CanMove = bool.Parse(Line[30]);
                CurrentMonster.Behaviour.Tameable = bool.Parse(Line[31]);
                CurrentMonster.Behaviour.Moveable = bool.Parse(Line[32]);

                CurrentMonster.Behaviour.Turnable = bool.Parse(Line[34]);
                CurrentMonster.Behaviour.Passive = bool.Parse(Line[35]);


                if (!string.IsNullOrEmpty(Line[36])) CurrentMonster.Stats.Advanced.Light = sbyte.Parse(Line[36]);
                if (!string.IsNullOrEmpty(Line[37])) CurrentMonster.Stats.Advanced.Luck = sbyte.Parse(Line[37]);

                if (!string.IsNullOrEmpty(Line[38])) CurrentMonster.Stats.Elemental.FireA = sbyte.Parse(Line[38]);
                if (!string.IsNullOrEmpty(Line[39])) CurrentMonster.Stats.Elemental.LightningA = sbyte.Parse(Line[39]);
                if (!string.IsNullOrEmpty(Line[40])) CurrentMonster.Stats.Elemental.IceA = sbyte.Parse(Line[40]);
                if (!string.IsNullOrEmpty(Line[41])) CurrentMonster.Stats.Elemental.WindA = sbyte.Parse(Line[41]);
                if (!string.IsNullOrEmpty(Line[42])) CurrentMonster.Stats.Elemental.HolyA = sbyte.Parse(Line[42]);
                if (!string.IsNullOrEmpty(Line[43])) CurrentMonster.Stats.Elemental.DarkA = sbyte.Parse(Line[43]);
                if (!string.IsNullOrEmpty(Line[44])) CurrentMonster.Stats.Elemental.IllusionA = sbyte.Parse(Line[44]);

                if (!string.IsNullOrEmpty(Line[45]))  CurrentMonster.Stats.Elemental.FireD = sbyte.Parse(Line[45]);
                if (!string.IsNullOrEmpty(Line[46])) CurrentMonster.Stats.Elemental.LightningD = sbyte.Parse(Line[46]);
                if (!string.IsNullOrEmpty(Line[47])) CurrentMonster.Stats.Elemental.IceD = sbyte.Parse(Line[47]);
                if (!string.IsNullOrEmpty(Line[48])) CurrentMonster.Stats.Elemental.WindD = sbyte.Parse(Line[48]);
                if (!string.IsNullOrEmpty(Line[49])) CurrentMonster.Stats.Elemental.HolyD = sbyte.Parse(Line[49]);
                if (!string.IsNullOrEmpty(Line[50])) CurrentMonster.Stats.Elemental.DarkD = sbyte.Parse(Line[50]);
                if (!string.IsNullOrEmpty(Line[51])) CurrentMonster.Stats.Elemental.IllusionD = sbyte.Parse(Line[51]);
                Main.SData.MonsterList.Add(CurrentMonster);
                MonsterListBox.Items.Add(CurrentMonster);
            }
        }

        private void MonsterForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = true;
            this.Hide();
        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            int Milli;

            if (int.TryParse(textBox1.Text, out Milli))
            {
                CurrentMonster.MoveSpeed = new TimeSpan(0, 0, 0, 0, Milli);
                textBox1.BackColor = Color.Green;
            }
            else
                textBox1.BackColor = Color.Red;
        }

    }
}
