﻿using System.Windows.Forms;
using Clash_Library;
using System.Drawing;
using System.Collections.Generic;
using System;

namespace Mir_Manager
{
    public partial class Map_Form : Form
    {
        private bool Selecting = false;
        public CurrencyManager MapManager, MoveManager, SpawnManager;

        public Map_Form()
        {
            InitializeComponent();
            MapFlagsList.Items.AddRange(Enum.GetNames(typeof(MapFlags)));

        }

        public void RefreshLists()
        {
            bool[] SelList = new bool[MapList.Items.Count];
            for (int X = 0; X < SelList.Length; X++)
                SelList[X] = MapList.GetSelected(X);

            MapManager.Refresh();
            MapCombo.Items.Clear();
            MapCombo.Items.AddRange(Main.SData.MapList.ToArray());
            MapCombo.SelectedItem = null;
            MapCombo.Text = "";
            MoveManager.Refresh();


            for (int X = 0; X < SelList.Length; X++)
                MapList.SetSelected(X, SelList[X]);

            if (MovementList.SelectedItem != null)
                panel1.Enabled = true;
            else
                panel1.Enabled = false;

            if (SpawnList.SelectedItem != null)
                panel2.Enabled = true;
            else
                panel2.Enabled = false;


        }
        public void RefreshMoveList()
        {
            int I = MovementList.SelectedIndex;
            MoveManager.Refresh();
            MovementList.SelectedIndex = I;
        }
        public void RefreshSpawnList()
        {
            int I = SpawnList.SelectedIndex;
            SpawnManager.Refresh();
            SpawnList.SelectedIndex = I;
        }
        public void SortList()
        {
            for (int I = Main.SData.MapList.Count - 1; I >= 0; I--)
                Main.SData.MapList[I].Index = I;
        }

        public void DataLoaded()
        {
            MapList.DataSource = Main.SData.MapList;
            MapManager = (CurrencyManager)BindingContext[Main.SData.MapList];
            MapCombo.Items.Clear();
            MapCombo.Items.AddRange(Main.SData.MapList.ToArray());

            MonsterCombo.Items.Clear();
            MonsterCombo.Items.AddRange(Main.SData.MonsterList.ToArray());

            if (MapList.SelectedItems.Count == 0)
                tabControl1.Hide();
            else
                tabControl1.Show();

            if (MapList.SelectedItems.Count == 1)
            {
                MovementList.DataSource = (MapList.SelectedItem as MapInfo).MoveList;
                MoveManager = (CurrencyManager)BindingContext[(MapList.SelectedItem as MapInfo).MoveList];

                SpawnList.DataSource = (MapList.SelectedItem as MapInfo).RespawnList;
                SpawnManager = (CurrencyManager)BindingContext[(MapList.SelectedItem as MapInfo).RespawnList];
            }

            if (MovementList.SelectedItem != null)
                panel1.Enabled = true;
            else
                panel1.Enabled = false;

            if (SpawnList.SelectedItem != null)
                panel2.Enabled = true;
            else
                panel2.Enabled = false;

        }

        private void NewMap_Click(object sender, System.EventArgs e)
        {
            Main.SData.MapList.Add(new MapInfo());
            SortList();
            RefreshLists();
        }
        private void DeleteMap_Click(object sender, System.EventArgs e)
        {
            if (MapList.SelectedItems.Count == 0) return;

            if (MessageBox.Show("Are you sure you want to delete the selected map(s)", "Delete Map(s)", MessageBoxButtons.YesNo) == DialogResult.No)
                return;

            foreach (MapInfo MI in MapList.SelectedItems)
            {
                Main.SData.MapList.Remove(MI);
                foreach (MapInfo M in Main.SData.MapList)
                    for (int I = M.MoveList.Count - 1; I >= 0; I--)
                        if (M.MoveList[I].DestMapInfo == MI)
                            M.MoveList.RemoveAt(I);
            }
            SortList();
            MapManager.Refresh();
            MapCombo.Items.Clear();
            MapCombo.Items.AddRange(Main.SData.MapList.ToArray());
            MapCombo.SelectedItem = null;
            MapCombo.Text = "";
            MoveManager.Refresh();

            if (MapList.SelectedItems.Count == 0)
                tabControl1.Hide();
            else
                tabControl1.Show();

            if (MapList.SelectedItems.Count == 1)
            {
                MovementList.DataSource = (MapList.SelectedItem as MapInfo).MoveList;
                MoveManager = (CurrencyManager)BindingContext[(MapList.SelectedItem as MapInfo).MoveList];

                SpawnList.DataSource = (MapList.SelectedItem as MapInfo).RespawnList;
                SpawnManager = (CurrencyManager)BindingContext[(MapList.SelectedItem as MapInfo).RespawnList];
            }
        }

        private void Mir3Map_CheckedChanged(object sender, System.EventArgs e)
        {
            if (Selecting) return;
            Selecting = true;

            Mir2Map.Checked = false;
            Mir3Map.Checked = true;
            foreach (MapInfo M in MapList.SelectedItems)
                M.MapType = MapType.Mir3;
            Selecting = false;
        }
        private void Mir2Map_CheckedChanged(object sender, System.EventArgs e)
        {
            if (Selecting) return;
            Selecting = true;

            Mir3Map.Checked = false;
            Mir2Map.Checked = true;
            foreach (MapInfo M in MapList.SelectedItems)
                M.MapType = MapType.Mir2;
            Selecting = false;
        }

        private void ClearInterface()
        {
            MapFileName.Text = "";
            MapName.Text = "";
            MapIndex.Text = "";

            Mir2Map.Checked = false;
            Mir3Map.Checked = false;

            MapFlagsList.ClearSelected();
        }

        private void ClearMoveInterface()
        {
            MapCombo.SelectedItem = null;
            MapCombo.Text = "";
        }
        private void ClearSpawnInterface()
        {
            MonsterCombo.SelectedItem = null;
            MonsterCombo.Text = "";

            X.Text = "";
            Y.Text = "";

            Amount.Text = "";
            Radius.Text = "";

            Delay.Text = "";
            Distortion.Text = "";

            MoveEnabled.Checked = false;
        }

        private void FillInterface()
        {
            MapFlags F = 0;
            bool Checked = false;

            foreach (MapInfo MI in MapList.SelectedItems)
            {
                if (MapIndex.Text == "") MapIndex.Text = MI.Index.ToString();
                if (MapIndex.Text != MI.Index.ToString()) MapIndex.Text = "#####";

                if (MapFileName.Text == "") MapFileName.Text = MI.FileName;
                if (MapFileName.Text != MI.FileName) MapFileName.Text = "#####";

                if (MapName.Text == "") MapName.Text = MI.MapDescription;
                if (MapName.Text != MI.MapDescription) MapName.Text = "#####";

                if (!Mir2Map.Checked && !Mir3Map.Checked)
                {
                    if (MI.MapType == MapType.Mir2)
                        Mir2Map.Checked = true;
                    else
                        Mir3Map.Checked = true;
                }

                if ((!Mir2Map.Checked && MI.MapType == MapType.Mir2) || (!Mir3Map.Checked && MI.MapType == MapType.Mir3))
                {
                    Mir2Map.CheckState = CheckState.Indeterminate;
                    Mir3Map.CheckState = CheckState.Indeterminate;
                }

                if (!Checked)
                {
                    F = MI.Flags;
                    for (int I = 0; I < Enum.GetValues(typeof(MapFlags)).Length; I++)
                        if ((F & (MapFlags)Enum.GetValues(typeof(MapFlags)).GetValue(I)) != 0)
                            MapFlagsList.SetItemChecked(I, true);
                        else MapFlagsList.SetItemChecked(I, false);
                }

                if (F != MI.Flags)
                {
                    for (int I = 0; I < MapFlagsList.Items.Count; I++)
                        MapFlagsList.SetItemCheckState(I, CheckState.Indeterminate);
                }

                Checked = true;
            }

        }
        public void FillMoveInterface()
        {
            if (MovementList.SelectedItem != null)
            {
                MovementInfo M = MovementList.SelectedItem as MovementInfo;

                FromX.Text = M.SourcePoint.X.ToString();
                FromY.Text = M.SourcePoint.Y.ToString();

                ToX.Text = M.DestinationPoint.X.ToString();
                ToY.Text = M.DestinationPoint.Y.ToString();

                MapCombo.SelectedItem = M.DestMapInfo;
            }
        }
        public void FillSpawnInterface()
        {
            if (SpawnList.SelectedItem != null)
            {
                RespawnInfo RI = SpawnList.SelectedItem as RespawnInfo;

                X.Text = RI.Location.X.ToString();
                Y.Text = RI.Location.Y.ToString();

                Amount.Text = RI.Amount.ToString();
                Radius.Text = RI.Spread.ToString();

                MonsterCombo.SelectedItem = RI.Monster;

                Delay.Text = RI.RespawnTime.TotalMinutes.ToString();
                Distortion.Text = RI.DistortionTime.ToString();

            }
        }
        private void MapList_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            Selecting = true;
            SpawnList.ClearSelected();
            MovementList.ClearSelected();
            ClearInterface();
            FillInterface();

            Selecting = false;

            if (MapList.SelectedItems.Count == 0)
                tabControl1.Hide();
            else
                tabControl1.Show();

            if (MapList.SelectedItems.Count == 1)
            {
                MovementList.DataSource = (MapList.SelectedItem as MapInfo).MoveList;
                MoveManager = (CurrencyManager)BindingContext[(MapList.SelectedItem as MapInfo).MoveList];
                Selecting = true;
                ClearMoveInterface();
                FillMoveInterface();
                Selecting = false;

                SpawnList.DataSource = (MapList.SelectedItem as MapInfo).RespawnList;
                SpawnManager = (CurrencyManager)BindingContext[(MapList.SelectedItem as MapInfo).RespawnList];

                Selecting = true;
                ClearSpawnInterface();
                FillSpawnInterface();
                Selecting = false;
            }
            if (MovementList.SelectedItem != null)
                panel1.Enabled = true;
            else
                panel1.Enabled = false;

            if (SpawnList.SelectedItem != null)
                panel2.Enabled = true;
            else
                panel2.Enabled = false;

        }

        private void MapFileName_TextChanged(object sender, System.EventArgs e)
        {
            if (MapIndex.Text == "#####") return;
            if (string.IsNullOrWhiteSpace(MapFileName.Text))
            {
                MapFileName.BackColor = Color.RosyBrown;
                MapFileName.ForeColor = Color.White;
            }

            MapFileName.BackColor = Color.White;
            MapFileName.ForeColor = Color.Black;

            if (Selecting) return;

            foreach (MapInfo MI in MapList.SelectedItems)
                MI.FileName = MapFileName.Text != null ? MapFileName.Text : "";
        }

        private void MapName_Leave(object sender, System.EventArgs e)
        {
            if (MapIndex.Text == "#####") return;
            if (Selecting) return;
            if (string.IsNullOrWhiteSpace(MapName.Text))
            {
                MapName.BackColor = Color.RosyBrown;
                MapName.ForeColor = Color.White;
            }
            MapName.BackColor = Color.White;
            MapName.ForeColor = Color.Black;

            foreach (MapInfo MI in MapList.SelectedItems)
                MI.MapDescription = MapName.Text != null ? MapName.Text : "";

            RefreshLists();
        }

        private void MapIndex_Leave(object sender, System.EventArgs e)
        {
            if (MapIndex.Text == "#####") return;
            if (Selecting) return;
            int I;
            if (int.TryParse(MapIndex.Text, out I))
            {
                MapIndex.BackColor = Color.White;
                MapIndex.ForeColor = Color.Black;


                foreach (MapInfo MI in MapList.SelectedItems)
                    MI.Index = I;

                RefreshLists();
            }
            else
            {
                MapIndex.BackColor = Color.RosyBrown;
                MapIndex.ForeColor = Color.White;
            }
        }

        private void MapFlagsList_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (Selecting) return;

            foreach (MapInfo MI in MapList.SelectedItems)
                if (e.NewValue == CheckState.Checked)
                {
                    if (!MI.Flags.HasFlag((MapFlags)Enum.GetValues(typeof(MapFlags)).GetValue(e.Index)))
                        MI.Flags += (int)Enum.GetValues(typeof(MapFlags)).GetValue(e.Index);
                }
                else if (MI.Flags.HasFlag((MapFlags)Enum.GetValues(typeof(MapFlags)).GetValue(e.Index)))
                    MI.Flags -= (int)Enum.GetValues(typeof(MapFlags)).GetValue(e.Index);
        }

        private void NewMovement_Click(object sender, EventArgs e)
        {
            (MapList.SelectedItem as MapInfo).MoveList.Add(new MovementInfo());
            MoveManager.Refresh();

            if (MovementList.SelectedItem != null)
                panel1.Enabled = true;
            else
                panel1.Enabled = false;

            if (Selecting)
            {
                ClearMoveInterface();
                FillMoveInterface();
            }
            else
            {
                Selecting = true;
                ClearMoveInterface();
                FillMoveInterface();
                Selecting = false;
            }
        }

        private void MovementList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (Selecting)
            {
                ClearMoveInterface();
                FillMoveInterface();
            }
            else
            {
                Selecting = true;
                ClearMoveInterface();
                FillMoveInterface();
                Selecting = false;
            }

            if (MovementList.SelectedItem != null)
                panel1.Enabled = true;
            else
                panel1.Enabled = false;

        }

        private void DeleteMovement_Click(object sender, EventArgs e)
        {
            (MapList.SelectedItem as MapInfo).MoveList.Remove(MovementList.SelectedItem as MovementInfo);
            MoveManager.Refresh();

            if (MovementList.SelectedItem != null)
                panel1.Enabled = true;
            else
                panel1.Enabled = false;

            if (Selecting)
            {
                ClearMoveInterface();
                FillMoveInterface();
            }
            else
            {
                Selecting = true;
                ClearMoveInterface();
                FillMoveInterface();
                Selecting = false;
            }
        }

        private void FromX_Leave(object sender, EventArgs e)
        {
            if (Selecting) return;
            short I;
            if (short.TryParse(FromX.Text, out I))
            {
                FromX.BackColor = Color.White;
                FromX.ForeColor = Color.Black;
                (MovementList.SelectedItem as MovementInfo).SourcePoint.X = I;
                RefreshMoveList();
            }
            else
            {
                FromX.BackColor = Color.RosyBrown;
                FromX.ForeColor = Color.White;
            }
        }

        private void FromY_Leave(object sender, EventArgs e)
        {
            if (Selecting) return;
            short I;
            if (short.TryParse(FromY.Text, out I))
            {
                FromY.BackColor = Color.White;
                FromY.ForeColor = Color.Black;
                (MovementList.SelectedItem as MovementInfo).SourcePoint.Y = I;
                RefreshMoveList();
            }
            else
            {
                FromY.BackColor = Color.RosyBrown;
                FromY.ForeColor = Color.White;
            }
        }

        private void ToY_Leave(object sender, EventArgs e)
        {
            if (Selecting) return;
            short I;
            if (short.TryParse(ToY.Text, out I))
            {
                ToY.BackColor = Color.White;
                ToY.ForeColor = Color.Black;
                (MovementList.SelectedItem as MovementInfo).DestinationPoint.Y = I;
                RefreshMoveList();
            }
            else
            {
                ToY.BackColor = Color.RosyBrown;
                ToY.ForeColor = Color.White;
            }
        }

        private void ToX_Leave(object sender, EventArgs e)
        {

            if (Selecting) return;
            short I;
            if (short.TryParse(ToX.Text, out I))
            {
                ToX.BackColor = Color.White;
                ToX.ForeColor = Color.Black;
                (MovementList.SelectedItem as MovementInfo).DestinationPoint.X = I;
                RefreshMoveList();
            }
            else
            {
                ToX.BackColor = Color.RosyBrown;
                ToX.ForeColor = Color.White;
            }
        }

        private void MoveEnabled_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void MapCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (Selecting) return;
            (MovementList.SelectedItem as MovementInfo).DestMapInfo = MapCombo.SelectedItem as MapInfo;
            RefreshMoveList();
        }

        private void SpawnList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (Selecting)
            {
                ClearSpawnInterface();
                FillSpawnInterface();
            }
            else
            {
                Selecting = true;
                ClearSpawnInterface();
                FillSpawnInterface();
                Selecting = false;
            }

            if (SpawnList.SelectedItem != null)
                panel2.Enabled = true;
            else
                panel2.Enabled = false;

        }

        private void NewSpawn_Click(object sender, EventArgs e)
        {
            (MapList.SelectedItem as MapInfo).RespawnList.Add(new RespawnInfo());
            SpawnManager.Refresh();

            if (SpawnList.SelectedItem != null)
                panel2.Enabled = true;
            else
                panel2.Enabled = false;

            if (Selecting)
            {
                ClearSpawnInterface();
                FillSpawnInterface();
            }
            else
            {
                Selecting = true;
                ClearSpawnInterface();
                FillSpawnInterface();
                Selecting = false;
            }

        }

        private void DeleteSpawn_Click(object sender, EventArgs e)
        {
            (MapList.SelectedItem as MapInfo).RespawnList.Remove(SpawnList.SelectedItem as RespawnInfo);
            SpawnManager.Refresh();

            if (SpawnList.SelectedItem != null)
                panel2.Enabled = true;
            else
                panel2.Enabled = false;

            if (Selecting)
            {
                ClearSpawnInterface();
                FillSpawnInterface();
            }
            else
            {
                Selecting = true;
                ClearSpawnInterface();
                FillSpawnInterface();
                Selecting = false;
            }

        }

        private void MonsterCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (Selecting) return;
            (SpawnList.SelectedItem as RespawnInfo).Monster = MonsterCombo.SelectedItem as MonsterInfo;
            RefreshSpawnList();
        }

        private void Amount_Leave(object sender, EventArgs e)
        {
            if (Selecting) return;
            ushort I;
            if (ushort.TryParse(Amount.Text, out I))
            {
                Amount.BackColor = Color.White;
                Amount.ForeColor = Color.Black;
                (SpawnList.SelectedItem as RespawnInfo).Amount = I;
                RefreshSpawnList();
            }
            else
            {
                Amount.BackColor = Color.RosyBrown;
                Amount.ForeColor = Color.White;
            }
        }

        private void Radius_Leave(object sender, EventArgs e)
        {
            if (Selecting) return;
            ushort I;
            if (ushort.TryParse(Radius.Text, out I))
            {
                Radius.BackColor = Color.White;
                Radius.ForeColor = Color.Black;
                (SpawnList.SelectedItem as RespawnInfo).Spread = I;
                RefreshSpawnList();
            }
            else
            {
                Radius.BackColor = Color.RosyBrown;
                Radius.ForeColor = Color.White;
            }
        }

        private void X_Leave(object sender, EventArgs e)
        {
            if (Selecting) return;
            ushort I;
            if (ushort.TryParse(X.Text, out I))
            {
                X.BackColor = Color.White;
                X.ForeColor = Color.Black;
                (SpawnList.SelectedItem as RespawnInfo).Location.X = I;
                RefreshSpawnList();
            }
            else
            {
                X.BackColor = Color.RosyBrown;
                X.ForeColor = Color.White;
            }
        }

        private void Y_Leave(object sender, EventArgs e)
        {
            if (Selecting) return;
            ushort I;
            if (ushort.TryParse(Y.Text, out I))
            {
                Y.BackColor = Color.White;
                Y.ForeColor = Color.Black;
                (SpawnList.SelectedItem as RespawnInfo).Location.Y = I;
                RefreshSpawnList();
            }
            else
            {
                Y.BackColor = Color.RosyBrown;
                Y.ForeColor = Color.White;
            }
        }

        private void Delay_Leave(object sender, EventArgs e)
        {

            if (Selecting) return;
            int I;
            if (int.TryParse(Delay.Text, out I))
            {
                Delay.BackColor = Color.White;
                Delay.ForeColor = Color.Black;
                (SpawnList.SelectedItem as RespawnInfo).RespawnTime = TimeSpan.FromMinutes(I);
                RefreshSpawnList();
            }
            else
            {
                Delay.BackColor = Color.RosyBrown;
                Delay.ForeColor = Color.White;
            }
        }

        private void Distortion_Leave(object sender, EventArgs e)
        {
            if (Selecting) return;
            int I;
            if (int.TryParse(Distortion.Text, out I))
            {
                Distortion.BackColor = Color.White;
                Distortion.ForeColor = Color.Black;
                (SpawnList.SelectedItem as RespawnInfo).DistortionTime = I;
                RefreshSpawnList();
            }
            else
            {
                Distortion.BackColor = Color.RosyBrown;
                Distortion.ForeColor = Color.White;
            }
        }

    }
}
